/**  ClientDispCfg - display di configurazione per il client
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.awt.Frame;
import java.awt.Label;
import java.awt.TextField;
import java.awt.Checkbox;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Font;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.net.InetAddress;


//-------------- I "LISTENER" DEI BOTTONI ---------------------------------------------------------

class ListenCliCfgAn implements ActionListener {
  // Astrazione di un listener che usa un'applicazione Client e
  // la finestra di configurazione (da distruggere)
  protected ClientApplet  CliApp;   // Riferimento al Client da usare
  protected ClientDispCfg FinCfg;   // Finestra coi dati

  public ListenCliCfgAn (ClientApplet CliApp, ClientDispCfg FinCfg) { 
    this.CliApp = CliApp;
    this.FinCfg = FinCfg;
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    // Non modifica i dati del Client
    FinCfg.distruggiDisplay ();
  } //actionPerformed

} //ListenCliCfgAn


class ListenCliCfgOK extends ListenCliCfgAn implements ActionListener {

  public ListenCliCfgOK (ClientApplet CliApp, ClientDispCfg FinCfg) { 
    super (CliApp, FinCfg);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    // MODIFICA I DATI DEL CLIENT:
    String StrInd;
    for (byte i=ClientApplet.MIN_NUM_SER; i<=ClientApplet.MAX_NUM_SER; i++)
      try {
        StrInd = FinCfg.AreaIndSer[i].getText();

        if (StrInd.equals("")) 
          CliApp.defIndirizzoServer (i, StrInd);
        else {
          String IndAttuale = CliApp.rendiIndirizzoServer(i);
          // Se non e' stato modificato, non faccio nulla; se no..
          if (! StrInd.equals(IndAttuale)) {
            // Prova a ricavarne l'indirizzo IP (se no, eccezione)
            InetAddress IPAddr = InetAddress.getByName (StrInd);

            CliApp.defIndirizzoServer (i, StrInd);
          }
        }
      } 
      catch (Exception E) {
        FinCfg.apriDialogoErrModale (Dialogo.BOTT_OK, 
                      (i+1)+" Indirizzo ERRATO;", E.getMessage());
      }

    // Costanti di ritrasmissione
    String InStr;
    InStr = FinCfg.AreaTimeOut.getText();
    try {
    	 ClientApplet.TIMEOUT_CLI = (Integer.valueOf(InStr).intValue()) * 1000;
      if (ClientApplet.TIMEOUT_CLI < 1000)
        ClientApplet.TIMEOUT_CLI=1000;
    } 
    catch (Exception E) { }

    InStr = FinCfg.AreaNumRit.getText();
    try {
    	 ClientApplet.MAX_TENTA_CLI = Integer.valueOf(InStr).intValue();
      if (ClientApplet.MAX_TENTA_CLI < 1)  
        ClientApplet.MAX_TENTA_CLI=1;
    } 
    catch (Exception E) { }

    // Finestra di Debug
    CliApp.apriDebug (FinCfg.CheckDebug.getState());

    // Finestra di copyright all'avvio
    CliApp.GigiRight = FinCfg.CheckGigi.getState();

    CliApp.salvaConfig ();        // Dopo le modifiche, salvo la configurazione!

    FinCfg.distruggiDisplay ();
  } //actionPerformed

} //ListenCliCfgOK


class ListenCliCercaSer extends ListenCliCfgAn implements ActionListener {

  public ListenCliCercaSer (ClientApplet CliApp, ClientDispCfg FinCfg) { 
    super (CliApp, FinCfg);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    // Prima ridefinisco il primo indirizzo del server 
    // (l'utente puo' averlo modificato e non ha ancora premuto OK)
    byte Num = ClientApplet.MIN_NUM_SER;
    String StrInd = FinCfg.AreaIndSer[Num].getText();
    CliApp.defIndirizzoServer (Num, StrInd);

    // E quindi chiedo al client di richiederli al server
    CliApp.eseguiDammiSpettacPostoServer (ClientApplet.DAMMI_SER); //cerca server

    // Quindi rileggo i nuovi indirizzi dei server e li metto nelle aree di testo
    for (byte i=ClientApplet.MIN_NUM_SER; i<=ClientApplet.MAX_NUM_SER; i++)
      try {
        FinCfg.AreaIndSer[i].setText (CliApp.rendiIndirizzoServer(i));
      } 
      catch (Exception E) { }

  } //actionPerformed

} //ListenCliCercaSer



//============== IL DISPLAY DI CONFIGURAZIONE =====================================================

class ClientDispCfg extends Display {
  /** Il display del programma (applet) client
   */

  // Costanti:
  protected static final Color COL_SFO_BOTCS = Color.green;
  protected static final Color COL_TXT_BOTCS = Color.black;

  // Uso il pattern Model-View-Control:
  // Il client usa un display che a sua volta usa il client
  protected ClientApplet MyCliAp;

  public TextField[] AreaIndSer  = new TextField [ClientApplet.MAX_NUM_SER+1];
  public TextField   AreaTimeOut;
  public TextField   AreaNumRit;
  public Checkbox    CheckDebug;
  public Checkbox    CheckGigi;


  public ClientDispCfg (ClientApplet MyClient) {
    this (MyClient, "");
  } //costruttore

  public ClientDispCfg (ClientApplet MyClient, String TitoloFin) {
    super (TitoloFin);
    MyCliAp = MyClient;   // SALVA RIFERIMENTO A APPLICAZIONE
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore

  public ClientDispCfg (ClientApplet MyClient, String TitoloFin, Frame FinPapa) {
    super (TitoloFin, FinPapa);
    MyCliAp = MyClient;   // SALVA RIFERIMENTO A APPLICAZIONE
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore

  public ClientDispCfg (ClientApplet MyClient, Frame FinGiaCreata) {
    super (FinGiaCreata);
    MyCliAp = MyClient;
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore


  protected void CostruisciDisplay () {
    super.CostruisciDisplay ();

    // Riempie le caselle dei dati
    for (byte i=ClientApplet.MIN_NUM_SER; i<=ClientApplet.MAX_NUM_SER; i++)
      try {
        AreaIndSer[i] = new TextField (15);
        AreaIndSer[i].setText (MyCliAp.rendiIndirizzoServer(i));
      } 
      catch (Exception E) { }

    AreaTimeOut = new TextField (3);
    AreaTimeOut.setText (Integer.toString(ClientApplet.TIMEOUT_CLI / 1000));
    AreaNumRit  = new TextField (3);
    AreaNumRit.setText (Integer.toString(ClientApplet.MAX_TENTA_CLI));

    CheckDebug = new Checkbox ();
    CheckDebug.setState (Debug.esisteFinestra());

    CheckGigi = new Checkbox ();
    CheckGigi.setState (MyCliAp.GigiRight);

    GridBagLayout      GBL = new GridBagLayout (); 
    GridBagConstraints GBC = new GridBagConstraints ();
    setLayout(GBL);

    Label L0  = new Label ("CONFIGURAZIONE");
    L0.setFont(FONT_TEXT0);
    
    GBC.anchor = GridBagConstraints.CENTER;
    GBC.gridx = 1;  GBC.gridy = 1;
    GBL.setConstraints (L0, GBC);
    add (L0);

    Label LI  = new Label ("Indirizzi Possibili Server:");
    GBC.anchor = GridBagConstraints.EAST;
    GBC.gridx = 1;  GBC.gridy = 2;
    GBL.setConstraints (LI, GBC);
    add (LI);

    Label LRT = new Label ("Time-Out per Ritrasmissione a Server (in sec):");
    GBC.gridx = 1;  GBC.gridy = 1+2+ClientApplet.MAX_NUM_SER;
    GBL.setConstraints (LRT, GBC);
    add (LRT);

    Label LRN = new Label ("Numero Massimo di Ritrasmissioni:");
    GBC.gridx = 1;  GBC.gridy = 2+2+ClientApplet.MAX_NUM_SER;
    GBL.setConstraints (LRN, GBC);
    add (LRN);

    Label LD  = new Label ("Finestra di Debug:");
    GBC.gridx = 1;  GBC.gridy = 3+2+ClientApplet.MAX_NUM_SER;
    GBL.setConstraints (LD, GBC);
    add (LD);

    Label LGG  = new Label ("Finestra Info all'Avvio:");
    GBC.gridx = 1;  GBC.gridy = 4+2+ClientApplet.MAX_NUM_SER;
    GBL.setConstraints (LGG, GBC);
    add (LGG);

    GBC.anchor = GridBagConstraints.WEST;
    for (byte i=ClientApplet.MIN_NUM_SER; i<=ClientApplet.MAX_NUM_SER; i++) {
      GBC.gridx = 2;  GBC.gridy = 2+i;
      GBL.setConstraints (AreaIndSer[i], GBC);
      add(AreaIndSer[i]);
    }

    Bottone.defColoriFont (COL_SFO_BOTCS, COL_TXT_BOTCS, FONT_BOTT);
    Bottone BottCerSer  = new Bottone ("RICHIEDI ALTRI");
    BottCerSer.addActionListener (new ListenCliCercaSer(MyCliAp, this));
    GBC.gridx = 3;  GBC.gridy = 2;
    GBL.setConstraints (BottCerSer, GBC);
    add (BottCerSer);

    GBC.gridx = 2;  GBC.gridy = 1+2+ClientApplet.MAX_NUM_SER;
    GBL.setConstraints (AreaTimeOut, GBC);
    add(AreaTimeOut);

    GBC.gridx = 2;  GBC.gridy = 2+2+ClientApplet.MAX_NUM_SER;
    GBL.setConstraints (AreaNumRit, GBC);
    add(AreaNumRit);

    GBC.gridx = 2;  GBC.gridy = 3+2+ClientApplet.MAX_NUM_SER;
    GBL.setConstraints (CheckDebug, GBC);
    add (CheckDebug);

    GBC.gridx = 2;  GBC.gridy = 4+2+ClientApplet.MAX_NUM_SER;
    GBL.setConstraints (CheckGigi, GBC);
    add (CheckGigi);

    Bottone.defColoriFont (COL_SFO_BOTOK, COL_TXT_BOTOK, FONT_BOTT);
    Bottone BottCfgOK  = new Bottone ("AGGIORNA");
    Bottone.defColori (COL_SFO_BOTAN, COL_TXT_BOTAN);
    Bottone BottCfgAn  = new Bottone ("ANNULLA");

    GBC.anchor = GridBagConstraints.WEST;
    BottCfgOK.addActionListener (new ListenCliCfgOK(MyCliAp, this));
    GBC.gridx = 1;  GBC.gridy = 7+2+ClientApplet.MAX_NUM_SER;
    GBL.setConstraints (BottCfgOK, GBC);
    add (BottCfgOK);

    GBC.anchor = GridBagConstraints.EAST;
    BottCfgAn.addActionListener (new ListenCliCfgAn(MyCliAp, this));
    GBC.gridx = 3;  GBC.gridy = 7+2+ClientApplet.MAX_NUM_SER;
    GBL.setConstraints (BottCfgAn, GBC);
    add (BottCfgAn);
  } //CostruisciDisplay

} //ClientDispCfg