/**  ClientDispLista - display Client per mostrare e scegliere da una lista
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.awt.Frame;
import java.awt.Panel;
import java.awt.Label;
import java.awt.List;
import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.util.Enumeration;
import java.util.Vector;


//-------------- I "LISTENER" DEI BOTTONI ---------------------------------------------------------

class ListenCliLstAn implements ActionListener {
  // Astrazione di un listener che usa un'applicazione Client e
  // la finestra di configurazione (da distruggere)
  protected ClientApplet    CliApp;   // Riferimento al Client da usare
  protected ClientDispLista FinCfg;   // Finestra coi dati

  public ListenCliLstAn (ClientApplet CliApp, ClientDispLista FinCfg) { 
    this.CliApp = CliApp;
    this.FinCfg = FinCfg;
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    // Non modifica i dati del Client
    FinCfg.distruggiDisplay ();
  } //actionPerformed

} //ListenCliLstAn


class ListenCliLstOK extends ListenCliLstAn implements ActionListener {

  public ListenCliLstOK (ClientApplet CliApp, ClientDispLista FinCfg) { 
    super (CliApp, FinCfg);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    // MODIFICA I DATI DEL CLIENT:
    // Preleva il N. di voce scelto nella lista su AWT
    int N = FinCfg.AwtList.getSelectedIndex();
    if (N >= 0) {
      // Preleva l'oggetto dalla lista (come Vector)
      Object OO = FinCfg.VectDato.elementAt(N);

      // E lo inserisce nel display del cliente
      if (OO instanceof Spettacolo) 
        CliApp.rendiDisplay().riempiSpettacolo ((Spettacolo) OO);
      if (OO instanceof Posto)
        CliApp.rendiDisplay().riempiPosto ((Posto) OO);

      FinCfg.distruggiDisplay ();
    }
  } //actionPerformed

} //ListenCliLstOK


//============== IL DISPLAY VISUALIZZAIZONE LISTA =================================================

class ClientDispLista extends Display {
  /** Il display del programma (applet) client
   */

  // Costanti pubbliche:
  public    static final byte  USO_ELENCO_SC   = 0;
  public    static final byte  USO_ELENCO_PL   = 1;
  public    static final byte  USO_ELENCO_POME = 2;

  // Costanti:
  protected static final Color COL_SFO_LST   = Color.black;
  protected static final Color COL_TXT_LST   = Color.green;
  protected static final Color COL_SFO_BOTOK = Color.green;
  protected static final Color COL_TXT_BOTOK = Color.black;
  protected static final Color COL_SFO_BOTAN = Color.red;
  protected static final Color COL_TXT_BOTAN = Color.white;
  
  // Uso il pattern Model-View-Control:
  // Il client usa un display che a sua volta usa il client
  protected ClientApplet MyCliAp;

  public List     AwtList;
  public Vector   VectDato;    // pubblici: acceduti dal listener "ok"
  public byte     UsoChe;      // idem, per sapere cosa fare..


  public ClientDispLista (ClientApplet MyClient, Vector Lista, byte USO_CHE) {
    this (MyClient, Lista, USO_CHE, "", null);
  } //costruttore

  public ClientDispLista (ClientApplet MyClient, Vector Lista, byte USO_CHE, 
                          String TitoloFin) {
    this (MyClient, Lista, USO_CHE, TitoloFin, null);
  } //costruttore

  public ClientDispLista (ClientApplet MyClient, Vector Lista, byte USO_CHE, 
                          String TitoloFin, Frame FinPapa) {
    super (TitoloFin, FinPapa);
    MyCliAp = MyClient;   // SALVA RIFERIMENTO A APPLICAZIONE
    VectDato= Lista;
    UsoChe  = USO_CHE;
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore

  public ClientDispLista (ClientApplet MyClient, Vector Lista, byte USO_CHE,
                          Frame FinGiaCreata) {
    super (FinGiaCreata);
    MyCliAp = MyClient;
    VectDato= Lista;
    UsoChe  = USO_CHE;
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore


  protected void CostruisciDisplay () {
    super.CostruisciDisplay ();

    // Riempie le caselle dei dati
    AwtList = new List (20);
    AwtList.setMultipleMode (false);
    AwtList.setBackground (COL_SFO_LST);
    AwtList.setForeground (COL_TXT_LST);

    Enumeration Enum = VectDato.elements();
    if (Enum.hasMoreElements())
      while (Enum.hasMoreElements()) {             // Inserisce elementi in lista
        AwtList.add ((Enum.nextElement()).toString());
      } //while
    else {
      String Frase;
      switch (UsoChe) {
        case USO_ELENCO_SC  : { Frase = "<nessuno spettacolo in programma>";       break; }
        case USO_ELENCO_PL  : { Frase = "<tutto esaurito: nessuno posto libero>";  break; }
        case USO_ELENCO_POME: { Frase = "<nessuno posto prenotato>";               break; }
        default             :   Frase = "<lista vuota>";
      }
      AwtList.add (Frase);
      AwtList.setEnabled(false);  // disabilita la lista!
    }

    setLayout (new BorderLayout());

    String CheCosa;
    switch (UsoChe) {
      case USO_ELENCO_SC  : { CheCosa = "Ecco l'Elenco degli Spettacoli in Programma"; break; }
      case USO_ELENCO_PL  : { CheCosa = "Ecco la Lista di Tutti i Posti Liberi:";      break; }
      case USO_ELENCO_POME: { CheCosa = "Ecco la Lista dei Tuoi Posti Prenotati:";     break; }
      default             :   CheCosa = "scegli:";
    }

    Label L1  = new Label (CheCosa);
    L1.setFont(FONT_TEXT0);
    add (L1, "North");

    add (AwtList, "Center");

    Panel PamPam = new Panel ();

    Bottone.defFont (FONT_BOTT);        // Definizione per il tasto "verde"
    String TxtBot;
    switch (UsoChe) {
      case USO_ELENCO_SC  : { TxtBot = "USA LO SPETTACOLO SELEZIONATO"; break; }
      case USO_ELENCO_PL  :  // uguale a sotto
      case USO_ELENCO_POME: { TxtBot = "USA IL POSTO SELEZIONATO";      break; }
      default             : { TxtBot = null; } // Tutti gli altri casi (niente bottone)
    }
    if (TxtBot != null) {
      Bottone.defColori (COL_SFO_BOTOK, COL_TXT_BOTOK);
      Bottone BottCfgOK  = new Bottone (TxtBot);
      BottCfgOK.addActionListener (new ListenCliLstOK(MyCliAp, this));
      PamPam.add (BottCfgOK);
    }

    Bottone.defColori (COL_SFO_BOTAN, COL_TXT_BOTAN);
    Bottone BottCfgAn  = new Bottone ("ANNULLA");
    BottCfgAn.addActionListener (new ListenCliLstAn(MyCliAp, this));
    PamPam.add (BottCfgAn);

    add (PamPam, "South");

    riposiziona (120, 10);

  } //CostruisciDisplay

} //ClientDispLista