/**  ClientDisplay - display per il client
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.awt.Frame;
import java.awt.Panel;
import java.awt.Font;
import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;


//-------------- I "LISTENER" DEI BOTTONI ---------------------------------------------------------

abstract class ListenCliGenerico {
  // Astrazione di un listener che usa una ClientApplet
  protected ClientApplet CliApp;   // Riferimento alla ClientApplet da usare

  public ListenCliGenerico (ClientApplet CliApp) { 
    this.CliApp = CliApp;
  } //costruttore

} //ListenCliGenerico


class ListenCliPrenota extends ListenCliGenerico implements ActionListener {

  public ListenCliPrenota (ClientApplet CliApp) { 
    super (CliApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    CliApp.eseguiPrenotaCancella (true); // Prenota
  } //actionPerformed

} //ListenCliPrenota


class ListenCliCancella extends ListenCliGenerico implements ActionListener {

  public ListenCliCancella (ClientApplet CliApp) { 
    super (CliApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    CliApp.eseguiPrenotaCancella (false); // Cancella
  } //actionPerformed

} //ListenCliCancella


class ListenCliTermina extends ListenCliGenerico implements ActionListener {

  public ListenCliTermina (ClientApplet CliApp) { 
    super (CliApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    CliApp.eseguiTermina();
  } //actionPerformed

} //ListenCliTermina


class ListenCliConfig extends ListenCliGenerico implements ActionListener {

  public ListenCliConfig (ClientApplet CliApp) { 
    super (CliApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    CliApp.eseguiConfig();
  } //actionPerformed

} //ListenCliConfig


class ListenCliDammiQualcosa extends ListenCliGenerico implements ActionListener {

  protected byte DammiCheCosa;

  public ListenCliDammiQualcosa (ClientApplet CliApp, byte DammiCheCosa) { 
    super (CliApp);
    this.DammiCheCosa = DammiCheCosa;
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    CliApp.eseguiDammiSpettacPostoServer (DammiCheCosa);
  } //actionPerformed

} //ListenCliDammiQualcosa


class ListenCliPulisci extends ListenCliGenerico implements ActionListener {

  public ListenCliPulisci (ClientApplet CliApp) { 
    super (CliApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    CliApp.rendiDisplay().cancella();
  } //actionPerformed

} //ListenCliPulisci


class ListenCliInfo extends ListenCliGenerico implements ActionListener {

  public ListenCliInfo (ClientApplet CliApp) { 
    super (CliApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    CliApp.eseguiInfo();
  } //actionPerformed

} //ListenCliInfo


//============== IL DISPLAY DELL'APPLET "ClientApplet" ============================================

class ClientDisplay extends Display {
  /** Il display del programma (applet) ClientApplet
   */

  // Costanti:
  protected static final Color COL_SFO_BOT1 = Color.green;
  protected static final Color COL_TXT_BOT1 = Color.black;
  protected static final Color COL_SFO_BOTC = Color.yellow;
  protected static final Color COL_TXT_BOTC = Color.blue;
  protected static final Color COL_SFO_BOTE = Color.red;
  protected static final Color COL_TXT_BOTE = Color.cyan;
  protected static final Color COL_SFO_BOTX = Color.lightGray;
  protected static final Color COL_TXT_BOTX = Color.black;
  protected static final Color COL_SFO_BOTL = Color.cyan;
  protected static final Color COL_TXT_BOTL = Color.black;
  protected static final Color COL_SFO_BOTI = Color.black;
  protected static final Color COL_TXT_BOTI = Color.green;

  protected static Font FONT_BOTTP = new Font ("DialogInput", Font.BOLD,  18);

  // Uso il pattern Model-View-Control:
  // Il client usa un display che a sua volta usa il client
  protected ClientApplet MyCliAp;

  protected SpettacoloZona SpecZona;   // Zona per lo spettacolo
  protected PostoZona      PostZona;   // Zona per il posto
  protected SpettatoreZona SperZona;   // Zona per lo spettatore


  public ClientDisplay (ClientApplet MyClient) {
    this (MyClient, "");
  } //costruttore

  public ClientDisplay (ClientApplet MyClient, String TitoloFin) {
    super (TitoloFin);
    MyCliAp = MyClient;   // SALVA IL RIFERIMENTO AL SUO APPLET
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore

  public ClientDisplay (ClientApplet MyClient, Frame FinGiaCreata) {
    super (FinGiaCreata);
    MyCliAp = MyClient;
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore


  protected void CostruisciDisplay () {
    super.CostruisciDisplay ();

    GridBagLayout      GBL = new GridBagLayout (); 
    GridBagConstraints GBC = new GridBagConstraints ();
    setLayout (GBL);

    Bottone.defFont (FONT_BOTT);

    GBC.anchor = GridBagConstraints.WEST;
    SperZona = new SpettatoreZona ();          // Zona Spettatore
    GBC.gridx = 1;  // colonna
    GBC.gridy = 1;  // riga  (1,1)=alto-sin
    GBL.setConstraints (SperZona, GBC);
    add (SperZona);

    SpecZona = new SpettacoloZona ();          // Zona Spettacolo
    GBC.gridx = 1;  GBC.gridy = 2;  // riga succ.
    GBL.setConstraints (SpecZona, GBC);
    add (SpecZona);

    PostZona = new PostoZona ();               // Zona Posto
    GBC.gridx = 1;  GBC.gridy = 3;
    GBL.setConstraints (PostZona, GBC);
    add (PostZona);

    GridBagLayout      GBL2 = new GridBagLayout (); 
    GridBagConstraints GBC2 = new GridBagConstraints ();
    Panel PanBottEx = new Panel (GBL2);
    GBC2.anchor = GridBagConstraints.CENTER;

    // Aggiunge i bottoni, creandone i rispettivi "listener"
    Bottone BottPrenota  = new Bottone ("FAI PRENOTAZIONE", COL_SFO_BOT1, COL_TXT_BOT1, FONT_BOTTP);
    BottPrenota.addActionListener (new ListenCliPrenota(MyCliAp));
    GBC2.gridx = 1;  GBC2.gridy = 1;
    GBL2.setConstraints (BottPrenota, GBC2);
    PanBottEx.add (BottPrenota);
    Bottone BottCancella = new Bottone ("CANCELLA PRENOTAZIONE", COL_SFO_BOT1, COL_TXT_BOT1, FONT_BOTTP);
    BottCancella.addActionListener (new ListenCliCancella(MyCliAp));
    GBC2.gridx = 2;  GBC2.gridy = 1;
    GBL2.setConstraints (BottCancella, GBC2);
    PanBottEx.add (BottCancella);
    Bottone BottConfig   = new Bottone ("CONFIGURA", COL_SFO_BOTC, COL_TXT_BOTC);
    BottConfig.addActionListener (new ListenCliConfig(MyCliAp));
    GBC2.gridx = 1;  GBC2.gridy = 2;
    GBL2.setConstraints (BottConfig, GBC2);
    PanBottEx.add (BottConfig);
    Bottone BottTermina  = new Bottone ("ESCI", COL_SFO_BOTE, COL_TXT_BOTE);
    BottTermina.addActionListener (new ListenCliTermina (MyCliAp));
    GBC2.gridx = 2;  GBC2.gridy = 2;
    GBL2.setConstraints (BottTermina, GBC2);
    PanBottEx.add(BottTermina);

    GBC.anchor = GridBagConstraints.EAST;
    GBC.gridx = 1;  GBC.gridy = 4;        // Pannello con Bottoni,in basso 
    GBL.setConstraints (PanBottEx, GBC);
    add (PanBottEx);

    GBC.anchor = GridBagConstraints.WEST;
    Bottone BottPulisci  = new Bottone ("PULISCI", COL_SFO_BOTX, COL_TXT_BOTX);
    BottPulisci.addActionListener (new ListenCliPulisci (MyCliAp));
    GBC.gridx = 4;  GBC.gridy = 1;  // a dx di spettatore
    GBL.setConstraints (BottPulisci, GBC);
    add (BottPulisci);

    Bottone BottCercaSp  = new Bottone ("CERCA SPETTACOLO", COL_SFO_BOTL, COL_TXT_BOTL);
    GBC.gridx = 4;  GBC.gridy = 2;  // a dx di spettacolo
    GBL.setConstraints (BottCercaSp, GBC);
    BottCercaSp.addActionListener (new ListenCliDammiQualcosa (MyCliAp, ClientApplet.DAMMI_SC));
    add (BottCercaSp);

    Bottone BottCercaPo  = new Bottone ("CERCA POSTO IN SPETTACOLO", COL_SFO_BOTL, COL_TXT_BOTL);
    GBC.gridx = 4;  GBC.gridy = 3;  // a dx di posto
    GBL.setConstraints (BottCercaPo, GBC);
    BottCercaPo.addActionListener (new ListenCliDammiQualcosa (MyCliAp, ClientApplet.DAMMI_PL));
    add (BottCercaPo);

    GBC.anchor = GridBagConstraints.SOUTHWEST;
    Bottone BottCercaPOMe  = new Bottone ("MOSTRA MIEI POSTI", COL_SFO_BOTL, COL_TXT_BOTL);
    GBC.gridx = 4;  GBC.gridy = 3;  // a dx di posto
    GBL.setConstraints (BottCercaPOMe, GBC);
    BottCercaPOMe.addActionListener (new ListenCliDammiQualcosa (MyCliAp, ClientApplet.DAMMI_POME));
    add (BottCercaPOMe);

    GBC.anchor = GridBagConstraints.SOUTHEAST;
    Bottone BottInfo     = new Bottone ("INFO", COL_SFO_BOTI, COL_TXT_BOTI);
    GBC.gridx = 4;  GBC.gridy = 4;  // a dx di posto e in giu'
    GBL.setConstraints (BottInfo, GBC);
    BottInfo.addActionListener (new ListenCliInfo (MyCliAp));
    add (BottInfo);

    Frame MyFrame = (Frame) rendiFinContenente();
    MyFrame.setResizable(false);
    // MyFrame.toFront();

  } //CostruisciDisplay


  public Prenotazione rendiPrenotazione () throws Exception {
    return new Prenotazione ((Spettacolo) SpecZona.rendiDati(),
                             (Posto)      PostZona.rendiDati(),
                             (Spettatore) SperZona.rendiDati() );
  } //rendiPrenotazione


  public void riempiPrenotazione (Prenotazione P) {
     riempiSpettacolo ((Spettacolo) P.rendiSpettacolo());
     riempiPosto      ((Posto)      P.rendiPosto());
     riempiSpettatore ((Spettatore) P.rendiSpettatore());
  } //riempiPrenotazione

  public void riempiSpettacolo (Spettacolo S) {
     SpecZona.riempiDati(S);
  } //riempiSpettacolo

  public void riempiPosto (Posto P) {
     PostZona.riempiDati(P);
  } //riempiPosto

  public void riempiSpettatore (Spettatore S) {
     SperZona.riempiDati(S);
  } //riempiSpettatore


  public Spettacolo rendiSpettacolo () throws Exception {
     return (Spettacolo) SpecZona.rendiDati();
  } //rendiSpettacolo

  public Posto rendiPosto () throws Exception {
     return (Posto) PostZona.rendiDati();
  } //rendiPosto

  public Spettatore rendiSpettatore () throws Exception {
     return (Spettatore) SperZona.rendiDati();
  } //rendiSpettatore


  public void cancella () {
    SpecZona.cancella();
    PostZona.cancella();
    SperZona.cancella();
  } //rendiPrenotazione

  public void windowClosing  (WindowEvent evt) {    
    MyCliAp.eseguiTermina();
  } //windowClosing

} //ClientDisplay