/**  DataBase
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.lang.Exception;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.io.Serializable;



class DBEntry implements Serializable {
  /** E' cio' che viene memorizzato nel DataBase per ogni voce "Spettalcolo-Posto".
    */

  protected Spettatore Spettat;      // Lo spettatore registrato
  protected Priorita   Priorit;      // Priorit della macchina che l'ha inserito


  public DBEntry (Spettatore Spettat, Priorita Priorit) {
    this.Spettat = Spettat;
    this.Priorit = Priorit;
  } //costruttore


  public Spettatore cheSpettatore () {
    return Spettat;
  } //cheSpettatore

  public Priorita chePriorita () {
    return Priorit;
  } //chePriorita


  public String toString() {
    return new String("<St="+Spettat+",P="+Priorit.toString()+">");
  } //toString

} //DBEntry



public class DataBase implements Serializable {
  /** E' il data-base vero e proprio.
   *  In esso si memorizzano tutte le prenotazioni fatte da qualunque utente su 
   *  qualunque server!
   *  L'accesso e' in mutua esclusione (uso di metodi "synchronized")
   *  Deve essere inviato agli altri server (implemento "Serializable")
   */

  // Variabile di classe per memorizzare la tabella principale
  protected Hashtable TabellaST;

  protected transient Priorita DefaultPriority;
                               // Priorita' di default associata alle entry inserite
                               // NB: "transient" = NON VIENE SERIALIZZATA!

  // Messaggi di errore (lanciati con le eccezioni)
  protected static final String STRERR_GIASPET = "Tentata Aggiunta di uno Spettacolo gia' Esistente";
  protected static final String STRERR_NOSPETT = "Lo Spettacolo non e' Presente nel DataBase";
  protected static final String STRERR_OCCUPAT = "Tentata Prenotazione di un Posto gia' Prenotato";
  protected static final String STRERR_LIBERO  = "Tentata Cancellazione di un Posto Non Occupato";
  protected static final String STRERR_NONSUO  = "Tentata Cancellazione di un Posto Occupato da un Altro";

  
  public DataBase (Priorita DefaultPriority) {
    TabellaST = new Hashtable ();
    this.DefaultPriority = DefaultPriority;
  } //costruttore


  // OPERAZIONI GENERALI (NON VISIBILI ALL'ESTERNO): -----------------------------------------------

  protected Hashtable controllaERendiTeatro (Spettacolo Sc) throws Exception {
    // INTERNA (chiamata da metodi "synchronized"!
    // Rende il Teatro (hastable con posto-spettatore) associato allo spettacolo Sc
    Hashtable Teatro = (Hashtable) TabellaST.get (Sc);
    if (Teatro != null)                    // Preleva il Teatro associato allo spettacolo
      return Teatro;                       // e, se esiste, lo rende
    else                                   // altrimenti, segnala l'errore sullo spettacolo
      throw new Exception (STRERR_NOSPETT);
  } //RendiTeatro


  // OPERAZIONI SUGLI SPETTACOLI (ESISTE/INSERIMENTO/CANCELLAZIONE): -------------------------------

  public synchronized boolean esisteSpettacolo (Spettacolo Sc) {
    // Controlla se esiste lo spettacolo dato; se no rende false.
    return (TabellaST.get(Sc) != null);   // Guarda se lo spettacolo esiste gia'
  } //esisteSpettacolo


  public synchronized void aggiungiSpettacolo (Spettacolo Sc) throws Exception {
    // Aggiunge un nuovo spettacolo con tutti i posti vuoti.
    if (TabellaST.get(Sc) != null)          // Guarda se lo spettacolo esiste gia'
      throw new Exception (STRERR_GIASPET); // se si', genera eccezione
    else
      // Se no, crea un teatro vuoto e lo inserisce sotto la voce "St".
      TabellaST.put (Sc, new Hashtable());
  } //aggiungiSpettacolo


  public synchronized void aggiungiSpettacoloSeNonCe (Spettacolo Sc) {
    // Aggiunge un nuovo spettacolo con tutti i posti vuoti,
    // ma NON rende errore se c'e' gia'.
    if (TabellaST.get(Sc) == null)          // Guarda se lo spettacolo esiste gia'
      // Se no, crea un teatro vuoto e lo inserisce sotto la voce "St".
      TabellaST.put (Sc, new Hashtable());
  } //aggiungiSpettacoloSeNonCe


  public synchronized void togliSpettacolo (Spettacolo Sc) throws Exception {
    // Toglie uno spettacolo esistente (e tutti i posti occupati).
    if (TabellaST.get(Sc) == null)          // Guarda se lo spettacolo esiste gia'
      throw new Exception (STRERR_NOSPETT); // se no, genera eccezione
    else
      // Se no, toglie il teatro dello spettacolo "Sc"
      TabellaST.remove (Sc);
  } //togliSpettacolo


  public synchronized void togliSpettacoloSeCe (Spettacolo Sc) {
    // Toglie uno spettacolo esistente (e tutti i posti occupati), 
    // ma NON rende errore se non c'e'.
    if (TabellaST.get(Sc) != null)          // Guarda se lo spettacolo esiste gia'
      // Se no, toglie il teatro dello spettacolo "Sc"
      TabellaST.remove (Sc);
  } //togliSpettacoloSeCe


  // OPERAZIONI CLASSICHE DI INSERISCI/CANCELLA, MA NON SONO USATE! --------------------------------

  public synchronized void inserisci (Prenotazione P) throws Exception {
    inserisci (P.rendiSpettacolo(), P.rendiPosto(), P.rendiSpettatore());
  } //inserisci

  public synchronized void inserisci (Spettacolo Sc,  Posto P,  Spettatore St) throws Exception {
    // Inserimento di una terna (Spettacolo, Posto, Spettatore) in un posto vuoto;
    // Se il posto non e' vuoto, viene reso errore
    Hashtable  Teatro = controllaERendiTeatro (Sc);  // Preleva il Teatro assoc. a spettacolo 
    DBEntry ChiCe = (DBEntry) Teatro.get (P);        // Guarda se e' gia' prenotato (null=no)
    if (ChiCe == null) {                             // (se lo e', non inserisce il nuovo!)
      DBEntry EntryNew = new DBEntry (St, DefaultPriority);
      Teatro.put (P, EntryNew);                      // inserisce lo spettat. nel teatro, nel posto P
    }
    else 
      throw new Exception (STRERR_OCCUPAT);
  } //inserisci


  public synchronized void cancella (Prenotazione P) throws Exception {
    cancella (P.rendiSpettacolo(), P.rendiPosto(), P.rendiSpettatore());
  } //cancella

  public synchronized void cancella (Spettacolo Sc, Posto P, Spettatore St) throws Exception {
    // Cancellazione di una terna completa (Spettacolo, Posto, Spettatore)
    // Se si trova spettacolo e posto, ma lo spettatore non e' St, si genera un'eccezione
    Hashtable  Teatro = controllaERendiTeatro (Sc);  // Preleva il Teatro assoc. a spettacolo 
    DBEntry ChiCe  = (DBEntry) Teatro.get (P);       // Guarda chi e' lo spettatore 
    if (ChiCe == null)
      throw new Exception (STRERR_LIBERO);    // se posto libero, errore
    else
      if (ChiCe.cheSpettatore().equals(St))
        Teatro.remove (P);                    // Se lo spettatore e' lo stesso, lo toglie
      else
        throw new Exception (STRERR_NONSUO);  // se no, errore!
  } //cancella



  // OPERAZIONI SULLE PRENOTAZIONI (INSERIMENTO/CANCELLAZIONE/LIBERO/CHI_CE)------------------------

  public void defDefaultPriority (Priorita DefaultPriority) {
    this.DefaultPriority = DefaultPriority;
  } //defDefaultPriority


  public synchronized void inserisciSeNonCe (Prenotazione P) throws Exception {
    inserisciSeNonCe (P.rendiSpettacolo(), P.rendiPosto(), P.rendiSpettatore());
  } //inserisciSeNonCe

  public synchronized void inserisciSeNonCe (Spettacolo Sc, Posto P, Spettatore St) throws Exception {
    // Inserimento di una terna (Spettacolo, Posto, Spettatore) "per" un certo spettatore;
    // Se il posto e' libero, lo occupa;
    // Se il posto e' occupato ma dallo stesso spettatore St, non lo reinserisce ma ritorna "ok";
    // Se il posto e' occupato da qualcun altro, non puo' occuparlo e quindi genera un'eccezione
    // All'"entry" inserita e' associata la priorita' attuale ("DefaultPriority").
    Hashtable  Teatro = controllaERendiTeatro (Sc);  // Preleva il Teatro assoc. a spettacolo 
    DBEntry ChiCe = (DBEntry) Teatro.get (P);        // Guarda se e' gia' prenotato (null=libero)
    if (ChiCe == null) {                             // Se e' "null" il posto e' libero, percio'
      DBEntry EntryNew = new DBEntry (St, DefaultPriority);
      Teatro.put (P, EntryNew);                      // inserisce lo spettat. nel teatro, nel posto P
    }                                                // (assegnandogli la priorita' di default!)
    else 
      if (ChiCe.cheSpettatore().equals(St))          // Se c'e' gia' lo stesso spettatore,
        ;                                            // non fa nulla
      else
        throw new Exception (STRERR_OCCUPAT);        // posto occupato -> genera un'eccezione!
  } //inserisciSeNonCe


  public synchronized void inserisciSeNonCe (Prenotazione P,  Priorita Priority) throws Exception {
    inserisciSeNonCe (P.rendiSpettacolo(), P.rendiPosto(), P.rendiSpettatore(), Priority);
  } //inserisciSeNonCe

  public synchronized void inserisciSeNonCe (Spettacolo Sc, Posto P, Spettatore St,
                                             Priorita Priority) throws Exception {
    // Inserimento di una terna (Spettacolo, Posto, Spettatore) "per" un certo spettatore;
    // MA USANDO LA PRIORITA'!
    // Se il posto e' libero, lo occupa;
    // Se il posto e' occupato ma dallo stesso spettatore St, non lo reinserisce ma ritorna "ok";
    // Se il posto e' occupato da qualcun altro, SI CONTROLLA LA PRIORITA':
    //  - se il nuovo arrivato ha priorita' superiore, viene inserito e si rende "ok";
    //  - se invece ha priorita' inferiore, non viene inserito e si genera un'eccezione;
    Hashtable  Teatro = controllaERendiTeatro (Sc);  // Preleva il Teatro assoc. a spettacolo 
    DBEntry ChiCe = (DBEntry) Teatro.get (P);        // Guarda se e' gia' prenotato (null=libero)
    if (ChiCe == null) {                             // Se e' "null" il posto e' libero, percio'
      DBEntry EntryNew = new DBEntry (St, Priority);
      Teatro.put (P, EntryNew);                      // inserisce lo spettat. nel teatro, nel posto P
    }                                                // (assegnandogli la priorita' di default!)
    else
      if (ChiCe.cheSpettatore().equals(St))          // Se c'e' gia' lo stesso spettatore,
        ;                                            // non fa nulla
      else                                           // Ma se la priorita'del nuovo arrivato e'
        if (Priority.greatestThan(ChiCe.chePriorita())) {     // maggiore di quella della entry,
          DBEntry EntryNew = new DBEntry (St, Priority);      // ne forza l'inserimento.
          Teatro.put (P, EntryNew);                  // (nota: il vecchio spettatore e' perso!)
        }
        else
          throw new Exception (STRERR_OCCUPAT);      // posto occupato da piu' prioritario-->eccezione!
  } //inserisciSeNonCe


  public synchronized void cancellaSeCe (Prenotazione P) throws Exception {
    cancellaSeCe (P.rendiSpettacolo(), P.rendiPosto(), P.rendiSpettatore());
  } //cancellaSeCe

  public synchronized void cancellaSeCe (Spettacolo Sc, Posto P, Spettatore St) throws Exception {
    // Cancellazione di una terna completa (Spettacolo, Posto, Spettatore)
    // Lavora come le "cancella", ma se il posto e' libero o occupato da un altro,
    // non genera l'eccezione
    Hashtable  Teatro = controllaERendiTeatro (Sc);  // Preleva il Teatro assoc. a spettacolo 
    DBEntry ChiCe  = (DBEntry) Teatro.get (P);       // Guarda chi e' lo spettatore 
    if (ChiCe == null)
      ;                                              // se posto libero, non fa nulla
    else
      if (ChiCe.cheSpettatore().equals(St))
        Teatro.remove (P);                           // Se lo spettatore e' lo stesso, lo toglie
      else
        ;                                            // se e' un altro, non fa nulla!
  } //cancellaSeCe


  public synchronized boolean libero (Spettacolo Sc, Posto P) throws Exception {
    // Dato spettacolo e posto, dice se e' libero o no
    Hashtable  Teatro = controllaERendiTeatro (Sc);  // Preleva il Teatro assoc. a spettacolo 
    DBEntry ChiCe  = (DBEntry) Teatro.get (P);       // Guarda se e' gia' prenotato (null=libero)
    return (ChiCe == null);
  } //libero


  public synchronized boolean liberoOCe (Prenotazione P) throws Exception {
    return liberoOCe (P.rendiSpettacolo(), P.rendiPosto(), P.rendiSpettatore());
  } //liberoOCe

  public synchronized boolean liberoOCe (Spettacolo Sc, Posto P, Spettatore St) throws Exception {
    // Dato spettacolo e posto, dice se e' libero o no per lo spettatore St
    // Ossia rende true se il posto e' libero OPPURE e' prenotato dallo stesso St
    Hashtable  Teatro = controllaERendiTeatro (Sc);  // Preleva il Teatro assoc. a spettacolo 
    DBEntry ChiCe  = (DBEntry) Teatro.get (P);       // Guarda se e' gia' prenotato (null=libero)
    if (ChiCe == null)
      return true;
    else
      return  ChiCe.cheSpettatore().equals(St);
  } //liberoOCe
  

  public synchronized boolean ceSpettatore (Prenotazione P) throws Exception {
    return ceSpettatore (P.rendiSpettacolo(), P.rendiPosto(), P.rendiSpettatore());
  } //ceSpettatore
  
  public synchronized boolean ceSpettatore (Spettacolo Sc, Posto P, Spettatore St) throws Exception {
    // Dato spettacolo e posto, rende true se e solo se e' occupato dallo spettatore St (passato)
    Hashtable  Teatro = controllaERendiTeatro (Sc);  // Preleva il Teatro assoc. a spettacolo 
    DBEntry ChiCe = (DBEntry) Teatro.get (P);        // Guarda se e' gia' prenotato (null=libero)
    if (ChiCe != null)                               // (rende null se non c'e')
      return ChiCe.cheSpettatore().equals(St);       // confronta spettatore presente con quello dato
    else
      return false;
  } //ceSpettatore


  public synchronized Spettatore chiCe (Spettacolo Sc, Posto P) throws Exception {
    // Dato spettacolo e posto, rende lo spettatore che lo occupa (o "null" se e' libero)
    Hashtable  Teatro = controllaERendiTeatro (Sc);  // Preleva il Teatro assoc. a spettacolo 
    DBEntry ChiCe  = (DBEntry) Teatro.get (P);       // Guarda se e' gia' prenotato (null=libero)
    if (ChiCe == null)                               // (rende null se non c'e')
      return null;
    else
      return ChiCe.cheSpettatore();                  // Rende lo spettatore
  } //chiCe


  // OPERAZIONI DI RAPPRESENTAZIONE DB IN LISTE: ---------------------------------------------------

  protected void aggiungiOrdinato (Vector Elenco, Spettacolo Elem) {
    Enumeration Enum = Elenco.elements();        // Scorre gli elementi gi presenti
    int i = 0;
    boolean Cont = true;
    while (Enum.hasMoreElements() && Cont)
      if (Elem.minoreDi( (Spettacolo)Enum.nextElement()) )  // Se Elem < Elenco[i]
        Cont = false;
      else
        i++;
    if (i <= Elenco.size()) 
      try {
        Elenco.insertElementAt (Elem, i);   // Inserisco nella posizione trovata
      }
      catch (Exception E) { System.out.println (E); }
    else
       Elenco.addElement (Elem);             // Oppure alla fine del vettore
  } //aggiungiOrdinato

  
  public synchronized Vector rendiElencoSpettacoli () {
    // Elenco da costruire (di dimensione pari al N.di chiavi)
    Vector Elenco = new Vector (TabellaST.size());
      
    Enumeration Enum = TabellaST.keys ();        // Scorre le chiavi!
    while (Enum.hasMoreElements())               // e le inserisce nella lista
      aggiungiOrdinato (Elenco, (Spettacolo)Enum.nextElement());
     
    Elenco.trimToSize();
    return Elenco;
  } //rendiElencoSpettacoli


  public synchronized Vector rendiElencoPostiLiberi (Spettacolo Sc) throws Exception {
    // Rende l'elenco di tutti i posti liberi di un certo spettacolo
    Hashtable  Teatro = controllaERendiTeatro (Sc);
    Vector     Elenco = new Vector (Teatro.size());

    Posto P = Posto.primoPosto();                // Prende il primo posto del teatro
    while (P.corretto()) {                       // Scorre tutti i posti possibili
      if (! Teatro.containsKey(P))               // Se il posto NON e' occupato,
        Elenco.addElement (P);                   // Inserisce nella lista
      P = Posto.successivoPosto (P);             // Passa al posto successivo (nuova istanza)
    }
    Elenco.trimToSize();
    return Elenco;
  } //rendiElencoPostiLiberi


  public synchronized Vector rendiElencoPostiOccDa (Spettacolo Sc, Spettatore St) throws Exception {
    // Rende l'elenco di tutti i posti OCCUPATI da uno spettatore in uno spettacolo 
    Hashtable  Teatro = controllaERendiTeatro (Sc);
    Vector     Elenco = new Vector (Teatro.size());

    Posto P = Posto.primoPosto();                // Prende il primo posto del teatro
    while (P.corretto()) {                       // Scorre tutti i posti possibili
      DBEntry ChiCe  = (DBEntry) Teatro.get (P); // Guarda chi e' lo spettatore
      if (ChiCe != null) 
        if (ChiCe.cheSpettatore().equals(St))    // Se c'e' lo stesso spettatore,
          Elenco.addElement (P);                 // Inserisce nella lista
      P = Posto.successivoPosto (P);             // Passa al posto successivo (nuova istanza)
    }
    Elenco.trimToSize();
    return Elenco;
  } //rendiElencoPostiOccDa


  public synchronized Vector rendiElencoPostiOccStr (Spettacolo Sc) throws Exception {
    // Rende l'elenco di tutti i posti OCCUPATI con i relativi spettatori;
    // NOTA: questo metodo rende un vettore di STRINGHE, NON di coppie posto+spettatore
    Hashtable  Teatro = controllaERendiTeatro (Sc);
    Vector     Elenco = new Vector (Teatro.size());

    Posto P = Posto.primoPosto();                // Prende il primo posto del teatro
    while (P.corretto()) {                       // Scorre tutti i posti possibili
      DBEntry ChiCe  = (DBEntry) Teatro.get (P); // Guarda chi e' lo spettatore
      if (ChiCe != null)
        Elenco.addElement (new String(P+" = "+ChiCe.cheSpettatore()));
      P = Posto.successivoPosto (P);             // Passa al posto successivo (nuova istanza)
    }
    Elenco.trimToSize();
    return Elenco;
  } //rendiElencoPostiOccStr


  public String toString() {
    return this.toString("");
  } //toString


  public String toString (String Header) {
    // Rappresentazione in stringa (formattata) del DataBase
    StringBuffer TmpS, Spazio;
    Spettacolo   Sc;
    Posto        P;
    Enumeration  EnumSc, EnumP;
    int          i;
    String       S;
    Hashtable    Teatro;

    TmpS = new StringBuffer (Header);
    TmpS.append(' ');
    Spazio = new StringBuffer("");
    for (i=0; i<=Header.length(); i++)
      Spazio.append (' ');                 // aggiunge caratteri " " per inizio
    TmpS.append("DefaultPriority="+DefaultPriority+"\n"+Spazio);

    Vector Elenco = rendiElencoSpettacoli ();
    EnumSc = Elenco.elements();            // Scorre tutti gli spettacoli
    if (EnumSc.hasMoreElements())
      while (EnumSc.hasMoreElements()) {
        Sc = (Spettacolo) EnumSc.nextElement();
        S  = Sc.toString();
        TmpS.append (S);                     // Inserisce nome spettacolo
        for (i=S.length(); i<40; i++)
          TmpS.append ('.');                 // aggiunge caratteri "." dopo il titolo

        try {
          Teatro = controllaERendiTeatro (Sc);
          TmpS.append (Teatro.toString());
        } 
        catch (Exception E) { }

        TmpS.append ("\n");
        if (EnumSc.hasMoreElements()) TmpS.append (Spazio);  // Inserisce spazio iniziale
      }
    else
      TmpS.append ("<data base vuoto>\n");

    return TmpS.toString();
  } //toString

} //DataBase