/**  Dialogo  (chi lo usa deve implementare DialogoAvvisa)
 *     @autore     Luigi Antenucci
 *     @versione   1.0
 *     @linguaggio jdk 1.2
 */

import java.awt.Frame;
import java.awt.Dialog;
import java.awt.Panel;
import java.awt.Label;
import java.awt.Font;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


public class Dialogo extends Dialog implements ActionListener {
  /** Finestra di dialogo con l'utente
   *  Puo' avere diversi tasti come nessuno
   */

  // Costanti:
  public static final boolean MODALE_SI = true;    // dialogo "modale"?
  public static final boolean MODALE_NO = false;

  public static final int BOTT_OK    = 0;  // Creazione bottoni
  public static final int BOTT_SINO  = 1;
  public static final int BOTT_nulla = 2;

  public static final int PREMUTO_NULLA = 55; // Che bottone ho premuto
  public static final int PREMUTO_OK    = 0; 
  public static final int PREMUTO_SI    = 0;
  public static final int PREMUTO_NO    = 1;

  protected static final Font FONT_MESS = new Font ("Serif",       Font.PLAIN, 24);
  protected static final Font FONT_DBOT = new Font ("DialogInput", Font.BOLD,  20);
  // in Java 1.2 uso "Serif"       anziche' "TimesRoman"

  // Variabili di classe:
  protected static Color   ColSfo   = Color.green;  // Colori per sfondo e testo
  protected static Color   ColTxt   = Color.black;
  protected static Color[] ColSfoBo = {Color.green, Color.pink }; // Colori bottoni
  protected static Color[] ColTxtBo = {Color.blue,  Color.black};
  
  // Variabili d'istanza
  protected int MAX_BOT = 2;  // se = 2, ho bottoni da 0 a 1;
  protected Bottone Bott[] = new Bottone [MAX_BOT];
  protected DialogoAvvisa AvvisareChi;  // Chi si deve avvisare


  public Dialogo (Frame FinPapa,
                  int BOTT_XX, String Messaggio1, String Messaggio2) {
    this(FinPapa, MODALE_SI, null, BOTT_XX, Messaggio1, Messaggio2);
    // per default e' modale e non avvisa nessuno!
  } //costruttore

  public Dialogo (Frame FinPapa,
                  DialogoAvvisa AvvisareChi, 
                  int BOTT_XX, String Messaggio1, String Messaggio2) {
    this(FinPapa, MODALE_SI, AvvisareChi, BOTT_XX, Messaggio1, Messaggio2);
    // per default e' solamente modale
  } //costruttore

  public Dialogo (Frame FinPapa,
                  boolean MODALE_SN,
                  int BOTT_XX, String Messaggio1, String Messaggio2) {
    this(FinPapa, MODALE_SN, null, BOTT_XX, Messaggio1, Messaggio2);
    // per default e' solamente modale
  } //costruttore

  public Dialogo (Frame FinPapa, 
                  boolean MODALE_SN,
                  DialogoAvvisa AvvisareChi, 
                  int BOTT_XX, String Messaggio1, String Messaggio2) {
    super (FinPapa);
    setModal (MODALE_SN);        // Il "dialog" e' "modale"?
    this.AvvisareChi = AvvisareChi;

    Panel Pan = new Panel ();

    GridBagLayout      GBL = new GridBagLayout ();
    GridBagConstraints GBC = new GridBagConstraints ();
    Pan.setLayout (GBL);
    
    Pan.setBackground (ColSfo);
    Pan.setForeground (ColTxt);

    Label Msg1 = new Label (Messaggio1, Label.CENTER);
  		Msg1.setFont (FONT_MESS);
    GBC.anchor = GridBagConstraints.CENTER;
    GBC.gridx = 2;  GBC.gridy = 1;  // (colonna,riga) (1,1)=alto-sin
    GBL.setConstraints (Msg1, GBC);
    Pan.add (Msg1);

    if (! Messaggio2.equals("")) {
      Label Msg2 = new Label (Messaggio2, Label.CENTER);
      Msg2.setFont (FONT_MESS);
      GBC.gridx = 2;  GBC.gridy = 2;
      GBL.setConstraints (Msg2, GBC);
      Pan.add (Msg2);
    }

    switch (BOTT_XX) {
      case BOTT_nulla: { break; }  // NESSUN BOTTONE
      case BOTT_OK   : { GBC.anchor = GridBagConstraints.CENTER;
                         Bott[0] = new Bottone ("    OK    ", ColSfoBo[0], ColTxtBo[0], FONT_DBOT);
                         GBC.gridx = 2;  GBC.gridy = 3;  // colonna-riga
                         Bott[0].addActionListener (this);
                         GBL.setConstraints (Bott[0], GBC);
                         Pan.add (Bott[0]);
                         break; }
      case BOTT_SINO : { GBC.anchor = GridBagConstraints.WEST;
                         Bott[0] = new Bottone ("    S    ", ColSfoBo[0], ColTxtBo[0], FONT_DBOT);
                         GBC.gridx = 1;  GBC.gridy = 3;  // colonna-riga
                         Bott[0].addActionListener (this);
                         GBL.setConstraints (Bott[0], GBC);
                         Pan.add (Bott[0]);
                         Bott[1] = new Bottone ("    No    ", ColSfoBo[1], ColTxtBo[1], FONT_DBOT);
                         GBC.gridx = 3;  GBC.gridy = 3;  // colonna-riga
                         Bott[1].addActionListener (this);
                         GBL.setConstraints (Bott[1], GBC);
                         Pan.add (Bott[1]);
                         break; }
    } //switch
    add (Pan);

    setTitle ("(messaggio)");
    setCursor (new Cursor (Cursor.HAND_CURSOR));
    pack();
    setLocation (80,180);
    setResizable (false);
    setVisible (true);
  } //costruttore


  public static void defColoriDialogo (Color CheColSfo, Color CheColTxt) {
    ColSfo = CheColSfo;
    ColTxt = CheColTxt;
  } //defColori

  public static void defColoriBottone (int NumBot, Color CheColSfoBo, Color CheColTxtBo) {
    if ((NumBot >= 0) && (NumBot < ColSfoBo.length)) {
      ColSfoBo[NumBot] = CheColSfoBo;
      ColTxtBo[NumBot] = CheColTxtBo;
    }
  } //defColoriBottone


  public void dispose () {
    setVisible (false);
    super.dispose();
  } //dispose


  protected void Notifica (int PREMUTO_XX) {
    if (AvvisareChi != null)
      AvvisareChi.AvvisoDaBottone (PREMUTO_XX); 
    dispose();
  } //Notifica


  public void actionPerformed (ActionEvent evt) {
	   Object BottX = evt.getSource();
	    if (BottX == Bott[0]) 
       Notifica (PREMUTO_SI); // nota: SI' = OK
     else
       if (BottX == Bott[1]) 
         Notifica (PREMUTO_NO);
 	} //actionPerformed

} //Dialogo