/**  Display - classe generale per tutti i tipi di "display"
 *     @autore     Luigi Antenucci
 *     @versione   1.0
 *     @linguaggio jdk 1.2
 */

import java.awt.Window;
import java.awt.Frame;
import java.awt.Dialog;
import java.awt.Panel;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Font;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;


class Display extends Panel implements DialogoAvvisa, WindowListener {
  /** Un generico "display"
   */

  // Costanti:
  protected static Color COL_SFO_FIN   = Color.blue;      // Finestra
  protected static Color COL_TXT_FIN   = Color.white;

  protected static Color COL_SFO_BOTOK = Color.green;     // Bottoni
  protected static Color COL_TXT_BOTOK = Color.black;
  protected static Color COL_SFO_BOTAN = Color.red;
  protected static Color COL_TXT_BOTAN = Color.white;

  protected static final Color DCOL_SFO_NOR = Color.yellow; // Fin.Dialogo
  protected static final Color DCOL_CAR_NOR = Color.blue;
  protected static final Color DCOL_SFO_ERR = Color.red;
  protected static final Color DCOL_CAR_ERR = Color.white;
                                                               // FONT:
  protected static Font FONT_BOTT  = new Font ("DialogInput", Font.BOLD,  16);
  protected static Font FONT_TEXT  = new Font ("Serif",       Font.PLAIN, 20);
  protected static Font FONT_TEXT0 = new Font ("Serif",       Font.BOLD,  22);
  // in Java 1.2 uso "Serif"       anziche' "TimesRoman"
  //                 "DialogInput" anziche' "Helvetica"


  protected Window FinConten = null;   // la finestra che contiene il Panel
  protected Frame  FinPapa   = null;   // la finestra padre (se Dialog) o null

  protected int    PremutoChe;    // Notifica del bottone di "dialogo" premuto


  public Display () {
    this ("", null);
  } //costruttore

  public Display (String TitoloFin) {
    // Crea il display, creando ANCHE la FINESTRA che lo contiene
    // Se FinPapa = null, crea un FRAME, altrim. un DIALOG
    this (TitoloFin, null);
  } //costruttore

  public Display (String TitoloFin, Frame FinPapa) {
    // Crea il display, creando ANCHE la FINESTRA che lo contiene
    // Se FinPapa = null, crea un FRAME, altrim. un DIALOG
    super ();
    this.FinPapa = FinPapa;
    if (FinPapa == null)                     // Crea la finestra
      FinConten = new Frame (TitoloFin);
    else {
      FinConten = new Dialog (FinPapa, TitoloFin);
      ((Dialog)FinConten).setModal (true);
      FinConten.setLocation (50,80);
     }
    FinConten.addWindowListener (this);    // Gestione finestra
  } //costruttore

  public Display (Window FinGiaCreata) {
    // Crea il display, inserendolo nella finestra gia' creata
    // PS: passo una finestra (FRAME o DIALOG)
    super ();
    FinConten = FinGiaCreata;               // Salva riferimento finestra
    FinGiaCreata.addWindowListener (this);  // Gestione finestra
  } //costruttore


  protected void CostruisciDisplay () {
    // Da ridefinire nelle sottoclassi, per costruire il display;
    // Va chiamata nel costruttore
    setBackground (COL_SFO_FIN);
    setForeground (COL_TXT_FIN);
    setFont (FONT_TEXT);
    setCursor (new Cursor (Cursor.HAND_CURSOR));
  } //CostruisciDisplay


  protected void FineCostrDisplay () {
    // Da invocare dopo aver costruito il display (aggiungendo tutti
    // i componenti voluti; serve per aggiungere il display alla
    // finestra e renderla visibile all'utente.
    FinConten.add (this);    // AGGIUNGE SE STESSO ALLA FINESTRA!
    FinConten.pack();
    FinConten.setVisible (true);
  } //FineCostrDisplay


  public Window rendiFinContenente () {
    // rende la finestra del display (Dialog o Frame)
    return FinConten;
  } //rendiFinContenente


  public Frame rendiFinPapa () {
    // rende la finestra padre (se il display e' un Dialog)
    // rende null se il display  un Frame
    return FinPapa;
  } //rendiFinPapa

  
  public void riposiziona (int x, int y) {
    if (FinConten != null)
      FinConten.setLocation (x, y);
  } //riposiziona


  public void ridimensiona (int x, int y) {
    if (FinConten != null)
      FinConten.setSize (x, y);
  } //ridimensiona


  public void rititola (String NuovoTitolo) {
    if (FinConten != null)
      ((Frame)FinConten).setTitle (NuovoTitolo);
  } //rititola


  public void distruggiDisplay () {
    if (FinConten != null) {
      FinConten.setVisible (false);
      if (FinPapa == null) ((Frame) FinConten).dispose();
      else                 ((Dialog)FinConten).dispose();
      FinConten = null;
    }
  } //distruggiDisplay


  public void AvvisoDaBottone (int BottonePremuto) {
    // Implementazione del metodo dell'interfaccia "DialogoAvvisa"
    PremutoChe = BottonePremuto;
  } //AvvisoDaBottone


  public Dialogo apriDialogoLibero (int BOTT_XX, String Mess1, String Mess2) {
    // Apre una finestra di dialogo che NON blocca il chiamante
    // Rende il "Dialogo" su cui fare la "dispose()" per rimuoverlo
    Dialogo.defColoriDialogo (DCOL_SFO_NOR, DCOL_CAR_NOR);
    return new Dialogo ((Frame) FinConten,
                        Dialogo.MODALE_NO,
                        BOTT_XX, Mess1, Mess2);
  } //apriDialogoLibero

  public int apriDialogoModale (int BOTT_XX, String Mess1, String Mess2) {
    // Rende PREMUTO_XXX (vedi "Dialogo")
    Dialogo.defColoriDialogo (DCOL_SFO_NOR, DCOL_CAR_NOR);
    Dialogo MyDialogo = new Dialogo ((Frame) FinConten, 
                                     Dialogo.MODALE_SI, this,
                                     BOTT_XX, Mess1, Mess2);
    return PremutoChe;
  } //apriDialogoModale

  public int apriDialogoErrModale (int BOTT_XX, String Mess1, String Mess2) {
    // Rende PREMUTO_XXX (vedi "Dialogo")
    Dialogo.defColoriDialogo (DCOL_SFO_ERR, DCOL_CAR_ERR);
    Dialogo MyDialogo = new Dialogo ((Frame) FinConten, 
                                     Dialogo.MODALE_SI, this,
                                     BOTT_XX, Mess1, Mess2);
    return PremutoChe;
  } //apriDialogoErrModale


  // Azioni sulla finestra
  public void windowClosed      (WindowEvent evt) { }
  public void windowDeiconified (WindowEvent evt) { }
  public void windowIconified   (WindowEvent evt) { }
  public void windowActivated   (WindowEvent evt) { }
  public void windowDeactivated (WindowEvent evt) { }
  public void windowOpened      (WindowEvent evt) { }
  public void windowClosing     (WindowEvent evt) { }

} //Display