/**  GestCli
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.net.Socket;
import java.net.InetAddress;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.ObjectOutputStream;
import java.util.Vector;
import java.io.IOException;
import java.util.Hashtable;
import java.lang.InterruptedException;


public class GestCli extends Thread implements GestSerAvvisa {
  /** Gestione di un messaggio arrivato da un client.
   *  Qui sono racchiuse le politiche relative a ogni messaggio.
   */

  // Costanti per indicare a GestCli lo stato della prenotazione
  public static final int FATTA_SI  = 0;  // prenotazione fatta con successo
  public static final int FATTA_NO  = 1;  // non e' stato possibile propagarla
  public static final int FATTA_ERR = 2;  // errore durante la tranzazione 
  protected static char[] FATTA_2_CHAR = {'S', 'N', 'E'} ;

  // Variabili pubbliche per definire la RITRASMISSIONE del GEST-CLI:
  public    static int  TIMEOUT_GC   = 10000;  // TIME-OUT per RITRASMISSIONE (in millisecondi)
  public    static int  MAX_TENTA_GC = 5;      // Massimo numero di ritrasmissioni

  // Variabile di classe per distinguere con un numero i vari thread concorrenti
  protected static  int NumIstanza = 0;

  // Variabili d'istanza:
  protected String  MioNome;
  protected int     TransazioneFatta;  // Notifica dello stato della transazione
  protected Socket  SockCli;           // Socket di ricezione/invio dati


  public GestCli (Socket CliSock) {
    // CliSock e' la socket che viene resa dopo un "accept"; da essa questo GestCli leggera'
    // i dati e creera' il messaggio che li rappresenta.
    NumIstanza++;
    String O;
    if (NumIstanza <= 9) O = "0";
    else                 O = "";
    MioNome = "GestCli"+O+String.valueOf(NumIstanza);;
    setName (MioNome);
    SockCli=CliSock;                         // si copia la socket di comunicazione
    TransazioneFatta = FATTA_ERR;
  } // GestCli


  public void run() {
    MessCli MyMess;   // NB: gestisce un MESSAGGIO DI CLIENTE!

    DebugInfo ("lanciato");

    try {

      DataInputStream DIS = new DataInputStream (SockCli.getInputStream());
      MyMess = (MessCli) TraslaMess.creaMess (DIS);
      // Non devo fare "DIS.close();" se no mi chiude la socket!

      DebugInfo ("riconosciuto messaggio "+MyMess.toString());

      if ((MyMess instanceof MessCliRichiedi) ||     //MyMess=Richiesta/Cancella
          (MyMess instanceof MessCliCancella)) 
            Servi_MessCliRichiediCancella (MyMess);
      
      if (MyMess instanceof MessCliDammi)            //MyMess="sottoclassi" di Dammi
        Servi_MessCliDammi ((MessCliDammi)MyMess);   // (serve tutte le sue sottoclassi)

    } 
    catch (Exception Err) { Errore(Err); }
    finally {
      DebugInfo ("chiusura socket ricezione");
      try {
        SockCli.close();
      }
      catch (Exception Err) { Errore(Err); };
      DebugInfo ("TERMINATO");
    }
  } //run

  
  public synchronized void operazioneFatta (int TransazioneFattaSiNo) {
    // Implementazione del metodo dell'interfaccia ""
    // Metodo usato per sbloccare il thread mentre e' in attesa della risposta da parte di
    // un (qualsiasi) thread "GestSer" (operazione terminata con successo o non fatta).
    // Serve per far uscire dal ciclo "while" (lo stato dell'operazione e' stato salvato)
    TransazioneFatta=TransazioneFattaSiNo;
    notify();   // NOTIFICA RISVEGLIO THREAD
  } //operazioneFatta


// METODI INTERNI; ESEGUONO LA POLITICA DI GESTIONE DI OGNI MESSAGGIO:
// (il metodo "Servi_MessCliXXX" gestisce un "MessCliXXX")


  protected synchronized void Servi_MessCliRichiediCancella (MessCli MyMess) {
    // La "Avvio_XXX" e' contenuta in "GestSer"
    // e' "synchronized" perche' per fare la "wait" deve possedere il monitor
    // INTERNA
    Prenotazione     Prenot;
    GestSer          MyGestSer;
    MessSerDgPrenota MessBis;
    MessCliInfo      MsgCli;

    // Leggo la prenotazione contenuta nel messaggio arrivato
    Prenot = ((MessCliRichiedi) MyMess).chePrenotaz();

    // Controllo che i dati siano "corretti".
    if (! Prenot.corretto()) {

      // INVIA MESSAGGIO A CLIENTE E TERMINA!!!
      String Frase ="DATI INSUFFICIENTI";
      DebugInfo (Frase);
      InviaMessaggioAlCliente ( new MessCliInfo(Frase) );

      return;  // FINE del metodo
    } //if
 
    // Se e solo se e' la prima prenotazione (tentativo=1) e non e'
    // in atto nessun altro GestCli (lo vedo controllando in 
    // "MiePren") si controlla nel data-base per l'esistenza!
    if (MyMess.cheTentativo() == 1)
      try {
        if (! Server.MiePren.contains(Prenot)) {   // Non e' in atto,

          // Controllo: esiste lo spettacolo?
          if (! Server.IDataBase.esisteSpettacolo (Prenot.rendiSpettacolo())) {
            InviaMessaggioAlCliente ( new MessCliInfo("LO SPETTACOLO NON E` IN PROGRAMMA") );
            return; // EXIT;
          }

          // Controllo: si pu eseguire l'operazione (o non si pu gi in partenza)?
          boolean Libero = Server.IDataBase.libero (Prenot.rendiSpettacolo(),
                                                    Prenot.rendiPosto()      );
          String Frase = null;
          if (MyMess instanceof MessCliCancella)
            if (Libero)
              Frase = "IL POSTO E` GIA' LIBERO";
            else 
              if (! Server.IDataBase.ceSpettatore (Prenot))
                Frase = "IL POSTO NON E` OCCUPATO DALLO SPETTATORE";
              else
                ; //nulla
          else  //e' un MessCliRichiedi
            if (! Libero) {
              Frase = "IL POSTO E` OCCUPATO ";
              if (Server.IDataBase.ceSpettatore (Prenot))
                Frase = Frase.concat("DALLO STESSO SPETTATORE");
              else
                Frase = Frase.concat("DA QUALCUN ALTRO");
            }
          if (Frase != null) {

            // INVIA MESSAGGIO A CLIENTE E TERMINA!!!
            InviaMessaggioAlCliente ( new MessCliInfo(Frase) );

            return;  //EXIT
          }
        } //if
      } 
      catch (Exception E) { }

    // Creo in MessBis il messaggio che sara' passato tra i server
    // NB: controllo prima "CANCELLA" perche' e' sottoclasse di PRENOTA

    if (MyMess instanceof MessCliCancella)
      MessBis = new MessSerDgCancella (Prenot);
    else
      MessBis = new MessSerDgPrenota  (Prenot);

    MessBis.defMacchina (Server.MaMa);    // Definisce la macchina di origine del mess.

    TransazioneFatta=FATTA_ERR;           // per ora, non e' stata fatta la prenotazione
    // NOTA: questo processo NON scrive nel DataBase (lo fara' il "GestSer")

    DebugInfo ("inserendo prenotazione in 'MiePren'");
    Debug.out ("[MiePren] "+Server.MiePren.toString());
    Object X = Server.MiePren.put (Prenot, this);  // memorizza la prenotazione e se stesso!
    if (X != null) {                               // Se c'era gia',
      DebugInfo ("in MiePren c'e' gia' la stessa prenotazione -> risveglio il vecchio GestCli con 'no'");
      // Devo risvegliare il (GestCli)TGC ? Si', ma gli dico che non ho fatto nulla!
      GestCli ThrGestCli = (GestCli) X;
      try {
        if (ThrGestCli.isAlive())       // Prima controllo che sia ancora vivo.
          ThrGestCli.operazioneFatta (FATTA_NO);
      }
      catch (Exception E) { Errore (E); }
    } //if 
    Debug.outln (" --> "+Server.MiePren.toString());

    byte Tenta=0;
    do {

      Tenta++;
      DebugInfo ("innescando un thread GestSer (tentativo "+Tenta+")");
      // crea un processo di gestione GestSer per il messaggio appena creato, 
      MyGestSer = new GestSer (MessBis, GestSer.GEN_ORA_DA_ME);
      // lo innesca e si mette in ATTESA!
      MyGestSer.start();

      // CONTROLLO LO STATO DELLE TRANSAZIONI IN "AVANTI"
      try {
   	    // NB: SI VIENE SOSPESI FINCHE' LE TRANSAZIONI NON SONO ABILITATE!
        Server.GestTransaz.AttendiSeTransazBloccate (MessSer.AVANTI);
      }
      catch (Exception E) { Errore (E); }

      DebugInfo ("attendendo (con 'wait') il risveglio da un nuovo GestSer o il time-out..");

      // Attende finche' o scade il tempo o transazione fatta (Si' o No),
      // ossia attende il risveglio con "notify" entro il "time-out"
      // Il notify e' invocato dalla "operazioneFatta"
      try {
        try {
          wait (TIMEOUT_GC);
        } 
        catch (InterruptedException E) { Debug.outln ("'wait' GestCli interrotta"); }
      }
      catch (Exception E2) { Errore (E2); }

      // Ora per sapere cosa e' successo, basta vedere se e' stata 
      // modificata la variabile "TransazioneFatta".
      if (TransazioneFatta == FATTA_ERR)
        DebugInfo ("si e' superato il TIME-OUT!");
      else
        DebugInfo ("sono stato risvegliato, con TransFatta="+FATTA_2_CHAR[TransazioneFatta]);

    }
    while ((TransazioneFatta == FATTA_ERR) &&   // esce quando transazione fatta S/N,
           (Tenta <  MAX_TENTA_GC));            // o in ogni caso dopo MAX tentativi

    if (TransazioneFatta == FATTA_ERR) {        // NON SONO RIUSCITO A FARE LA TRANSAZIONE!
      Protocollo Pro = new Protocollo ();       // LANCIA IL PPROTOCOLLO "PASSAMI" PER
      Pro.Avvio_MessSerPassami ();              // CONTROLLARE E RECUPERARE UN GUASTO.
    }

    // Prepara la risposta per il processo client (in attesa sulla SockCli)
    // (a seconda del contenuto di "TransazioneFatta")
    DebugInfo ("inviando risposta a Client tramite socket");
    String Frase;
    if (TransazioneFatta == FATTA_SI)
        Frase = "Operazione Completata con Successo";
    else
      if (TransazioneFatta == FATTA_NO)
        Frase = "NON E' STATO POSSIBILE COMPLETARE L'OPERAZIONE";
      else
        Frase = "NON E' STATO POSSIBILE FARE L'OPERAZIONE";

    InviaMessaggioAlCliente ( new MessCliInfo(Frase) );

  } //Servi_MessCliRichiediCancella


  protected void Servi_MessCliDammi (MessCliDammi MyMess) {
    if ((MyMess instanceof MessCliDammiSc)  ||
        (MyMess instanceof MessCliDammiPL)  ||
        (MyMess instanceof MessCliDammiPOMe)  )
          try {
            DebugInfo ("costruendo l'elenco");
            Vector Elenco;

            // In base al messaggio, creo un vettore con elementi differenti
            if (MyMess instanceof MessCliDammiSc)       // Elenco degli spettacoli
               Elenco = Server.IDataBase.rendiElencoSpettacoli ();
            else {
              // Legge dal messaggio che spettacolo si vuole
              Spettacolo Sc = ((MessCliDammiPL) MyMess).cheSpettac();

              if (MyMess instanceof MessCliDammiPOMe) {   // Elenco dei posti occupati da ME
                // Lo spettacolo e' gia' in Sc
                Spettatore St = ((MessCliDammiPOMe)MyMess).cheSpettat();
                // Crea un vettore con l'elenco degli spettacoli
                Elenco = Server.IDataBase.rendiElencoPostiOccDa (Sc, St);
              }
              else  // (MyMess instanceof MessCliDammiPL)
                Elenco = Server.IDataBase.rendiElencoPostiLiberi (Sc);  // (e' sottoclasse)

            } //else

            // Invia all'utante l'intero vettore usando la "serializzazione"
            // dell'oggetto "Vector" (che implementa "Serializable"
            DebugInfo ("inviando l'elenco come un oggetto");
            ObjectOutputStream MyOut = new ObjectOutputStream (SockCli.getOutputStream());
            MyOut.writeObject(Elenco);
            MyOut.flush();
          }
          catch (Exception Err) { Errore(Err); }

    else
      if (MyMess instanceof MessCliDammi2Ser) {
            // Rendo al client due stringhe contenenti due indirizzi di server (i miei vicini)
            Macchina Mak;
            String   Addr;
            byte     dir;
            try {

              DataOutputStream DOS = new DataOutputStream (SockCli.getOutputStream());

              byte N = (byte) (MessSer.INDIETRO+1);
              DebugInfo ("inviando "+N+" stringhe:");
              DOS.writeByte (N);   // Dico che inviero' DUE STRINGHE

              for (dir=MessSer.AVANTI; dir<=MessSer.INDIETRO; dir++) {
                Mak = Server.GestTransaz.rendiMacchinaVicina (dir);
                if (Mak.equals(Server.MaMa))     // Non rendo il mio stesso indirizzo!
                  Addr = "";
                else
                  Addr = Mak.cheIndirizzo();
                DebugInfo ("inviando '"+Addr+"'");
                DOS.writeUTF (Addr);                                  // Eccole qui
              }
              DOS.flush();
            }
            catch (Exception Err) { Errore(Err); }
      }
  } //Servi_MessCliDammi


// METODO GENERALE PER INVIARE UN MESSAGGIO "MessCli" AL CLIENTE: ---------------------
  protected void InviaMessaggioAlCliente (MessCli MsgPerCli) {
    // INTERNA
    // Passa il messaggio al cliente tramite la socket-stream (aperta)

    try {
      DebugInfo ("inviando al cliente un messaggio "+MsgPerCli.toString());
      DataOutputStream MyOut = new DataOutputStream (SockCli.getOutputStream());
      TraslaMess.scriviMess (MsgPerCli, MyOut);
      MyOut.flush();
      // Nota: non faccio "MyOut.close();" se no chiudo anche la socket
    } 
    catch (IOException Err) { Errore(Err); }

  } //InviaMessaggioAlCliente


  protected void DebugInfo (String Frase) {
    Debug.outln (MioNome + "- "+Frase);
  } //DebugInfo

  protected void Errore (Exception Err) {
    Debug.outln (MioNome + "- ERRORE: "+Err);
  } //Errore

} //GestCli