/**  GestSer
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Hashtable;


public class GestSer extends Thread {
  /** Gestione di un messaggio arrivato da un server.
   *  Qui sono racchiuse le politiche relative a ogni messaggio.
   */

  // COSTANTI
  public final static byte GEN_ORA_DA_ME = 0;   // Costanti per generazione "ora"
  public final static byte GEN_DA_SERVER = 1;

  // Variabile di classe per distinguere con un numero i vari thread concorrenti
  protected static  int NumIstanza = 0;

   // Variabili di istanza 
  protected String     MioNome;
  protected MessSerDg  MyMess;    // Per ricordarsi il messaggio 
  protected byte       GenOra;    // e se e' stato generato ora.

  
  public GestSer (MessSerDg MyMess, byte GenOra) {
    // MyMess e' il messaggio (di classe MessSerDg) che questo GestSer deve gestire
    // Se GenOra=GEN_ORA_DA_ME, il messaggio e' stato appena creato da un processo
    // di classe GestCli e quindi non si deve controllare se e' in "MiePren".
    NumIstanza++;
    String O;
    if (NumIstanza <= 9) O = "0";
    else                 O = "";
    MioNome = "GestSer"+O+String.valueOf(NumIstanza);
    setName (MioNome);
    this.GenOra = GenOra;
    this.MyMess = MyMess;
  } // GestSer


  public void run() {
    DebugInfo ("lanciato");

    DebugInfo ("riconosciuto messaggio "+MyMess.toString());


    // Controllo se il messaggio e' arrivato da una delle macchine definite come
    // "vicine" a quella corrente; se NO, il messaggio VA ELIMINATO!
    if (MyMess instanceof MessSerDg)

      if ((MyMess instanceof MessSerDgPrenota)  ||       // MyMess = Prenota/Cancella
          (MyMess instanceof MessSerDgCancella) ||
          (MyMess instanceof MessSerDgAggSc)    ||
          (MyMess instanceof MessSerDgCanSc)      )
            Servi_MessSerDg_PrenCanc_AggCanSc ();       // side-effect su MyMess e GenOra
      else
        DebugInfo ("MESSAGGIO SCONOSCIUTO!");

    else
      DebugInfo ("IL MESSAGGIO NON E' DI CLASSE 'MessSerDg' --> NON POSSO SERVIRLO");

    DebugInfo ("TERMINATO");
  } //run


  protected void RisvegliaThreadAssoc (Dato PreSpe, int TransFattaSiNo) {
    // Risveglia il thread GestCli (in attesa) associato alla prenotazione Pre in "MiePren", 
    // dandogli lo stato della transazione.
    // Oppure se PreSpe e' uno spettacolo, e' il Server (unico) che sta aspettando.
    // Se Pre non c'e' in MiePren o se Spe non e' quello atteso, si genera un'eccezione!

    if (PreSpe instanceof Prenotazione) {
      GestCli ThrGestCli = (GestCli) Server.MiePren.remove (PreSpe);

      // Se la prenotazione e' stata trovata, 
      if (ThrGestCli != null) {

        // Altrimenti risveglia il thread dandogli lo stato dell'operazione
        DebugInfo ("ho trovato la prenotazione in 'MiePren'");

        // Prima controllo che sia ancora vivo.
        boolean Vivo;
        try {
        	 Vivo = ThrGestCli.isAlive();
        } 
        catch (Exception E) { Vivo = false; }

        // informo il thread sullo stato dell' operazione (fatta S/N)
        if (Vivo) {
          DebugInfo ("risveglio del thread GestCli associato (in 'MiePren')");
          ThrGestCli.operazioneFatta (TransFattaSiNo); 
        }
        else
          DebugInfo ("il thread GestCli associato e' gia' morto!");
      } //if
      else {  // Prenotazione non trovata
        DebugInfo ("Prenotazione "+((Prenotazione)PreSpe).toString()+
                   " NON trovata in 'MiePren'");
        DebugInfo ("[MiePren] "+Server.MiePren.toString());
      } //else

    } //if
    else 

      if (PreSpe instanceof Spettacolo) {                      // e' uno Spettacolo
        // L'unico spettacolo "atteso" e' in "Server.MioSpet"
        // Se non corrisponde con quello passato, genera un'eccezione
        if (Server.MioSpet != null) {

          if (Server.MioSpet.equals((Spettacolo)PreSpe)) {      // Coincidono?

            // Se si', risveglia il thread Server dandogli lo stato dell'operazione
            DebugInfo ("risveglio del Server");
            Server.ServerAllocato.operazioneFatta (TransFattaSiNo); 

            Server.MioSpet = null;   // NOTA BENE!

          }
          else
            DebugInfo ("Spettacolo "+((Spettacolo)PreSpe).toString()+
                       " NON coincide con quello in 'MioSpet' ("+Server.MioSpet+")");
        }
        else 
          DebugInfo ("Non sono in attesa di Spettacolo ('MioSpet' e' vuoto)");
      }
      else
          DebugInfo ("Risveglio disponibile solo per Prenotazioni e Spettacoli");

  } //RisvegliaThreadAssoc

  
  protected void Servi_MessSerDg_PrenCanc_AggCanSc () {
    // INTERNA
    // Esegue la politica di gestione associata ai messaggi di classe MessSerDgPrenota/Cancella
    // Le due "Avvio_XXX", relative alle due classi primarie del messaggio, sono
    // contenute rispattivamente in "GestCli" e "Server"
    // Preleva i dati contenuti nel messaggio
    byte     Direz    = MyMess.cheDirez();
    Macchina MacOrig  = MyMess.cheMacchina();           // leggo macchina del messaggio
    Priorita Priority = MacOrig.chePriorita();
                          // Uso protocollo basato sulla "prenotazione" o su "spettacolo":
    boolean  UsoPren = (MyMess instanceof MessSerDgPrenota) ||
                       (MyMess instanceof MessSerDgCancella);

    Dato PreSpe;                            // Prenotazione o Spettacolo, a seconda del protocollo
    if (UsoPren)  PreSpe = ((MessSerDgPrenota) MyMess).chePrenotaz();     // Legge la prenotazione
    else          PreSpe = ( (MessSerDgAggSc)  MyMess).cheSpettacolo();   // Legge lo spettacolo

    // SEGNALO L'INIZIO DI UNA TRANSAZIONE ********************
    // NB: SI VIENE SOSPESI FINCHE' LE TRANSAZIONI NON SONO ABILITATE!
    try {
      Server.GestTransaz.iniziaTransazione (Direz);
    } 
    catch (Exception E) { Errore(E); }

    // CONTROLLO SULLA PROVENIENZA DEL MESSAGGIO (GENERATO DA ME O ARRIVATO DA RETE)
    if (GenOra == GEN_DA_SERVER) {  // Se il messaggio e' arrivato da un server,
      DebugInfo ("il mess. e' arrivato da un server (GenOra=GEN_DA_SERVER)");

      // Controlla se l'indirizzo del server contenuto nel messaggio e' lo stesso di
      // quello del server corrente (memorizzato in "Server.MaMa").
      // Se si', il messaggio l'avevo generato io, percio' c'e' un thread GestCli da 
      // svegliare (lo trovo cercando la prenotazione in MiePren).
      

      if (Server.MaMa.equals(MacOrig)) {                 // e' quella locale?
        DebugInfo ("riconosciuto arrivo di un MIO MESSAGGIO (stessa macchina)");

        // Definisce lo stato della transazione (se l'aveva iniziato questo server)
        int TransFattaSiNo;
        if (Direz == MessSer.AVANTI)
          TransFattaSiNo = GestCli.FATTA_SI;
        else
          TransFattaSiNo = GestCli.FATTA_NO;
        // nota: se la direzione del messaggio e' "avanti", significa che il messaggio
        // (che so di aver generato io in avanti) e' tornato.. ha fatto l'intero giro!
        // Poteva essere sia una prenotazione sia una cancellazione; ha avuto successo.
        // Se e' arrivato con direzione "indietro", significa che e' una cancellazione
        // di un messaggio di prenotazione che ho inviato.
        // In conclusione: direzione avanti=successo; indietro=errore.

        // Se e' tornata "indietro", era una "cancellazione" di una prenotazione,
        // Qui devo ancora cancellarla (ossia, manca da disfare la prima fatta!)
        if (TransFattaSiNo == GestCli.FATTA_NO)
          try {
            if (UsoPren)  Server.IDataBase.cancellaSeCe        ((Prenotazione)PreSpe);
            else          Server.IDataBase.togliSpettacoloSeCe ( (Spettacolo) PreSpe);
          }
          catch (Exception E) { Errore (E); }

        // Risveglia il processo sospeso; se non c'e', non fa nulla
        RisvegliaThreadAssoc (PreSpe, TransFattaSiNo);

        // SEGNALO LA FINE DELLA TRANSAZIONE ********************
        Server.GestTransaz.terminaTransazione();

        return;    // FINE DEL METODO (NON si esegue il codice sottostante!)

      } //if (MaMa.equals(MacOrig))

    } //if (GenOra == GEN_DA_SERVER)
    else
      DebugInfo ("il mess. e' stato appena generato da me (GenOra=GEN_ORA_DA_ME)");

    // SE SONO QUI,  GenOra=GEN_ORA_DA_ME
    //       oppure  GenOra=GEN_DA_SERVER ma NON e' un messaggio che ho generato io (MacOrig<>MaMa)

    // Aggiunge nel DataBase la nuova prenotazione
    try {
      if (UsoPren)
        if (MyMess instanceof MessSerDgCancella)
          // Cancella da DB; se c'e' lo stesso spettatore o se e' libero, rende ok.
          Server.IDataBase.cancellaSeCe ((Prenotazione)PreSpe);
        else
          if (GenOra == GEN_ORA_DA_ME)
            Server.IDataBase.inserisciSeNonCe ((Prenotazione)PreSpe);
          else
            Server.IDataBase.inserisciSeNonCe ((Prenotazione)PreSpe, Priority);
              // Inserisce in DB; se posto libero o occupato da stesso spettatore, rende ok.
              // NOTA BENE: se l'ho generato io NON faccio il controllo di priorita', mentre
              // se arriva dall'anello inserisco "con priorita'" (se il posto e' gia'
              // occupato, tra il vecchio e il nuovo dato "vince" chi ha maggior priorita')
              // Se il nuovo NON vince, viene generata un'eccezione dal DataBase.
              // NB: la "cancellazione" NON ha "priorita': o il posto e' mio (e allora posso
              // cancellarmi; o c'e' qualcun altro, e allora non posso toccarlo).
      else  // Analogamente per gli SPETTACOLI, MA NON C'E' IL CONCETTO DI PRIORITA',
            // per non gravare troppo nell'accesso al DataBase.
        if (MyMess instanceof MessSerDgCanSc)
          Server.IDataBase.togliSpettacoloSeCe ((Spettacolo)PreSpe);
        else
          Server.IDataBase.aggiungiSpettacoloSeNonCe ((Spettacolo)PreSpe);

      // Se sono qui, tutto e' andato bene (ho inserito o cancellato la prenotazione)
      // Passo quindi la prenotazione nell'anello (stessa direz. e una vita in meno)
      DebugInfo ("DataBase modificato con successo");
      Debug.out (Server.IDataBase.toString("[DataBase]"));

      // Passa in "avanti" il messaggio (senza conferma)
      FaiCircolareMess (MyMess);

    } 
    catch (Exception Err) {

      // SE SONO QUI, non sono riuscito a scrivere la prenotazione nel DB
      Errore (Err);
      Debug.out (Server.IDataBase.toString("[DataBase]"));
 
      if (GenOra == GEN_DA_SERVER) {  
        // Se il messaggio e' arrivato da un server, invio un messaggio 
        // di cancellazione della stessa prenotazione nella direzione opposta.
        // Nota: una cancellazione NON puo' andare male se ho GenOra=false.
        DebugInfo ("invio INDIETRO un messaggio di cancellazione");
  
        MessSerDg MsgBis;
        if (UsoPren)  MsgBis = new MessSerDgCancella ((Prenotazione)PreSpe);
        else          MsgBis = new MessSerDgCanSc    ( (Spettacolo) PreSpe);
        byte DirOpp = MessSer.cambiaDirez(MyMess.cheDirez());
        MsgBis.defDirez (DirOpp);                        // in direzione opposta a MyMess
        MsgBis.defMacchina (MyMess.cheMacchina());       // mantiene l'indirizzo macchina

        // NON faccio piu' circolare il MyMess, 
        // MA mando all'indietro il MsgBis
        FaiCircolareMess (MsgBis);

      }                  // Se invece il messaggio e' stato appena generato,
      else               // Risveglia il processo sospeso; se non c'e', non fa nulla
        RisvegliaThreadAssoc (PreSpe, GestCli.FATTA_NO);

    } //catch            // E ora che non sono piu' "attivo",
    finally {            // SEGNALO LA FINE DELLA TRANSAZIONE ********************
        Server.GestTransaz.terminaTransazione();
    }
  } //Servi_MessSerDg_PrenCanc_AggCanSc


  protected void FaiCircolareMess (MessSerDg MsgSer) {
    // INTERNA
    // Passa il messaggio al successore/predecessore (in base alla "direzione" contenuta 
    // nel messaggio) ma prima ne decrementa la "vita" (una vita in meno)
    String          IndVicino;
    DatagramSocket  Sock;
    DatagramPacket  Pacchetto;
    InetAddress     IndirSer;
    DataOutputStream      DOS;
    ByteArrayOutputStream BAOS;

    if (MsgSer.cheVita() > MessSerDg.VITA_ZERO) {
      MsgSer.decrementaVita();
      DebugInfo ("aggiornando vita messaggio a "+MsgSer.cheVita());

      try {
        Macchina MakVicina = Server.GestTransaz.rendiMacchinaVicina (MsgSer.cheDirez());
        IndVicino = MakVicina.cheIndirizzo();
        DebugInfo ("identificando server 'vicino': "+IndVicino);

        if (Server.sonoDaSolo()) {
          DebugInfo ("SONO UN MISERO SERVER UNICO!");
          Debug.outln ("  !!HEY!!  IL SISTEMA NON E' AFFIDABILE!");
          Debug.outln ("           INSERIRE AL PIU' PRESTO UN ALTRO SERVER NELL'ANELLO!");

          try {
            Dato PreSpe;
            // Sveglia il GestCli; se non c'e', non fa nulla
            if ((MsgSer instanceof MessSerDgPrenota) ||            // Uso prenotaz. o spettacolo?
                (MsgSer instanceof MessSerDgCancella)  )
              PreSpe = ((MessSerDgPrenota) MsgSer).chePrenotaz();     // Legge la prenotazione
            else
              PreSpe = ( (MessSerDgAggSc)  MsgSer).cheSpettacolo();   // Legge lo spettacolo

            RisvegliaThreadAssoc (PreSpe, GestCli.FATTA_SI);
          } 
          catch (Exception E) { }  // nulla

          return;  // VAI FUORI!
        } //if

        IndirSer = InetAddress.getByName (IndVicino);
        
        DebugInfo ("creando socket a datagrammi con "+IndirSer);
        Sock   = new DatagramSocket ();         // Crea una nuova socket e il pacchetto

        try {
          BAOS = new ByteArrayOutputStream (Comune.DIM_PACK);
          DOS  = new DataOutputStream (BAOS);
          TraslaMess.scriviMess (MsgSer, DOS);
          Pacchetto= new DatagramPacket (BAOS.toByteArray(), BAOS.size(),
                                         IndirSer, Comune.PORTA_DATAGRAM);
          // Non fare  DOS.close() e BAOS.close()
 
          DebugInfo ("inviando pacchetto di lunghezza "+Pacchetto.getLength());

          Sock.send (Pacchetto);

        }
        catch (Exception E) { Errore (E); }
        finally {
          Sock.close();                       // Chiudo la socket
        }
      } 
      catch (Exception Err) { Errore(Err); }

    } // if cheVita > MessSerDg.VITA_ZERO
    else {
      DebugInfo ("messaggio con VITA=0 --> eliminazione");
      // non passa piu' il messaggio!
    }
  } //FaiCircolareMess
 
  
  protected void DebugInfo (String Frase) {
    Debug.outln (MioNome + "- "+Frase);
  } //DebugInfo

  protected void Errore (Exception Err) {
    Debug.outln (MioNome + "- ERRORE: "+Err);
  } //Errore

} //GestSer