/**  GestSys
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.net.Socket;
import java.io.IOException;


public class GestSys extends Protocollo {
  /** Gestione di un messaggio di sistema arrivato da un server.
   *  Qui sono racchiuse le politiche relative a ogni messaggio.
   */

  // Variabile di classe per distinguere con un numero i vari thread concorrenti
  protected static  int NumIstanza = 0;

  // Variabili d'istanza
  protected String  MioNome;
  protected Socket  SysSock;     // Per memorizzare la socket


  public GestSys (Socket SysSock) {
    // SysSock e' la socket che viene resa dopo un "accept"; da essa questo GestSys 
    // leggera' i dati e creera' il messaggio che li rappresenta.
    NumIstanza++;
    String O;
    if (NumIstanza <= 9) O = "0";
    else                 O = "";
    MioNome = "GestSys"+O+String.valueOf(NumIstanza);
    setName (MioNome);
    this.SysSock = SysSock;
  } // costruttore


  public void run() {
    MessSer MyMess;   // NB: gestisce un MESSAGGIO DI SISTEMA DI SERVER!

    DebugInfo ("lanciato");

    try {

      MyMess = riceviMessaggio (SysSock);

      // DebugInfo ("riconosciuto messaggio "+MyMess.toString());
      // Non lo stampo qui, ma lo fara' "Protocollo", per ogni mess. ricevuto
      // RICORDA: "instanceof" PRIMA sulle sottoclassi, POI (in "else") sulla loro superclasse
      if ((MyMess instanceof MessSerSblocca) ||       //MyMess=Sblocca/blocca transazioni
          (MyMess instanceof MessSerBlocca) )
        Servi_MessSerBloccaSblocca ((MessSerBlocca)MyMess, SysSock);

      if (MyMess instanceof MessSerCambSer)           //MyMess=Cambia indirizzo server
        Servi_MessSerCambSer ((MessSerCambSer)MyMess, SysSock);

      if (MyMess instanceof MessSerTogMe)             //MyMess=Aggiungimi
        Servi_MessSerTogMe ((MessSerTogMe)MyMess, SysSock);
      else
        if (MyMess instanceof MessSerAggMe)           //MyMess=Toglimi
          Servi_MessSerAggMe ((MessSerAggMe)MyMess, SysSock);
          // NB: e' la superclasse di "Togli-me" --> va messa in "else"

      if (MyMess instanceof MessSerRiaggancio)        //MyMess=Riaggancio
        Servi_MessSerRiaggancio ((MessSerRiaggancio)MyMess);
      else
        if (MyMess instanceof MessSerRiaggConMe)      //MyMess=RiaggancioConMe
          Servi_MessSerRiaggConMe ((MessSerRiaggConMe)MyMess, SysSock);
        else
          if (MyMess instanceof MessSerPassami)       //MyMess=Passami
            Servi_MessSerPassami ((MessSerPassami)MyMess);

    } 
    catch (Exception Err) { Errore(Err); }
    finally {
      DebugInfo ("chiusura socket ricezione");
      try {
        SysSock.close();
      } 
      catch (Exception Err) { Errore(Err); };
      DebugInfo ("TERMINATO");
    }
  } //run


  protected void DebugInfo (String Frase) {
    Debug.outln (MioNome + "- "+Frase);
  } //DebugInfo

  protected void Errore (Exception Err) {
    Debug.outln (MioNome + "- ERRORE: "+Err);
  } //Errore

} //GestSys