/**  GestTransazione
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */


public class GestTransazione {
  /** Gestione delle transazioni verso i server "vicini" al server attuale.
   */

  // Costanti
  protected static final String STRERR_NODIREZ = "Direzione Inesistente";
  protected static final String STRERR_GIABLOC = "Direzione Gi Bloccata";
  protected static final String STRERR_NONBLOC = "Direzione Non Precedentemente Bloccata";

  // Variabili d'istanza 
  // Per mantenere gli indirizzi dei due server "vicini"
  protected Macchina[] MacchinaVicina  = new Macchina [MessSer.INDIETRO+1];

  // Per ricordarsi se le "transazioni" sono bloccate
  protected boolean[]  TransazBloccata = new boolean [MessSer.INDIETRO+1];

  // Per tenere il conto di quante transazioni sono "attive";
  protected int    QuanteTransAttive = 0;


  public GestTransazione (Macchina MacchinaLocale) {
    for (byte i=MessSer.AVANTI; i<=MessSer.INDIETRO;i++) {
      MacchinaVicina [i] = MacchinaLocale;
      TransazBloccata[i] = false;
    }
  } //costruttore


  public synchronized void defMacchinaVicina (byte IndAvInd, Macchina NuovaMak) throws Exception {
    if ((IndAvInd >= MessSer.AVANTI) && (IndAvInd <= MessSer.INDIETRO)) {
      Debug.outln ("GestTrans- macchina "+MessSer.NOME_DIR[IndAvInd]+" := "+NuovaMak);
      MacchinaVicina[IndAvInd] = NuovaMak;
    }
    else
      throw new Exception (STRERR_NODIREZ);   // o genera un'eccezione
  } //defIndirizzoVicino


  public Macchina rendiMacchinaVicina (byte Direz) throws Exception {
    if ((Direz >= MessSer.AVANTI) && (Direz <= MessSer.INDIETRO))
      return MacchinaVicina[Direz];
    else
      throw new Exception (STRERR_NODIREZ);
  } //rendiIndirizzoVicino


  public synchronized void bloccaTransazione (byte Direz) throws Exception {
    // Blocca la transazione nella direzione specificata
    // Se e' gia' bloccata, genera un'eccezione
    if ((Direz >= MessSer.AVANTI) && (Direz <= MessSer.INDIETRO)) {
      Debug.outln ("GestTrans- bloccando le transazioni in "+MessSer.NOME_DIR[Direz]);

      if (! TransazBloccata[Direz]) {

        // Pone a "true" il blocco; chiunque esegua "iniziaTransazione"
        // verra' bloccato (sospeso sul monitor di questo oggetto).
        TransazBloccata[Direz] = true;

        // Attende la fine delle transazioni ancora "attive"
        Debug.outln ("GestTrans- attendendo che si terminino i "+
                     QuanteTransAttive+" GestSer ancora attivi");
        while (QuanteTransAttive > 0) 
          try {
            Thread.currentThread().sleep (200);   // attende un po
          }
          catch (Exception E) { Debug.outln ("GestTrans- ERRORE: "+E); }
      }
      else                      // se e' gia' bloccata, genera un'eccezione
        throw new Exception (STRERR_GIABLOC);
    }
    else
      throw new Exception (STRERR_NODIREZ); 
  } //bloccaTransazione


  public synchronized void abilitaTransazione (byte Direz) throws Exception {
    // Abilita la transazione nella direzione specificata
    // Se e' gia' abilitata, genera un'eccezione
    if ((Direz >= MessSer.AVANTI) && (Direz <= MessSer.INDIETRO)) {
      Debug.outln ("GestTrans- abilitando le transazioni in "+MessSer.NOME_DIR[Direz]);

      if (TransazBloccata[Direz]) {

        // Pone a "false" il blocco;
        TransazBloccata[Direz] = false;  

        // Notifica a tutti i thread sospesi sul monitor di questo oggetto
        Debug.outln ("GestTrans- inviando 'notifica' ai thread sospesi");
        notifyAll();
      }
      else
        throw new Exception (STRERR_NONBLOC);
    }
    else
      throw new Exception (STRERR_NODIREZ);
  } //abilitaTransazione


  public boolean hoTransazioneBloccata (byte Direz) {
    if ((Direz >= MessSer.AVANTI) && (Direz <= MessSer.INDIETRO))
      return TransazBloccata[Direz];
    else
      return false;
  } //transazioniBloccate
  

  public synchronized void iniziaTransazione (byte Direz) throws Exception {
    // Servir a "GestSer" per indicare il punto di inizio di una transazione
    // (che sara' conclusa con "terminaTransazione"). Se la transazione e' bloccata
    // il thread viene sospeso su questo oggetto finche' qualcuno non chiama la
    // "abilitaTransazione" sull'oggetto stesso.
    if ((Direz >= MessSer.AVANTI) && (Direz <= MessSer.INDIETRO)) {

      Debug.outln ("GestTrans- INIZIO di una transazione  (N.Trans.Attive="+QuanteTransAttive+")");

      // Il thread e' sospeso fintanto che le transazioni non sono abilitate
      while (TransazBloccata[Direz])
        try {
          Debug.outln ("GestTrans- LE TRANSAZIONI SONO BLOCCATE-->sospendando il thread "+
                       Thread.currentThread().toString());
          wait ();     // Il thread e' sospeso sul monitor di questo oggetto!
        } 
        catch (InterruptedException E) { Debug.outln ("GestTrans- 'Wait' interrotta.."); }

      QuanteTransAttive++;   // Conta il numero di transazioni "attive"
    }
    else
      throw new Exception (STRERR_NODIREZ); 
  } //iniziaTransazione


  public synchronized void terminaTransazione () {
    // Indica la fine di una transazione di "GestSer"
    if (QuanteTransAttive > 0)
      QuanteTransAttive--;   // Tiene il conto delle transazioni "attive"

    Debug.outln ("GestTrans- FINE di una transazione  (N.Trans.Attive="+QuanteTransAttive+")");
  } //terminaTransazione


  public synchronized void AttendiSeTransazBloccate (byte Direz) throws Exception {
    // Usata da "GestCli": il thread viene bloccato finche' le transazioni non
    // sono abilitate. E' simile alla "iniziaTransazione" ma non altera il contatore
    // del numero di transazioni attive.
    if ((Direz >= MessSer.AVANTI) && (Direz <= MessSer.INDIETRO)) {

      while (TransazBloccata[Direz])
        try {
          Debug.outln ("GestTrans- LE TRANSAZIONI SONO BLOCCATE-->sospendando il thread "+
                       Thread.currentThread().toString());
          wait ();     // Il thread e' sospeso sul monitor di questo oggetto!
        } 
        catch (InterruptedException E) { Debug.outln ("GestTrans- 'Wait' interrotta.."); }
      // NB: NON conta il numero di transazioni "attive"
    }
    else
      throw new Exception (STRERR_NODIREZ); 
  } //AttendiSeTransazBloccate


  public int quanteTransAttive () {
    return QuanteTransAttive;
  } //quanteTransAttive


  public synchronized void azzeraTransAttive () {
    QuanteTransAttive = 0;
  } //azzeraTransAttive

} //GestTransazione