/**  InputSer
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.lang.Thread;
import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.net.SocketException;
import java.io.IOException;


public class InputSer extends Thread {
  /** Attende l'arrivo di messaggi da parte di un server.
   *  Usa connessione a datagrammi (UDP)
   *  Delega la gestione di ogni messaggio a un processo GestSer (ne
   *  crea uno nuovo per ogni messaggio) che esegue in parallelo.
   */

  // Variabili d'istanza:
  protected boolean        InCorsa;   // Indica se si e' richiesto di uscire
  protected DatagramSocket Sock;
  protected DatagramPacket Pacchetto;
  protected byte[]         Buff; 


  public InputSer () {
    setName ("InputSer");
    InCorsa=true;   // abilita ripetizione ciclo
    Sock=null;
    try {
 
      Debug.outln ("InputSer - creando socket-datagram");
      Sock      = new DatagramSocket (Comune.PORTA_DATAGRAM);    // Crea una nuova socket

      Buff      = new byte[Comune.DIM_PACK];                    // e alloca un array per ricevere 
    } 
    catch (IOException Err) { Errore(Err); }
  } //costruttore


  public void run() {
    if (Sock == null) 
      Debug.outln ("InputSer - ABORTITO!");             // errore creando la socket
    else {
      Debug.outln ("InputSer - lanciato!");
      MessSerDg NewMess;    // il messaggio che sara' ricevuto (E' UN MESSAGGIO DI SERVER)

      while (InCorsa) {                                 // Cicla finche' non si vuole finire

        try {
          Debug.outln ("InputSer - sono in attesa di messaggi dai server...");

          Pacchetto = new DatagramPacket (Buff, Comune.DIM_PACK);

          Sock.receive (Pacchetto);

          String DaChi = Pacchetto.getAddress().getHostAddress();
          Debug.outln ("InputSer - ricevuto Pacchetto di lunghezza "+Pacchetto.getLength()+
                       " da "+DaChi);
          Macchina MakAva = Server.GestTransaz.rendiMacchinaVicina (MessSer.AVANTI);
          Macchina MakInd = Server.GestTransaz.rendiMacchinaVicina (MessSer.INDIETRO);

          if (DaChi.equals(MakAva.cheIndirizzo()) ||              // ACCETTO IL MESSAGGIO SOLO
              DaChi.equals(MakInd.cheIndirizzo())  ) {            // SE L'HA GENERATO UN VICINO

            // PASSO DA ARRAY DI BYTE A DATA-INPUT-STREAM
            byte[] Dati = Pacchetto.getData();
            ByteArrayInputStream BAIS = new ByteArrayInputStream (Dati);
            DataInputStream DIS = new DataInputStream (BAIS);
            NewMess = (MessSerDg) TraslaMess.creaMess (DIS);
            DIS.close();
            BAIS.close();

            // CREA UN PROCESSO DI GESTIONE per il nuovo messaggio (gli passa il messaggio),
            Debug.outln ("InputSer - innescando un thread GestSer");
            GestSer MyGestSer = new GestSer (NewMess, GestSer.GEN_DA_SERVER);
            // lo innesca e torna e a accettare nuovi messaggi
            MyGestSer.start();

          }
          else
            Debug.outln ("IL PACCHETTO NON E' ARRIVATO DA UN VICINO MA DA QUALCUN ALTRO --> IGNORATO!");
        }
        catch (SocketException E) { Debug.outln ("InputSer - la Socket  stata chiusa!"); }
        catch (Exception Err)   { Errore(Err); }

      } // while - uscira' solo quando viene chiamata la "terminaEsecuzione",
        // che provvede anche a chiudere la socket SerSock.

      Debug.outln ("InputSer - TERMINATO");
    } //else
  } //run


  public void terminaEsecuzione () {
    Debug.outln ("InputSer - richiesta di terminare l'esecuzione");
    InCorsa = false;      // side-effect sul ciclo while di "run"
    if (Sock != null)
      try     {
        Debug.outln ("InputSer - sto chiudendo la socket");
        Sock.close();                             // Chiude la socket (non genera eccezione)
      } 
      catch (Exception Err) { Errore(Err); }
  } //terminaEsecuzione


  protected void Errore (Exception Err) {
    Debug.outln ("InputSer - ERRORE: "+Err);
  } //Errore

} //InputSer