/**  InputSys
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.lang.Thread;
import java.net.Socket;
import java.net.ServerSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.io.IOException;


public class InputSys extends Thread {
  /** Attende l'arrivo di messaggi di sistema da parte di un server.
   *  Usa connessione a stream (TCP)
   *  Delega la gestione di ogni messaggio a un processo GestSys (ne
   *  crea uno nuovo per ogni messaggio) che esegue in parallelo.
   */

  // Variabili d'istanza:
  protected boolean      InCorsa;   // Indica se si e' richiesto di uscire
  protected ServerSocket SerSock;


  public InputSys () {
    setName ("InputSys");
    InCorsa=true;   // abilita ripetizione ciclo
    SerSock=null;
    try {

      Debug.outln ("InputSys - creando socket-stream");
      SerSock = new ServerSocket (Comune.PORTA_STREAM_SYS, 20);  // Crea una nuova socket

    } 
    catch (IOException Err) { Errore(Err); }
  } //costruttore


  public void run() {
    if (SerSock == null) 
      Debug.outln ("InputSys - ABORTITO!");             // errore creando la socket
    else {
      Debug.outln ("InputSys - lanciato!");

      while (InCorsa) {                                 // Cicla finche' non si vuole finire

        try {
          Debug.outln ("InputSys - sono in attesa di messaggi di sistema dai server...");
          Socket SysSock = SerSock.accept();            // Connessione: crea nuova socket

          // crea un processo di gestione per la nuova connessione (gli passa la nuova socket),
          Debug.outln ("InputSys - innescando un thread GestSys");
          GestSys MyGestSys = new GestSys (SysSock);

          // lo innesca e torna e a accettare connessioni
          MyGestSys.start();
        }
        catch (SocketException E) { Debug.outln ("InputSys - la Socket  stata chiusa!"); }
        catch (IOException Err)   { Errore(Err); }

      } // while - uscira' solo quando viene chiamata la "terminaEsecuzione",
        // che provvede anche a chiudere la socket SerSock.

      Debug.outln ("InputSys - TERMINATO");
    } //else
  } //run


  public void terminaEsecuzione () {
    Debug.outln ("InputSys - richiesta di terminare l'esecuzione");
    InCorsa = false;      // side-effect sul ciclo while di "run"
    if (SerSock != null) 
      try     {
        Debug.outln ("InputSys - sto chiudendo la socket");
        SerSock.close();                          // Chiude la socket (non genera eccezione)
      } 
      catch (Exception Err) { Errore(Err); }
  } //terminaEsecuzione


  protected void Errore (Exception Err) {
    Debug.outln ("InputSys - ERRORE: "+Err);
  } //Errore

} //InputSys