/**  Macchina
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.Serializable;
import java.io.IOException;


class Macchina implements Dato, Serializable {
  /** Astrazione di una Macchina server che registra i dati
  */

  // Variabili d'istanza:
  protected String   Indirizzo;
  protected Priorita Priority;


  public Macchina () {
    // senza parametri (usato dai client)
    this (new String(""), Priorita.minPriorita);
  } //costruttore

  public Macchina (String Indirizzo, Priorita Priority) {
    this.Indirizzo = Indirizzo;
    this.Priority  = Priority;
  } //costruttore

  public Macchina (DataInputStream DIS) throws IOException {
    // rende un posto leggendo i dati da uno stream di byte
    Indirizzo = DIS.readUTF();
    Priority  = new Priorita (DIS);
  } //costruttore


  public void scriviInStream (DataOutputStream DOS) throws IOException {
    // scrive su uno stream di dati 
    DOS.writeUTF  (Indirizzo);
    Priority.scriviInStream (DOS);
  } //scriviInStream


  public String cheIndirizzo () {
    return Indirizzo;
  } //cheIndirizzo


  public Priorita chePriorita () {
    return Priority;
  } //chePriorita


  public void defPriorita (Priorita NuovaPriorita) {
    Priority = NuovaPriorita;
  } //defPriorita

  
  public boolean corretto () {
    if (Indirizzo != null) 
      return (! Indirizzo.equals(""));
    else
      return false;
  } //corretto
  
  
  public String toString() {
    // Overriding del metodo definito in "Object" per rappresentazione in stringa
    return new String ("[I="+Indirizzo+","+
                       "P="+Priority.toString()+"]");
  } //toString


  public boolean equals (Object obj) {
    // Overriding del metodo definito in "Object" per fare la ricerca nella Hashtable
    Macchina S2 = (Macchina)obj;
    return (  Indirizzo.equals( S2.Indirizzo ) );
    // NB: NON SI CONFRONTA LA PRIORITA'!
  } //equals

} //Macchina