/**  MessSerDg (figlia di Mess) da cui discendono i MessSerDgXxx
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;


abstract class MessSerDg extends MessSer {
  /** Astrazione dei messaggi SER<==>SER.
   *  Serve per realizzare l'astrazione di tutti i messaggi
   *  che possono essere scambiati tra server e server NELL'ANELLO.
   *  Racchiude tutte le funzionalita' comuni di tali messaggi.
   */

  // Costanti usabili per inizializzare un messaggio
  public final static byte VITA_DEF  = 20;   // vita di default
  public final static byte VITA_MIN  =  1;
  public final static byte VITA_ZERO =  0;

  // Variabili d'istanza:
  protected byte     Direz;    // Direzione di circolazione
  protected byte     Vita;     // Vita rimasta
  protected Macchina Mak;      // Macchina di origine del messaggio


  public MessSerDg () {
    this (AVANTI, VITA_DEF, new Macchina());
  } //costruttore

  public MessSerDg (byte Direzione) {
    this (Direzione, VITA_DEF, new Macchina());
  } //costruttore

  public MessSerDg (byte Direzione, byte VitaIniz) {
    this (Direzione, VitaIniz, new Macchina());
  } //costruttore

  public MessSerDg (byte Direzione, Macchina CheMacch) {
    this (Direzione, VITA_DEF, CheMacch);
  } //costruttore

  public MessSerDg (byte Direzione, byte VitaIniz, Macchina CheMacch) {
    Direz = Direzione;
    Vita  = VitaIniz;
    Mak   = CheMacch;
  } //costruttore

  public MessSerDg (DataInputStream DIS) throws IOException {
    Direz = DIS.readByte ();
    Vita  = DIS.readByte ();
    Mak   = new Macchina (DIS);
  } //costruttore

  
  public void scriviInStream (DataOutputStream DOS) throws IOException {
    DOS.writeByte (Direz);
    DOS.writeByte (Vita);
    Mak.scriviInStream (DOS);
  } //scriviInStream

  
  public byte cheDirez () {
    return Direz;
  } //cheDirez

  public void defDirez (byte NuovaDirez) { 
    Direz = NuovaDirez;
  } //defDirez

  public void cambiaDirez () { 
    if (Direz == AVANTI) 
      Direz = INDIETRO;
    else
      Direz = AVANTI;
  } //cambiaDirez

 
  public byte cheVita () {
    return Vita;
  } //cheVita

  public void defVita (byte NuovaVita) { 
    Vita = NuovaVita;
  } //defVita

  public void decrementaVita () {
    if (Vita > 0)
      Vita--;
  } //decrementaVita


  public Macchina cheMacchina () { 
    return Mak;
  } //cheMacchina

  public void defMacchina (Macchina cheMacchina) {
    Mak = cheMacchina;
  } //defMacchina


  public String toString () {
    return new String (getClass().getName()+"(Dir="+Byte.toString(Direz)+
                                             ",Vita="+Byte.toString(Vita)+
                                             ",Mak="+Mak.toString()+")");
  } //toString

} //MessSerDg