/**  Posto
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.Serializable;
import java.io.IOException;


class Posto implements Dato, Serializable  {
  /** Astrazione di un posto nel teatro
   */

  // Costanti 
  protected static final char FILA_MIN = 'A';
  protected static final char FILA_MAX = 'F';

  protected static final byte POSI_MIN = 1;
  protected static final byte POSI_MAX = 13;

  protected static final char CHRSEP   = '-';  // separatore

  // Variabili d'istanza:
  protected char Fila;
  protected byte Posi;


  public Posto (char Fila, byte Posi) {
    this.Fila = Fila;
    this.Posi = Posi;
  } //costruttore

  public Posto (char Fila, int Posi) {
    this.Fila = Fila;
    this.Posi = (byte) Posi;
  } //costruttore

  public Posto (DataInputStream DIS) throws IOException {
    // rende un posto leggendo i dati da uno stream di byte
    Fila = DIS.readChar();
    Posi = DIS.readByte();
  } //costruttore

  public Posto (String inStringa) throws Exception {
    // fa la conversione inversa di "rendiInStringa"
    // (legge i dati dalla stringa passata)
    if (inStringa.length() < 3)
      throw new Exception ("FORMATO ERRATO");
    Fila =  inStringa.charAt(0);
    // in posizione 1 c'e' il CHRSEP
    Posi = (Byte.valueOf( (inStringa.substring(2)) )).byteValue();
  } //costruttore


  public void scriviInStream (DataOutputStream DOS) throws IOException {
    // scrive su uno stream di dati i dati di uno spettatore
    DOS.writeChar (Fila);
    DOS.writeByte (Posi);
  } //scriviInStream


  public char rendiFila () {
    return Fila;
  } //rendiFila

  public byte rendiPosi () {
    return Posi;
  } //rendiPosi


  public String rendiInStringa () {
    // fa la conversione inversa di "Posto(String)"
    String SF = String.valueOf(Fila);
    String SP = String.valueOf((int)Posi);
    return SF + CHRSEP + SP;      // Es: "C-12"
  } //rendiInStringa


  public boolean corretto () {
    return  (Fila >= FILA_MIN) && (Fila <= FILA_MAX) &&
            (Posi >= POSI_MIN) && (Posi <= POSI_MAX);
  } //corretto


  // NOTA: per "scorrere" tutti i posti possibili si usera' un:
  //   Posto P = Posto.primoPosto();
  //   while (P.corretto()) {
  //     ..usa "P"..
  //     P.successivoPosto();
  //     (o "P=Posto.successivoPosto(P);" per una nuova istanza)
  //   }

  public static Posto primoPosto () {
    return new Posto (FILA_MIN, POSI_MIN);
  } //primoPosto

  public static Posto ultimoPosto () {
    return new Posto (FILA_MAX, POSI_MAX);
  } //ultimoPosto

  public static Posto successivoPosto (Posto P) {
    if (P.Posi < POSI_MAX) {
      byte NuoPosi =  P.Posi;
      NuoPosi++;
      return new Posto (P.Fila, NuoPosi);
    }
    else {
      char NuoFila = P.Fila;
      NuoFila++;
      return new Posto (NuoFila, POSI_MIN);
    }
  } //successivoPosto

  public void successivoPosto () {
    if (Posi < POSI_MAX)
      Posi++;
    else {
      Fila++;
      Posi = POSI_MIN;
    }
  } //successivoPosto


  public String toString() {
    // Overriding del metodo definito in "Object" per rappresentazione in stringa
    return rendiInStringa();
  } //toString

 
  public boolean equals (Object obj) {
    // Overriding del metodo definito in "Object" per fare la ricerca nella Hashtable
    Posto P2 = (Posto)obj;
    return (Fila == P2.Fila) && (Posi == P2.Posi);
  } //equals


  public int hashCode() {
    // Overriding del metodo definito in "Object" per fare la ricerca nella Hashtable
    return rendiInStringa().hashCode();
  } //hashCode

} //Posto