/**  Prenotazione (usa Spettacolo, Posto e Spettatore)
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;


class Prenotazione implements Dato {
  /** Astrazione usata per mantenere tutte le informazioni di una prenotazione.
   *  Usa le classi Spettacolo, Posto e Spettatore
   */

  // Variabili d'istanza:
  protected Spettacolo PSpettacolo;
  protected Posto      PPosto;
  protected Spettatore PSpettatore;

  
  public Prenotazione (Spettacolo Sc, Posto P, Spettatore St) {
    // Costruttore senza "macchina"
    PSpettacolo=Sc;
    PPosto     =P;
    PSpettatore=St;
  } //costruttore

  public Prenotazione (DataInputStream DIS) throws IOException {
    // rende una prenotazione leggendo i dati da uno stream di byte
    PSpettacolo= new Spettacolo (DIS);
    PPosto     = new Posto      (DIS);
    PSpettatore= new Spettatore (DIS);
  } //costruttore


  public void scriviInStream (DataOutputStream DOS) throws IOException {
    // scrive su uno stream di dati i dati della prenotazione
    PSpettacolo.scriviInStream (DOS);
    PPosto     .scriviInStream (DOS);
    PSpettatore.scriviInStream (DOS);
  } //scriviInStream


  public Spettacolo rendiSpettacolo() {
    return PSpettacolo;
  } //rendiSpettacolo
  
  public Posto rendiPosto() {
    return PPosto;
  } //rendiPosto
  
  public Spettatore rendiSpettatore () { 
    return PSpettatore; 
  } //rendiSpettatore
  

  public boolean corretto () {
    return PSpettacolo.corretto() && 
           PPosto     .corretto() &&
           PSpettatore.corretto();
  } //corretto

  
  public String toString() {
    // Overriding del metodo definito in "Object" per rappresentazione in stringa
    return new String ("SC="+PSpettacolo.toString()+
                       ",P="+PPosto.toString()+
                       ",ST="+PSpettatore.toString());
  } //toString


  public boolean equals (Object obj) {
    // Overriding del metodo definito in "Object" per fare la ricerca nella Hashtable
    Prenotazione P2 = (Prenotazione)obj;
    return ( PSpettacolo.equals( P2.PSpettacolo ) &&
             PPosto.     equals( P2.PPosto      ) &&
             PSpettatore.equals( P2.PSpettatore ) );
  } //equals


  public int hashCode() {
    // Overriding del metodo definito in "Object" per fare la ricerca nella Hashtable
    return toString().hashCode(); 
  } //hashCode

} //Prenotazione