/**  Priorita - per rappresentare la priorita' di una macchina che fa le prenotazioni
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.Serializable;
import java.io.IOException;


class Priorita implements Serializable {

  // Costanti:
  protected static final byte PRIOR_MIN = (byte) 0;
  protected static final byte PRIOR_MAX = (byte) 120;

  public static Priorita minPriorita = new Priorita (PRIOR_MIN);
  public static Priorita maxPriorita = new Priorita (PRIOR_MAX);
  // NB: queste costanti sono della stessa classe Priorita'.
  // Sono pubbliche poiche' il valore di un priorita' NON puo' essere cambiato
  // Ossia le "priorita'" sono "oggetti come VALORI" e non come "contenitori"

  // Variabile d'istanza:
  protected byte Prior;


  public Priorita (byte Valore) {
    Prior=Valore;
  } //costruttore

  public Priorita (Priorita P1, Priorita P2, boolean CEServerUnico) {
    // Crea una priorita' "equa" (di mezzo) tra le due priorita' P1 e P2.
    // Il calcolo varia a seconda che il server sia unico o meno.
    if (! CEServerUnico)                             // Se non SonoServerUnico
      if (P1.Prior > P2.Prior)                       //   Se Pmia > Psucc  allora
        Prior = (byte) ((P1.Prior + P2.Prior) / 2);  //     Pn :=( Pmia + Psucc) / 2
      else                                           //   Altrimenti
        if (P1.Prior < P2.Prior)                     //     Se Pmia < Psucc
          if (P2.Prior == PRIOR_MAX)                 //       Se Psucc = PMAX  allora
            Prior = (byte) ((P1.Prior + 0) / 2);     //      Pn := ( Pmia + 0 ) / 2
          else                                       //       Altrimenti
            Prior = PRIOR_MAX;                       //         Pn := PMAX
        else                                         //     Altrimenti
          Prior = P1.Prior;                          //       !NonPossoInserirlo!
    else                                             // Altrimenti (c un server unico)
      if (P1.Prior == PRIOR_MAX)                     //   Se Pmia = PMAX  allora
        Prior = (byte) ((P1.Prior + 0) / 2);         //     Pn := ( Pmia + 0 ) / 2
      else                                           //   Altrimenti
        Prior = PRIOR_MAX;                           //     Pn := PMAX
  } //costruttore

  public Priorita (String ValStr) {
    Prior=Byte.valueOf(ValStr).byteValue();
  } //costruttore

  public Priorita (DataInputStream DIS) throws IOException {
    // rende una priorita' leggendo i dati da uno stream di byte
    Prior = DIS.readByte ();
  } //costruttore

    
  public void scriviInStream (DataOutputStream DOS) throws IOException {
    // scrive su uno stream di dati i dati di uno spettatore
    DOS.writeByte  (Prior);
  } //scriviInStream


  public String toString () {
    return String.valueOf((int)Prior);
  } //toString


  public boolean equals (Object obj) {
    // Overriding del metodo definito in "Object" per confrontare due priorita'
    return Prior == ((Priorita)obj).Prior;
  } //equals


  public boolean greatestThan (Priorita P2) {
    return Prior > P2.Prior;
  } //greatestThan

} //Priorita