/**  Protocollo
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.net.Socket;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.InterruptedIOException;
import java.io.IOException;


class Protocollo extends Thread {
  /** Definizione di un protocollo di comunicazione su socket-stream.
   *  Questo protocollo e' usato da GestSys e da Server.
   *  Eredita da thread perche' GestSys deve esserlo.
   */

  // Costanti per la comunicazione dei messaggi
  public static final byte STATO_FATTO   = 0;
  public static final byte STATO_ERRORE  = 5;
  public static final byte STATO_TIMEOUT = 8;

  // Variabili pubbliche:
  public    static int  TIMEOUT_SYS  = 10000;  // TIME-OUT (in millisecondi)


  public Protocollo () {
    // non fa nulla, c' solo per far creare un'istanza
    // a chi vuole inviare solo un "passami".
  } //costruttore


  //--- METODI "PROTETTI" CHE ESEGUONO LA POLITICA DI GESTIONE DI OGNI MESSAGGIO ---
  // Nomenclatura: per ogni messaggio "MessSerXXX" esistono due metodi:
  //  - "Servi_MessSerXXX" per rispondere all'arrivo di un tale messaggio 
  //  - "Avvia_MessSerXXX" per iniziare il protocollo inviando tale messaggio
  // In questi metodi e' racchiusa tutta la logica del protocollo


// *** FATTO ***

  protected void Avvia_MessSerFatto (boolean Fatto, Socket Sock) {
    // Notifica "fatto S/N" al mittente (sulla socket data)
    // NB: non c'e' una conferma al "fatto" (se no avrei un protocollo infinito)
    MessSerFatto Msg = new MessSerFatto (Fatto);
    inviaMessaggio (Msg, Sock);
  } //Avvia_MessSerFatto


  protected boolean Servi_MessSerFatto (Socket Sock) {
    // Legge dalla socket un messaggio di classe "MessSerFatto";
    // Se il mess. e' di tale classe, rende il booleano contenuto in esso;
    // Altrimenti rende "false" (mess. non di tale classe)
    try {
      MessSer Msg = riceviMessaggio (Sock);        // legge risposta
      if (Msg instanceof MessSerFatto)
        return ((MessSerFatto)Msg).cheFatto();
      else
        return false;                              // Messaggio errato
    } 
    catch (Exception E) { return false; }
  } //Servi_MessSerFatto


// *** BLOCCA/SBLOCCA ***

  protected byte Avvia_MessSerBloccaSblocca (boolean Blocca, byte Direz, Socket Sock) {
    // Invia il messaggio Blocca o Sblocca al server su Sock e attende Fatto S/N
    if (Blocca) return Avvia_MessSerBlocca (Direz, Sock);
    else        return Avvia_MessSerSblocca(Direz, Sock);
  } //Avvia_MessSerBloccaSblocca

  protected byte Avvia_MessSerBlocca (byte Direz, Socket Sock) {
    // Invia il messaggio Blocca al server su Sock e attende Fatto S/N
    return inviaMessConRispTimeOut (new MessSerBlocca(Direz), Sock);
  } //Avvia_MessSerBlocca

  protected byte Avvia_MessSerSblocca (byte Direz, Socket Sock) {
    // Invia il messaggio Sblocca al server su Sock e attende Fatto S/N
    return inviaMessConRispTimeOut (new MessSerSblocca(Direz), Sock);
  } //Avvia_MessSerSblocca


  protected byte Servi_MessSerBloccaSblocca (MessSerBlocca MyMess, Socket Sock) {
    // Serve il messaggio Blocca/Sblocca e invia Fatto S/N su Sock
    // NOTA: side-effect su "Server.GestTransaz"
    boolean Fatto;
    // Leggo la direzione da bloccare/sbloccare
    byte Direz = MyMess.cheDirUsare();

    try {
      if (MyMess instanceof MessSerSblocca) {    // prima "sblocca", perche' sottoclasse
        // Provo a abilitare la transazione, se non riesce genera eccezione
        Server.GestTransaz.abilitaTransazione (Direz);
        Fatto = true;
      }
      else { // ho MessSerBlocca
        // Provo a bloccare la transazione, se non riesce genera eccezione
        Server.GestTransaz.bloccaTransazione (Direz);
        Fatto = true;
      }
    }
    catch (Exception E) {
      // SE NON RIESCE, LE TRANSAZ. SONO GIA' STATE S/BLOCCATE DA QUALCUN ALTRO
      Fatto = false;
      Errore (E);
    }

    // Notifica "fatto S/N" al mittente (del messaggio originale)
    Avvia_MessSerFatto (Fatto, Sock);
    if (Fatto) return STATO_FATTO;
    else       return STATO_ERRORE;
  } //Servi_MessSerBloccaSblocca


// *** CAMBIA-SERVER ***

  protected byte Avvia_MessSerCambSer (byte DirDaCamb, Macchina NuovoSer, Socket Sock) {
    // Invia il messaggio Cambia-server al server su Sock e attende Fatto S/N
    // Leggo la direzione da bloccare/sbloccare
    MessSerCambSer Msg = new MessSerCambSer (DirDaCamb, NuovoSer);
    return inviaMessConRispTimeOut (Msg, Sock);     // invia il messaggio
  } //Avvia_MessSerCambSer


  protected void Servi_MessSerCambSer (MessSerCambSer MyMess, Socket Sock) {
    // Serve il messaggio Cambia-server e invia Fatto S/N su Sock
    // NOTA: side-effect su "Server.GestTransaz"
    boolean Fatto;
    // Leggo i dati dal messaggio
    Macchina NuovoSer = MyMess.cheNuovaMak();
    byte     InCheDir = MyMess.cheDirCambiare();

    try {
      Server.GestTransaz.defMacchinaVicina (InCheDir, NuovoSer);
      Fatto = true;
    } 
    catch (Exception E) { 
      Fatto = false;
      Errore(E); 
    }
    // Notifica "fatto S/N" al mittente (del messaggio originale)
    Avvia_MessSerFatto (Fatto, Sock);
  } //Servi_MessSerCambSer


// *** AGGIUNGI-ME ***

  protected byte Avvio_MessSerAggMe (Socket SockEsist) {
    // Invia il messaggio Aggiungi-me al server su Sock e esegue l'intero
    // protocollo di scambio di dati per agganciare il server corrente alla catena
    // Side-effect su: Server.IDataBase,
    // E usa "Server.MaMa" e "Server.SerAggiunta" per il server locale e  quello
    // a cui si sta richiedendo di aggiungersi.
    byte Stato;
    MessSer Msg;

    // SNuovo==>SEsist - INVIA LA RICHIESTA NEL MESSAGGIO "MessSerAggMe" (e attende "fatto")
    MessSerAggMe MessAgg = new MessSerAggMe (Server.MaMa.cheIndirizzo());
    Stato = inviaMessConRispTimeOut (MessAgg, SockEsist);   // invia messaggio

    if (Stato == STATO_FATTO) {           // Se fatto, arriva l'oggetto "DataBase"
      try {
        DebugInfo ("Protocollo UNICO di Inserimento Server");

        // SNuovo<==SEsist - RICEVE L'INTERO DATA-BASE DAL SERVER (su MySock)
        DebugInfo ("ricevendo l'oggetto DataBase");
        ObjectInputStream OIS = new ObjectInputStream (SockEsist.getInputStream());
        Server.IDataBase = (DataBase) OIS.readObject ();
        // NOTA: la "IDataBase.defDefaultPriority()" e' fatta alla fine; 
        // ora non conosco ancora la mia priorita'!
        Debug.out (Server.IDataBase.toString("[DataBase]"));

        // SNuovo<==SEsist - SI ASPETTA UN "MessSerCambSer"
        Msg = riceviMessaggio (SockEsist);         // legge nuovo messaggio
        if (Msg instanceof MessSerCambSer)         // Mi aspettavo un "cambia server"
          Servi_MessSerCambSer ((MessSerCambSer) Msg, SockEsist);
        else                                       // Altrim, puo' arrivare un "fatto-no"
          throw new Exception ("Non e' arrivato il primo CambiaSer");

        // NON POSSO FARE
        //   Server.GestTransaz.defMacchinaVicina (MessSer.INDIETRO, Server.SerAggiunta);
        // perche' non conosco la priorita' del server a cui ho fatto la richiesta!
        // MI DEVO ASPETTARE UN ALTRO "MessSerCambSer"
        // SNuovo<==SEsist - SI ASPETTA UN "MessSerCambSer"
        Msg = riceviMessaggio (SockEsist);         // legge nuovo messaggio
        if (Msg instanceof MessSerCambSer)         // Mi aspettavo un "cambia server"
          Servi_MessSerCambSer ((MessSerCambSer) Msg, SockEsist);
        else                                       // Altrim, puo' arrivare un "fatto-no"
          throw new Exception ("Non e' arrivato il secondo CambiaSer");

        // CONOSCENDO I DUE SERVER VICINI E LA LORO PRIORITA', MI POSSO CALCOLARE LA MIA!
        Priorita MiaPrior;
        try {
        	 Macchina MioAvanti   = Server.GestTransaz.rendiMacchinaVicina (MessSer.AVANTI);
          Macchina MioIndietro = Server.GestTransaz.rendiMacchinaVicina (MessSer.INDIETRO);
          MiaPrior = new Priorita (MioIndietro.chePriorita(),
                                   MioAvanti.chePriorita(),
                                   Server.siamoSoloInDue());   // NB: tra indietro e avanti!
          // NOTA: se "siamoSoloInDue" vuol dire che il server di prima era "daSolo".
          DebugInfo ("calcolo della mia priorit: PIn="+MioIndietro.chePriorita()+
                     " PAv="+MioAvanti.chePriorita()+" Solo="+Server.siamoSoloInDue()+
                     " ==> PMia="+MiaPrior);
        }
        catch (Exception E) { MiaPrior = Priorita.minPriorita; }
        Server.MaMa.defPriorita (MiaPrior);                    // modifica la propria Priorita
        Server.IDataBase.defDefaultPriority (MiaPrior);        // E NE INFORMA IL DATA BASE

        // SNuovo<==SEsist - SI ASPETTA UN "MessSerFatto"
        Msg = riceviMessaggio (SockEsist);         // legge nuovo messaggio
        if (Msg instanceof MessSerFatto) {         // Mi aspettavo un "fatto"
          // Nota: le transazioni sono gia' abilitate! Il fatto e' che NON CI SONO 
          // ANCORA i processi che gestiscono l'arrivo dei messaggi.
          // Questo messaggio mi serve per sapere che tutto e' andato bene.
        }
        else
          throw new Exception ("Non e' arrivato un Fatto");
      } 
      catch (Exception E) { 
        Errore (E);
        try {
          Server.GestTransaz.defMacchinaVicina (MessSer.AVANTI,   Server.MaMa);
          Server.GestTransaz.defMacchinaVicina (MessSer.INDIETRO, Server.MaMa);
        }
        catch (Exception E2) { }
        Stato = STATO_ERRORE;
      }
      DebugInfo ("Aggiunta Server Terminata");
    }
    else
      DebugInfo ("Aggiunta Server ABORTITA");
    return Stato;
  } //Avvio_MessSerAggMe


  protected void Servi_MessSerAggMe (MessSerAggMe MsqAM, Socket SockNuovo) {
    // Leggo il nome del nuovo server da aggiungere
    // schema:  SNuovo <==> SIo <==> SAvanti
    byte Stato;
    try {
      // NuovoSer  = indirizzo del nuovo server da aggiungere
      // MioAvanti = indirizzo del mio corrente "avanti"
    	 Macchina MioAvanti = Server.GestTransaz.rendiMacchinaVicina (MessSer.AVANTI);
      String   NuoInd    = MsqAM.cheIndirizzo();  // legge dal messaggio il nuovo server
      // CALCOLA LA PROBABILITA' DI MEZZO (TRA IL LOCALE E L'AVANTI), PER IL NUOVO SERVER
      Priorita NuoPri    = new Priorita (Server.MaMa.chePriorita(), 
                                         MioAvanti.chePriorita(),
                                         Server.sonoDaSolo());
      Macchina NuovoSer  = new Macchina (NuoInd, NuoPri);    // ECCO IL NUOVO SERVER
      DebugInfo ("calcolo della priorit del nuovo server: PMia="+Server.MaMa.chePriorita()+
                     " PAv="+MioAvanti.chePriorita()+" Solo="+Server.sonoDaSolo()+
                     " ==> MNuo="+NuoPri);

      // CONTROLLO SE SI PUO' ATTRIBUIRE UNA PRIORITA' DISTINTA AL NUOVO 
      // SERVER, SE NON PUO' NOTIFICA UN "FATTO-NO"
      if ( NuoPri.equals(Server.MaMa.chePriorita()) || 
           NuoPri.equals(MioAvanti.chePriorita())     ) 
        throw new Exception ("IMPOSSIBILE ASSEGNARE UNA PRIORITA' DISTINTA AL NUOVO SERVER");
                // Notifichera' un "fatto NO'" al nuovo server (che quindi attender il DB)

      // Controllo se sono un server "solo" ("avanti" punta a se stesso)
      boolean Ridotto = Server.sonoDaSolo();

      if (Ridotto) DebugInfo ("Protocollo RIDOTTO di Inserimento Server");
      else         DebugInfo ("Protocollo GENERALE di Inserimento Server");

      // SIo - BLOCCA LE PROPRIE TRANSAZIONI IN AVANTI
      // SE HO LE TRANSAZIONI IN AVANTI GIA' BLOCCATE, NON POSSO PROCEDERE
      // (SIGNIFICA CHE STA FACENDO QUALCOSA QUALCUN ALTRO)
      // Nota: se non riesce a bloccarle, generera' un'eccezione
      Server.GestTransaz.bloccaTransazione (MessSer.AVANTI);

      try {

        // SIo==>SAvanti - ORDINA AL SERVER AVANTI DI CHIUDERE LE TRANSAZIONI INDIETRO
        if (Ridotto)    // Ridotto: blocco le MIE trans. indietro! (se non riesce, eccezione)
          Server.GestTransaz.bloccaTransazione (MessSer.INDIETRO);

        else {
          // MI COLLEGO AL MIO "AVANTI"
          // (NB: USO IL SUO INDIRIZZO, NON LA "DIREZIONE" POICHE' SI CAMBIA L'INDIRIZZO!)
          Socket SockAvanti;         // NB: inivio di singoli messaggi su nuova socket
          try {                      // Controllo che l'"avanti ci sia veramente"
            SockAvanti = creaConnessioneDiretta (MioAvanti);
          }
          catch (Exception E) {        // Se non riesco a connettermici,
            Avvio_MessSerPassami ();   // LANCIO PROTOCOLLO DI "PASSAMI"+"RIAGGANCIO"
            throw E;                   // Rilancio l'eccezione
          }
          Stato = Avvia_MessSerBlocca (MessSer.INDIETRO, SockAvanti);
          SockAvanti.close();
          if (Stato != STATO_FATTO) throw new Exception ("L'AVANTI HA RESO 'NO' AL BLOCCO");
        }

        // Notifica "fatto si'" al nuovo server (che quindi attender il DB)
        Avvia_MessSerFatto (true, SockNuovo);

        try {

          // SNuovo<==SIo - TRASFERISCE L'INTERO DATA-BASE AL NUOVO SERVER
          DebugInfo ("trasferendo l'oggetto DataBase su socket-stream");
          ObjectOutputStream OOS = new ObjectOutputStream (SockNuovo.getOutputStream());
          OOS.writeObject (Server.IDataBase);
          OOS.flush();
          Debug.out (Server.IDataBase.toString("[DataBase]"));

          // SNuovo<==SIo - TRASFERISCE AL NUOVO SERVER L'INDIRIZZO DEL PROPRIO AVANTI
          Stato = Avvia_MessSerCambSer (MessSer.AVANTI, MioAvanti, SockNuovo);
          if (Stato != STATO_FATTO) throw new Exception ("IL NUOVO NON VUOLE CAMBIARE L'AVANTI");

          // SNuovo<==SIo - TRASFERISCE AL NUOVO SERVER IL PROPRIO INDIRIZZO (CON LA PRIORITA')
          Stato = Avvia_MessSerCambSer (MessSer.INDIETRO, Server.MaMa, SockNuovo);
          if (Stato != STATO_FATTO) throw new Exception ("IL NUOVO NON VUOLE CAMBIARE L'INDIETRO");

          // SIo==>SAvanti - TRASFERISCE AL SERVER AVANTI L'INDIRIZZO DEL NUOVO SERVER
          if (Ridotto)   // Se sono solo, dirotto il MIO "indietro" a puntare al nuovo server
            Server.GestTransaz.defMacchinaVicina (MessSer.INDIETRO, NuovoSer);
          else {   // NB: MI DEVO RICOLLEGARE CON L'AVANTI!
            Socket SockAvanti = creaConnessioneDiretta (MioAvanti);
            Stato = Avvia_MessSerCambSer (MessSer.INDIETRO, NuovoSer, SockAvanti);
            SockAvanti.close();
            if (Stato != STATO_FATTO) throw new Exception ("L'AVANTI NON VUOLE CAMBIARE L'INDIETRO");
          }

          try {

            // SIo - MODIFICA IL PROPRIO AVANTI CON L'INDIRIZZO DEL NUOVO SERVER
            Server.GestTransaz.defMacchinaVicina (MessSer.AVANTI, NuovoSer);

            // SNuovo<==SIo  - DICE CHE TUTTO E' ANDATO BENE
            Avvia_MessSerFatto (true, SockNuovo);

            DebugInfo ("Aggiunta Server Completata con Successo");

          }
          catch (Exception Err) { // Errore cercando di cambiare il proprio ind.vicino

            // SIo==>SAvanti - TRASFERISCE AL SERVER AVANTI IL VECCHIO INDIRIZZO (IO)
            // (sto ANNULLANDO la modifica precedente per riportare il sistema all'inizio)
            try {
              Socket SockAvanti = creaConnessioneDiretta (MioAvanti);
              Avvia_MessSerCambSer (MessSer.INDIETRO, Server.MaMa, SockAvanti);
              SockAvanti.close();
            }
            catch (Exception Er) { Errore (Er); }
            
            // Notifica al NUOVO server: NON FATTO!
            Avvia_MessSerFatto (false, SockNuovo);  
            
            Errore (Err);
            DebugInfo ("Aggiunta Server ABORTITA");
          }

        }
        catch (Exception Err) { // Errore cercando di inviare il DB o l'indirizzo nuovo avanti
          Errore (Err);
          DebugInfo ("Aggiunta Server ABORTITA");
        }

        // SIo==>SAvanti - ORDINA DI RIABILITARE LE TRANSAZIONI
        if (Ridotto)
          Server.GestTransaz.abilitaTransazione (MessSer.INDIETRO);
        else
          try {   // NB: inivio di singoli messaggi su nuova socket
            Socket SockAvanti = creaConnessioneDiretta (MioAvanti);
            Avvia_MessSerSblocca (MessSer.INDIETRO, SockAvanti);
            SockAvanti.close();
          }
          catch (Exception E) { Errore (E); }

      }
      catch (Exception Err) { // Errore cercando di bloccare le transazioni dell'AVANTI
        Errore (Err);
        Avvia_MessSerFatto (false, SockNuovo);  // Notifica al NUOVO server: NON FATTO!
        DebugInfo ("Aggiunta Server ABORTITA");
      }

      // RIABILITO LE MIE TRANSAZIONI IN AVANTI
      Server.GestTransaz.abilitaTransazione (MessSer.AVANTI);
      DebugInfo ("Aggiunta Server Terminata");

    }
    catch (Exception E) {  // Errore cercando di bloccare le MIE transazioni
      Errore (E);
      Avvia_MessSerFatto (false, SockNuovo);  // Notifica al NUOVO server: NON FATTO!
      DebugInfo ("Aggiunta Server ABORTITA");
    }
  } //Servi_MessSerAggMe


// *** TOGLI-ME ***

  protected byte Avvio_MessSerTogMe (Socket SockAvanti, Socket SockIndietro) {
    // Invia il messaggio Togli-me ai server avanti e indietro, rispettivamente
    // su SockAvanti e SockIndietro, e esegue l'intero protocollo di scambio di
    // dati per sganciare il server corrente dalla catena
    // Schema:  SInditro <== SCorr ==> SAvanti
    byte     Stato;
    MessSer  Msg;
  	 Macchina MioAvanti;
    Macchina MioIndietro;
    try {

       // Provo a bloccare le transazioni in ambo i sensi, se non riesce genera eccezione
      Server.GestTransaz.bloccaTransazione (MessSer.AVANTI);
      Server.GestTransaz.bloccaTransazione (MessSer.INDIETRO);

      MioAvanti   = Server.GestTransaz.rendiMacchinaVicina (MessSer.AVANTI);
      MioIndietro = Server.GestTransaz.rendiMacchinaVicina (MessSer.INDIETRO);
    }
    catch (Exception E) { 
      return STATO_ERRORE;
    }

    // SCorr==>SAvanti - INVIA LA RICHIESTA NEL MESSAGGIO "MessSerTogMe"
    MessSerTogMe MessTog = new MessSerTogMe ();

    if (Server.siamoSoloInDue()) {         // Attenzione: POSSO AVERE SOLO 2 SERVER!
      DebugInfo ("Protocollo RIDOTTO di Rimozione Server");
      // Protocollo ridotto, ristretto a due soli server.
      Stato = inviaMessConRispTimeOut (MessTog, SockAvanti);       // invia messaggio avanti
      // NON invia il mess. all'indietro

      if (Stato == STATO_FATTO) {
        Stato = Avvia_MessSerBlocca  (MessSer.INDIETRO, SockAvanti);

        if (Stato == STATO_FATTO) {
          Stato = Avvia_MessSerCambSer (MessSer.INDIETRO, MioIndietro, SockAvanti);

          Avvia_MessSerSblocca (MessSer.INDIETRO, SockAvanti);
          // MioIndietro e' uguale a MioAvanti (e l'altro server lo sa)
        }
      }
    }
    else {
      // Protocollo generale, usabile in un anello con tre server o piu'.
      DebugInfo ("Protocollo GENERALE di Rimozione Server");
      Stato = inviaMessConRispTimeOut (MessTog, SockAvanti);       // invia messaggio avanti
      if (Stato == STATO_FATTO)
        Stato = inviaMessConRispTimeOut (MessTog, SockIndietro);   // e indietro (se serve)

      if (Stato == STATO_FATTO) {
        Stato = Avvia_MessSerBlocca  (MessSer.INDIETRO, SockAvanti);
        if (Stato == STATO_FATTO) {
          Stato = Avvia_MessSerBlocca  (MessSer.AVANTI, SockIndietro);
          
          if (Stato == STATO_FATTO) {

            Stato = Avvia_MessSerCambSer (MessSer.INDIETRO, MioIndietro, SockAvanti);

            if (Stato == STATO_FATTO) {

              Stato = Avvia_MessSerCambSer (MessSer.AVANTI,   MioAvanti,   SockIndietro);

              if (Stato == STATO_FATTO) {
                try {
                  // I riferimenti all'avanti/indietro devono puntare a se stessi
                  Server.GestTransaz.defMacchinaVicina (MessSer.AVANTI,   Server.MaMa);
                  Server.GestTransaz.defMacchinaVicina (MessSer.INDIETRO, Server.MaMa);
                }
                catch (Exception E) { Errore(E); }

                Avvia_MessSerSblocca  (MessSer.AVANTI, SockIndietro);
                Avvia_MessSerSblocca (MessSer.INDIETRO, SockAvanti);
                // NB: non le "annullo" perche' in caso di "abort",
                // la controparte riabilita da se' le sue transazioni
              }
              else {
                Avvia_MessSerCambSer (MessSer.INDIETRO, Server.MaMa, SockAvanti); // ANNULLA IL CAMBIA!
                Avvia_MessSerFatto (false, SockIndietro);   // Notifica un "fatto=no"
              }
            }
          }
        }
      }
    }
    try {
      Server.GestTransaz.abilitaTransazione (MessSer.AVANTI);
      Server.GestTransaz.abilitaTransazione (MessSer.INDIETRO);
    }
    catch (Exception E) {
      Stato = STATO_ERRORE;
    }

    DebugInfo ("Rimozione Server Terminata");
    return Stato;
  } //Avvio_MessSerTogMe


  protected void Servi_MessSerTogMe (MessSerTogMe MsqTM, Socket SockKill) {
    // Leggo il nome del nuovo server da aggiungere
    // Schema:  SIO <== SKill
    byte     Stato;
    MessSer  Msg;

    // Notifica "fatto si'" al nuovo server (che quindi attender il DB)
    Avvia_MessSerFatto (true, SockKill);

    if (Server.siamoSoloInDue()) {                 // Attenzione: POSSO AVERE SOLO 2 SERVER!
      DebugInfo ("Protocollo RIDOTTO di Rimozione Server");
      // Protocollo ridotto, ristretto a DUE soli server.
      try {
        Msg = riceviMessaggio (SockKill);          // legge nuovo messaggio
        if (! (Msg instanceof MessSerBlocca))      // Mi aspettavo un "blocca"
          throw new Exception ("Non e' arrivato un Blocca");

        byte Dir = ((MessSerBlocca)Msg).cheDirUsare();
        try {
          Server.GestTransaz.bloccaTransazione (MessSer.cambiaDirez(Dir));
        }
        catch (Exception E) {
          Avvia_MessSerFatto (false, SockKill);  // Notifica un "fatto=no"
          throw new Exception ("Impossibile Eseguire il MessSerBlocca sulla prima dir.");
        }
        Stato = Servi_MessSerBloccaSblocca ((MessSerBlocca)Msg, SockKill);
        if (Stato != STATO_FATTO) {
          try {
            // Riabilita le transazioni nella direzione bloccata precedentemente
            Server.GestTransaz.abilitaTransazione (MessSer.cambiaDirez(Dir));
          }
          catch (Exception Er) { }
          throw new Exception ("Impossibile Eseguire il MessSerBlocca sulla seconda dir.");
        }

        try {
          Msg = riceviMessaggio (SockKill);          // legge nuovo messaggio
          if (! (Msg instanceof MessSerCambSer))     // Mi aspettavo un "cambia"
            throw new Exception ("Non e' arrivato un CambSer");

          Dir = ((MessSerCambSer)Msg).cheDirCambiare();
          Macchina MakNuova = ((MessSerCambSer)Msg).cheNuovaMak();
          Server.GestTransaz.defMacchinaVicina (MessSer.cambiaDirez(Dir), MakNuova);
          Servi_MessSerCambSer ((MessSerCambSer) Msg, SockKill);

          Msg = riceviMessaggio (SockKill);          // legge nuovo messaggio
          if (Msg instanceof MessSerCambSer) {       // Puo' ritornare un "CambiaSer" per annullare
            Servi_MessSerCambSer ((MessSerCambSer) Msg, SockKill);
            throw new Exception ("E' Arrivato un Secondo MessSerCambSer --> abort"); // ABORT
          }
          if (! (Msg instanceof MessSerSblocca))     // Mi aspettavo uno "sblocca"
            throw new Exception ("Non e' arrivato uno Sblocca");

          Dir = ((MessSerSblocca)Msg).cheDirUsare();
          Server.GestTransaz.abilitaTransazione  (MessSer.cambiaDirez(Dir));
          Servi_MessSerBloccaSblocca ((MessSerSblocca) Msg, SockKill);
        }
        catch (Exception E2) {
          try {
            Server.GestTransaz.abilitaTransazione (MessSer.AVANTI);   // Riabilita le transazioni
            Server.GestTransaz.abilitaTransazione (MessSer.INDIETRO); // (se le ho bloccate io)
          }
          catch (Exception E3) { }
          throw E2;
        }
      }
      catch (Exception E) { 
        Errore (E);
      }
    }
    else {
      DebugInfo ("Protocollo GENERALE di Rimozione Server");
      // Protocollo generale, usabile in un anello con tre server o piu'.
      try {
        Msg = riceviMessaggio (SockKill);          // legge nuovo messaggio
        if (! (Msg instanceof MessSerBlocca))      // Mi aspettavo un "blocca"
          throw new Exception ("Non e' arrivato un Blocca");

        byte DirBloc = ((MessSerBlocca)Msg).cheDirUsare();
        Stato = Servi_MessSerBloccaSblocca ((MessSerBlocca) Msg, SockKill);
        if (Stato != STATO_FATTO)
          throw new Exception ("Impossibile Eseguire il MessSerBlocca");

        try {
          Msg = riceviMessaggio (SockKill);          // legge nuovo messaggio
          if (! (Msg instanceof MessSerCambSer))      // Mi aspettavo un "cambia"
            throw new Exception ("Non e' arrivato un CambSer");

          Servi_MessSerCambSer ((MessSerCambSer) Msg, SockKill);

          Msg = riceviMessaggio (SockKill);          // legge nuovo messaggio
          if (! (Msg instanceof MessSerSblocca))     // Mi aspettavo uno "sblocca"
            throw new Exception ("Non e' arrivato uno Sblocca");

          Servi_MessSerBloccaSblocca ((MessSerSblocca) Msg, SockKill);

        }
        catch (Exception E2) {
          try {
            Server.GestTransaz.abilitaTransazione (DirBloc); // Ripristina la direz.bloccata
          }                                                  // (ma solo se l'ho bloccata)
          catch (Exception E3) { }
          throw E2;
        }
      }
      catch (Exception E) { Errore (E); }
    }
    DebugInfo ("Rimozione Server Terminata");
  } //Servi_MessSerTogMe


// *** PASSAMI ***

  public void Avvio_MessSerPassami () {
    // PUBBLICA!!! 
    this.Avvio_MessSerPassami ("");
  } //Avvio_MessSerPassami

  public void Avvio_MessSerPassami (String FrasePopUp) {
    // PUBBLICA!!!
    // Invia il messaggio Passami al server AVANTI;
    // Non si attende la ricezione di un "fatto";
    // NON C'E' UNA SOCKET IN INGRESSO, PERCHE' E' CREATA ALL'INTERNO DEL METODO
    // Se non si riesce a connettersi, la "CONNECT" rende un'eccezione (e vuol 
    // dire che il server su cui si e' tentata la Sock e' caduto)

    MessSerPassami Msg = new MessSerPassami (null, FrasePopUp);

    // Nota: le azioni ri riaggancio sono fatte nella "Servi"; percio' fingo che
    // il messaggio sia appena arrivato, MA HA INDIRIZZO VUOTO (verr sostituito
    // col proprio indirizzo locale).
    Servi_MessSerPassami (Msg);

  } //Avvio_MessSerPassami


  protected void Servi_MessSerPassami (MessSerPassami MyMess) {
    // Serve il messaggio Passami; se la macchina locale non e' quella che
    // ha generato il messaggio, si connette al successore e glielo passa.
    // Se non riesce a fare la "connect" (si ha un'eccezione, il server e'
    // caduto!) si procede a iniziare il protocollo di "riaggancio" dell'anello.
    // NB: non usa una "Socket Sock" in ingresso (non rende il "fatto")
    boolean Fatto;
    // Leggo indirizzo macchina generatrice del messaggio
    Macchina MakGener  = MyMess.cheMakGener();
    Macchina MioAvanti;
    try {
    	 MioAvanti = Server.GestTransaz.rendiMacchinaVicina (MessSer.AVANTI);
    }
    catch (Exception E) { MioAvanti = Server.MaMa; }

    if (Server.sonoDaSolo()) {                       // SONO UN SERVER "SOLO"?
      DebugInfo ("Sono un SERVER UNICO e sono VIVO!");

      return;  // EXIT (non passo piu' in avanti il messaggio)!!!
    }
    else
      if (MakGener == null) {    // L'ho APPENA GENERATO io?
                                 // (l'ind. nullo e' inserito da "AvviaXx")
        // Ricrea di nuovo il messaggio, ma col proprio indirizzo da passare
        MakGener  = Server.MaMa;
        MyMess = new MessSerPassami (MakGener, MyMess.cheFrasePopUp());

      }
      else  // il messaggio e' arrivato da qualcuno
        if (MakGener.equals(Server.MaMa)) {          // L'AVEVO GENERATO IO?
          // fine del giro del messaggio
          DebugInfo (" ritornato il MessSerPassami che avevo generato io: ANELLO FUNZIONANTE!");

          return;  // EXIT (non passo piu' in avanti il messaggio)!!!
        }
        else {
        
          // UN MESSAGGIO NON MIO: eventualmente apre la finestra di pop-up
          String FrasePopUp = MyMess.cheFrasePopUp(); 
          if (! FrasePopUp.equals("")) {
            ServerDisplay Dsp = Server.rendiDisplay();
            Dsp.apriDialogoLibero (Dialogo.BOTT_OK,
                                   "MESSAGGIO DA "+MakGener.cheIndirizzo()+":", FrasePopUp);
          }
        }
        
      // Se sono qui, ho appena generato il messaggio o e'
      // arrivato da qualcuno ma non l'avevo generato io 

      try { 
        // Prova a creare una connessione col server in AVANTI
        // Se non riesce, viene generata un'eccezione!
        Socket SockAvanti = creaConnessione (MessSer.AVANTI);

        // Se sono qui, tutto ok --> passa in avanti lo stesso messaggio
        inviaMessaggio (MyMess, SockAvanti);

        SockAvanti.close();      // chiusura socket aperta qui
      }
      catch (Exception Err) {

        // ERRORE: IL SERVER "AVANTI" E' CADUTO ==> AVVIA IL PROTOCOLLO DI RIAGGANCIO!!!
        DebugInfo ("IL MIO SERVER AVANTI E' CADUTO! Si Inizia il protocollo di RIAGGANCIO!!!");
        try {
          try {
            // BLOCCA le transazioni in AVANTI (non arriverebbero mai a destinazione)
            Server.GestTransaz.bloccaTransazione (MessSer.AVANTI);
          }
          catch (Exception E2) { Errore (E2); }

          Macchina MioIndietro = Server.GestTransaz.rendiMacchinaVicina (MessSer.INDIETRO);

          if (MioAvanti.equals(MioIndietro)) {

            // Protocollo ridotto: ho solo due server di cui l'altro e' caduto!
            // BLOCCA anche le transazioni INDIETRO
            Server.GestTransaz.bloccaTransazione (MessSer.INDIETRO);

            // Aggiorna riferimenti verso se stesso!
            Server.GestTransaz.defMacchinaVicina (MessSer.AVANTI,   Server.MaMa);
            Server.GestTransaz.defMacchinaVicina (MessSer.INDIETRO, Server.MaMa);

            // Riabilita le transazioni
            Server.GestTransaz.abilitaTransazione (MessSer.INDIETRO);
            Server.GestTransaz.abilitaTransazione (MessSer.AVANTI);
          }
          else {

            // Protocollo generale: ho almento tre server (di cui uno e' caduto)
            // Crea una connessione col server INDIETRO E DICE CHE L'AVANTI E' CADUTO
            Socket SockIndietro = creaConnessione (MessSer.INDIETRO);

            // INVIA ALL'INDIETRO UN MESSAGGIO DI "RIAGGANCIO"!!!!!!
            // "MioAvanti" e' il server che e' caduto!
            MessSerRiaggancio Msg = new MessSerRiaggancio (MioAvanti, Server.MaMa);

            inviaMessaggio (Msg, SockIndietro);
            // Una volta a destinazione, questo messaggio far generare un messaggio
            // MessSerRiaggConMe che torner a questo server!
            // Sar ALLORA che si riabiliteranno le transazioni.

            SockIndietro.close();
          }
        }
        catch (Exception E) { Errore (E); }
      } //catch
  } //Servi_MessSerPassami


// *** RIAGGANCIO & RIAGGANCIO-CON-ME ***

  // NB: non ho le "Avvia_Xxx" per questi messaggi poiche' possono essere avviati
  // rispettivamente dai metodi "Servi_MessSerPassami" e "Servi_MessSerRiaggancio".


  protected void Servi_MessSerRiaggancio (MessSerRiaggancio MyMess) {
    // Serve il messaggio Riaggancio (che sta viaggiando INDIETRO);
    // Se il nome del server INDIETRO  uguale a quello contenuto nel campo 
    // "NomeCaduto" del messaggio, si invia al server il cui nome e' nel
    // campo NomePrec un messaggio di MessSerRiaggConMe.
    // Altrimenti il messaggio e' passato al server INDIETRO.
    // NB: non usa una "Socket Sock" in ingresso (non rende il "fatto")
    // NB: l'"AVVIA" e' fatta SOLO dal metodo "Servi_MessSerPassami"!
    try {
      
      Macchina MakCaduto   = MyMess.cheMakCaduto();
      Macchina MioIndietro = Server.GestTransaz.rendiMacchinaVicina (MessSer.INDIETRO);

      if (MakCaduto.equals(MioIndietro)) {
        // Il mio server "indietro" e' quello che e' stato riconosciuto come caduto!
        DebugInfo ("riconosciuto server Indietro = server Caduto --> invio di RiaggancioConMe");

        // BLOCCA anche le transazioni INDIETRO
        Server.GestTransaz.bloccaTransazione (MessSer.INDIETRO);

        Macchina MakGener  = MyMess.cheMakGener();

        // Si connette DIRETTAMENTE AL SERVER CHE HA GENERATO IL MESSAGGIO
        Socket SockDir = creaConnessioneDiretta (MakGener);

        MessSerRiaggConMe Msg = new MessSerRiaggConMe (Server.MaMa);
        byte Stato = inviaMessConRispTimeOut (Msg, SockDir);

        if (Stato == STATO_FATTO)
          Server.GestTransaz.defMacchinaVicina (MessSer.INDIETRO, MakGener);

        try {
          SockDir.close();
        }
        catch (Exception E2) { Errore (E2); }

        // Riabilita le transazioni
        Server.GestTransaz.abilitaTransazione (MessSer.INDIETRO);

      }
      else {
        // Il mio indietro NON e' quello caduto; gli passo questo messaggio;
        DebugInfo ("il server Indietro NON e' quello caduto -> passo indietro il MessSerRiaggancio");

        // Crea una connessione col server INDIETRO
        Socket SockIndietro = creaConnessione (MessSer.INDIETRO);

        // INVIA ALL'INDIETRO LO STESSO MESSAGGIO DI "RIAGGANCIO" ARRIVATO.
        inviaMessaggio (MyMess, SockIndietro);

        SockIndietro.close();
      }
    }
    catch (Exception E) { Errore (E); }
  } //Servi_MessSerRiaggancio

  
  protected void Servi_MessSerRiaggConMe (MessSerRiaggConMe MyMess, Socket Sock) {
    // Parte finale del protocollo di riaggancio; questo messaggio arriva
    // direttamente al server che per primo aveva generato un MessSerRiaggancio.
    // MyMess contiene l'indirizzo dell'altro server vicino a quello caduto.
    Macchina MakGener  = MyMess.cheMakGener();

    try {

      // Salta il server caduto riferendosi da ora in poi a quello successore a esso.
      Server.GestTransaz.defMacchinaVicina (MessSer.AVANTI, MakGener);

      // Riabilita le transazioni
      Server.GestTransaz.abilitaTransazione (MessSer.AVANTI);
    }
    catch (Exception E) { Errore (E); }

    // Notifica "fatto S" al mittente quando tutto e' finito
    Avvia_MessSerFatto (true, Sock);
  } //MessSerRiaggConMe


//--- METODI "PRIVATI" PER GESTIRE LE COMUNICAZIONI ------------------------------

  protected Socket creaConnessione (byte DirezSer) throws Exception {
    // Crea una connessione socket-stream col server della direzione data
    Macchina NomeSer = Server.GestTransaz.rendiMacchinaVicina (DirezSer);
    String IndSer = NomeSer.cheIndirizzo();
    DebugInfo ("connettendosi al server "+IndSer);
    return new Socket (IndSer, Comune.PORTA_STREAM_SYS);
  } //creaConnessione


  protected Socket creaConnessioneDiretta (Macchina MakServer) throws Exception {
    return creaConnessioneDiretta (MakServer.cheIndirizzo());
  } //creaConnessioneDiretta


  protected Socket creaConnessioneDiretta (String IndServer) throws Exception {
    // Crea una connessione socket-stream col server con l'indirizzo dato
    DebugInfo ("connettendosi DIRETTAMENTE al server "+IndServer);
    return new Socket (IndServer, Comune.PORTA_STREAM_SYS);
  } //creaConnessioneDiretta


  protected byte inviaMessaggio (MessSer MyMess, Socket MySock) {
    // Invia il messaggio di sistema sulla socket stream data
    // Il byte reso va confrontato con le costanti "STATO_XXX".
    DataOutputStream MyOutput = null;
    byte Stato = STATO_ERRORE;

    // Invia la richiesta
    DebugInfo ("Inviando il messaggio "+MyMess.toString());
    try {
      MyOutput = new DataOutputStream (MySock.getOutputStream());
      TraslaMess.scriviMess (MyMess, MyOutput);
      MyOutput.flush();

      Stato = STATO_FATTO;

    } 
    catch (IOException Err) { Errore(Err); }
    // NON faccio "MyOutput.close()" se no chiudo anche la socket!

    return Stato;
  } //inviaMessaggio


  protected byte inviaMessConRispTimeOut (MessSer MyMess, Socket MySock) {
    // Invia il messaggio di sistema sulla socket stream data e attende una
    // risposta di tipo "MessSerFatto" entro un certo time-out.
    // Rende:
    // STATO_FATTO  - se il mess. e' stato inviato e e' arrivata la risposta "fatto"
    // STATO_ERRORE - se il mess. non e' stato inviato o e' arrivata la risposta "NON FATTO" o
    //                e' arrivata una risposta non di tipo "fatto"
    // STATO_TIMEOUT- se il mess. e' stato inviato ma non e' arrivata risposta entro il time-out
    byte Stato;
    Stato = inviaMessaggio (MyMess, MySock);              // invia il messaggio

    if (Stato == STATO_FATTO) {

      try {

        MySock.setSoTimeout (TIMEOUT_SYS);        // ATTESA DEI DATI CON TIME-OUT!!!

        DebugInfo ("attendendo la risposta via socket-stream, con time-out");
      
        MessSer Msg = riceviMessaggio (MySock);   // ATTENDO QUI.. (e leggo la risposta)

        if (! (Msg instanceof MessSerFatto))
          throw new Exception ("MESSAGGIO DI RITORNO ERRATO");

        if ( ((MessSerFatto)Msg).cheFatto() )
          Stato = STATO_FATTO;  // Preleva lo stato dal messaggio
        else
          Stato = STATO_ERRORE;
      }
      catch (InterruptedIOException EInt) {
        Stato = STATO_TIMEOUT;             // Non e' arrivata risposta entro il time-out
        DebugInfo ("TIME-OUT SCADUTO!");
      }
      catch (Exception Err) {
        Stato = STATO_ERRORE;
        Errore (Err);
        ServerDisplay Dsp = Server.rendiDisplay();
        Dsp.apriDialogoErrModale (Dialogo.BOTT_OK,
                                  "ERRORE RICEVENDO I DATI", Err.getMessage());
      }
    } //if
    return Stato;

  } //inviaMessConRispTimeOut


  protected MessSer riceviMessaggio (Socket MySock) throws IOException {
    // Acquisisce il flusso di input dalla socket
    DataInputStream  MyInput = new DataInputStream (MySock.getInputStream());

    // Cerca di leggere il messaggio 
    MessSer MyMess = (MessSer) TraslaMess.creaMess (MyInput);
    DebugInfo ("Ricevuto il messaggio "+MyMess.toString());

    return MyMess;

  } //riceviMessaggio


  protected void DebugInfo (String Frase) {
    // Verra' sovrascritta nelle sottoclassi
    Debug.outln ("Protocol - "+Frase);
  } //DebugInfo


  protected void Errore (Exception Err) {
    // Verra' sovrascritta nelle sottoclassi
    Debug.outln ("Protocol - ERRORE: "+Err);
  } //Errore

} //Protocollo