/**  Server
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.awt.Frame;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Hashtable;
import java.util.Vector;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.FileNotFoundException;


public class Server extends Protocollo implements GestSerAvvisa, TimeOutAvvisa {
  /** Il programma SERVER
   */

  // Costanti private:
  protected static final String NOME_FILE_DB  = "DATABASE.SAV"; // File per salvare il DB
  protected static final String NOME_FILE_CFG = "SERVER.CFG";   // File di configurazione

  // Uso il pattern Model-View-Control:
  // Il server usa un display che a sua volta usa il server
  protected static ServerDisplay MySerDisp;
  // E' statica perche' alcuni oggetti useranno la classe Server 
  // (e non l'oggetto direttamente, tanto il server e' UNICO).

  // I thread che attendono l'arrivo di un messaggio via rete
  protected InputCli  MyInputCli;
  protected InputSer  MyInputSer;
  protected InputSys  MyInputSys;

  // Variabili di classe, servono per mantenere una sola istanza di
  // un particolare oggetto;
  // Sono pubbliche, per permettere gli accessi ai "GestXxx" e simili
  public static DataBase         IDataBase;    // Istanza del Data-Base
 
  public static Hashtable        MiePren;      // Le prenotazioni iniziate qui
  public static Spettacolo       MioSpet;      // L'unico spettacolo richiesto
  public static Server           ServerAllocato;  // Riferimento all'unico server
  
  public static GestTransazione  GestTransaz;  // il gestore delle transazioni

  public static Macchina MaMa      = null;     // La macchina locale (contiene la Priorita')

  public static int    TIMEOUT_CHECK = 300000; // TIME-OUT per CONTROLLO ANELLO (millisecondi)
  public        TimeOutConta  TimeOutCheck;    // "Sveglia" per fare il controllo anello

  public static String SerAggiunta = "";       // Indirizzo server per chiedere "aggiungimi"

  public static String FrasePopUp  = "";       // Messaggio per altri server

  // Variabili d'istanza:
  protected boolean Connesso  = false;         // true = e' connesso in anello

  protected int  TransazioneFatta;             // Notifica stato agg/can prenotazioni

  public boolean GigiRight    = true;          // Stampare finestra copyright a avvio?


  public Server () {
    if (ServerAllocato == null)
      ServerAllocato = this;           // Salva il riferimento all'UNICO server;
    else {
      System.out.println ("SI PUO' FARE UNA SOLO UN'ISTANZA DELL'OGGETTO SERVER");
      System.exit (2);
    }      
  } //costruttore

  public static void main (String[] args) {
    Server MySer = new Server ();
    MySer.start();
  } //main


  public void start () {

    MySerDisp = new ServerDisplay (this, "SERVER");  // Crea il proprio display

    caricaConfig ();         // All'avvio, CARICO LA CONFIGURAZIONE da disco (se c'e')

    // Ricava e stampa l'indirizzo della macchina locale
    try {
      InetAddress MioInd = InetAddress.getLocalHost();

      MaMa = new Macchina (MioInd.getHostAddress(), Priorita.maxPriorita);

      if (MaMa.cheIndirizzo().equals(""))
   	    throw new Exception ("Indirizzo Nullo");
    } 
    catch (Exception Err) {
 	    Errore(Err);
      int PremutoChe = MySerDisp.apriDialogoErrModale (Dialogo.BOTT_SINO, 
                                            "LA MACCHINA NON HA UNA SCHEDA DI RETE!",
                                            "Vuoi Continuare lo Stesso?");
      if (PremutoChe == Dialogo.PREMUTO_NO)
        System.exit(1);  // ABORT!
    }
    MySerDisp.rititola ("SERVER @ "+MaMa.cheIndirizzo());

    Debug.outln ("SERVER --- creando il 'DataBase'");
    IDataBase = new DataBase (Priorita.maxPriorita);  // Crea e alloca il data-base

    Debug.outln ("SERVER --- creando tabella 'MiePren'");
    MiePren = new Hashtable ();                      // Crea MiePren

    Debug.outln ("SERVER --- inizializzando oggetto 'MioSpet'");
    MioSpet = null;

    Debug.outln ("SERVER --- creando oggetto 'GestTransazione'");
    GestTransaz = new GestTransazione (MaMa);        // Lo crea e lo alloca


    Debug.outln ("SERVER --- creando e innescando 'TimeOutConta'");
    try {
      TimeOutCheck = new TimeOutConta (TIMEOUT_CHECK, TimeOutConta.PERSEMPRE, this);
      TimeOutCheck.start();
    }
    catch (Exception E) { Errore(E); }

    Debug.outln ("SERVER --- Il Mio Indirizzo Locale e' "+MaMa.cheIndirizzo());

    if (GigiRight)
      eseguiInfo();

    // Controllo se esiste un data base salvato
    try {
      FileInputStream FIS = new FileInputStream (NOME_FILE_DB);
      FIS.close();

      int PremutoChe = MySerDisp.apriDialogoModale (Dialogo.BOTT_SINO, 
	                                  "C' un Data Base Salvato su Disco,",
                                   "Vuoi Caricarlo?");
      if (PremutoChe == Dialogo.PREMUTO_SI)
        eseguiLoadDB();  // CARICAMENTO DATA BASE
    }
    catch (Exception E) { }  // nulla

  } //start


  public void run () {
    // nulla da fare (il thread e' usato attraverso l'AWT)
  } //run


  public void terminazione () {
    salvaConfig ();            // Prima di uscire salvo la configurazione!
    try {
      disconnessione ();  // Se errore, genera eccezione

      TimeOutCheck.termina ();

      Debug.distruggiFinestra();
      MySerDisp.distruggiDisplay();
      System.exit(0);

    }
    catch (Exception E) { } // nulla, il messaggio e' gia' stampato in "disconnessione"
  } //terminazione


  public static ServerDisplay rendiDisplay () {
    // E' "statica" poiche' "Protocollo" deve accedere al display, 
    // avendo solo il riferimento alla classe "Server".
    return MySerDisp;
  } //rendiDisplay()


  public void apriDebug (boolean apri) {
    boolean esisteGia = Debug.esisteFinestra();
    if (apri)
      if (esisteGia)
        ;  // nulla
      else {
        Debug.creaFinestra();      // Crea, se check=T e non esiste
        Debug.riposiziona  (340,0);
        Debug.ridimensiona (480,580);
      }
    else  // not apri -> chiudi
      if (esisteGia)
        Debug.distruggiFinestra(); // Toglie, se check=F e esiste
      else
        ; // nulla
  } //apriDebug


  public synchronized void operazioneFatta (int TransazioneFattaSiNo) {
    // Implementazione del metodo dell'interfaccia ""
    // Metodo usato per sbloccare il thread mentre e' in attesa della risposta da parte di
    // un (qualsiasi) thread "GestSer" (operazione terminata con successo o non fatta).
    // Serve per far uscire dal ciclo "while" (lo stato dell'operazione e' stato salvato)
    TransazioneFatta=TransazioneFattaSiNo;
    notify();   // NOTIFICA RISVEGLIO THREAD
  } //operazioneFatta


  public boolean sonoConnesso () {
    // Rende true se sono un server connesso nell'anello
    return Connesso;
  } //sonoConnesso 


  public static boolean sonoDaSolo () {
    // Rende true se sono un server "solo" (il riferimento "avanti" contiene se stessi)
    try {

      Macchina MioAvanti = GestTransaz.rendiMacchinaVicina (MessSer.AVANTI);
      return MioAvanti.equals(MaMa) ||                // Mi riferisco a me stesso
             MioAvanti.cheIndirizzo().equals("");     // o a nulla!
    }
    catch (Exception E) { return false; }
  } //sonoDaSolo


  public static boolean siamoSoloInDue () {
    // Rende true in anello ci sono solo due server
    try {
      Macchina MioAvanti   = GestTransaz.rendiMacchinaVicina (MessSer.AVANTI);
      Macchina MioIndietro = GestTransaz.rendiMacchinaVicina (MessSer.INDIETRO);

      return (! MioAvanti.equals(MaMa)) &&         // Non mi riferisco a me stesso
             MioAvanti.equals(MioIndietro);        // e l'avanti e' uguale a indietro
    }
    catch (Exception E) { return false; }
  } //siamoSoloInDue

  
  public void connessione () {
   	if ((! Connesso) && (! MaMa.cheIndirizzo().equals(""))) {

      // INPUT NOME SERVER A CUI CONNETTERSI
      Frame FinGen = (Frame) MySerDisp.rendiFinContenente ();
      ServerDispAgg MyDispAgg = new ServerDispAgg (this, "CONFIG", FinGen);
      if (SerAggiunta.equals (""))
        return;  // ESCO SE PREMO "ANNULLA" (rende stringa vuota)
                 // NB: se premo OK con testo nullo, il MyDispAgg rende l'indirizzo locale

      byte Stato;
      Dialogo MyDial = MySerDisp.apriDialogoLibero (Dialogo.BOTT_nulla,
                                                    "CONNESSIONE IN CORSO...",
                                                    "ATTENDERE PREGO");
      Debug.outln ("SERVER --- CONNESSIONE..");

      String MyAddress = MaMa.cheIndirizzo();
      if (SerAggiunta.equals(MyAddress)  ) {     // Se do il server locale,
          // Ok, sono un SERVER UNICO (PRIMA E UNICA "COPIA")
          try {
            GestTransaz.defMacchinaVicina (MessSer.AVANTI,   MaMa);   // Riempio i campi!
            GestTransaz.defMacchinaVicina (MessSer.INDIETRO, MaMa);
          }
          catch (Exception E) { Errore (E); }

          // Si attribuisce la massima priorita'
          Priorita MiaPrior = Priorita.maxPriorita;
          Server.MaMa.defPriorita (MiaPrior);                    // modifica la propria Priorita
          Server.IDataBase.defDefaultPriority (MiaPrior);        // E NE INFORMA IL DATA BASE

          Debug.outln ("SERVER --- SONO UN MISERO SERVER UNICO!");
          MySerDisp.apriDialogoErrModale (Dialogo.BOTT_OK,
	                                  "SONO UN SERVER UNICO! SISTEMA *NON* AFFIDABILE;",
                                   "INSERIRE AL PIU' PRESTO UN ALTRO SERVER!");
          Stato = Protocollo.STATO_FATTO;
      }
      else
        try {                          // ESEGUO IL PROTOCOLLO DI AGGIUNTA
          // Schema: SNuovo <==> SEsist
          // SNuovo==>SEsist - INVIA MESSAGGIO DI "AGGIUNGI-ME" AL SERVER NOTO

          Socket MySock = creaConnessioneDiretta (SerAggiunta);

          Stato = Avvio_MessSerAggMe (MySock);    // Invio mess. di "aggiungi-me"

          MySock.close();
        } 
        catch (Exception Err) {
          Stato = Protocollo.STATO_ERRORE;
          Errore(Err);
          MySerDisp.apriDialogoErrModale (Dialogo.BOTT_OK, 
                                          "ERRORE CONNETTENDOSI AL SERVER", Err.getMessage());
        }

      if (Stato == Protocollo.STATO_FATTO) {  // SE TUTTO BENE,

        // Creazione e innesco dei processi che attendono i messaggi via rete
        Debug.outln ("SERVER --- Creando e innescando i thread InputCli, InputSer e InputSys");
        MyInputSer = new InputSer();    // crea processi di ricezione messaggi da Cli e Ser
        MyInputCli = new InputCli();
        MyInputSys = new InputSys();

        MyInputCli.start();
        MyInputSer.start();             // e li innesca
        MyInputSys.start();

        Connesso = true;                // CONNESSO!
      } // if
      else {
        Debug.outln ("SERVER --- PROTOCOLLO DI CONNESSIONE ABORTITO!");
        MySerDisp.apriDialogoErrModale (Dialogo.BOTT_OK,
                                        "ERRORE: C' Stato un ERRORE Durante il Protocollo di",
                                        "Connesssione o il Server Dato ha Rifiutato di Farlo");
      }
      MyDial.dispose();
    } //if (! Connesso)y
  } //connessione


  public void disconnessione () throws Exception {
    if (Connesso) {
      String Frase;
      Dialogo MyDial, MyDial2;
      MyDial = MySerDisp.apriDialogoLibero (Dialogo.BOTT_nulla,
                          "DISCONNESSIONE IN CORSO...", "ATTENDERE PREGO");
      Debug.outln ("SERVER --- DISCONNESSIONE..");

      byte Stato;
      if (sonoDaSolo()) {                      // SE SONO UN SERVER DA SOLO
        // Ok, sono un SERVER UNICO (PRIMA E UNICA "COPIA")
        Debug.outln ("SERVER --- Sono un Server Unico!");
        Stato = Protocollo.STATO_FATTO; 
      }
      else {
        if (siamoSoloInDue())                  // SE HO UN ANELLO DI SOLI DUE SERVER
          // Schema: SNuovo<==>SA - INVIA UN MESSAGGIO DI "TOGLI-ME" ALL'UNICO VICINO
          Debug.outln ("SERVER --- inviando UN SOLO messaggio di togli-me");
        else                                 // HO ALMENO TRE SERVER IN ANELLO
          // Schema: SI<==>SNuovo<==>SA - INVIA UN MESSAGGIO DI "TOGLI-ME" A AMBO I VICINI
          Debug.outln ("SERVER --- inviando DUE messaggi di togli-me (a ambo i vicini)");

        try {
          Socket SockAvanti = creaConnessione (MessSer.AVANTI);

          try {

            Socket SockIndietro;
            if (siamoSoloInDue())          // SE HO UN ANELLO DI SOLI DUE SERVER
              SockIndietro = null;         // 2 server --> protocollo ridotto
            else
              SockIndietro = creaConnessione (MessSer.INDIETRO);

            Stato = Avvio_MessSerTogMe (SockAvanti, SockIndietro);  // Invio messaggi

            SockAvanti.close();
            if (SockIndietro != null)  SockIndietro.close();

          } 
          catch (Exception E) {
            SockAvanti.close();
            throw E;
          }
        }
        catch (Exception Err) {
          Errore(Err); 
          Avvio_MessSerPassami ();   // LANCIO PROTOCOLLO DI "PASSAMI"+"RIAGGANCIO"
          Stato = Protocollo.STATO_ERRORE;
        }
      } //else

      if (Stato == Protocollo.STATO_FATTO) {  // SE TUTTO BENE,
        // TERMINA L'ESECUZIONE DEI PROCESSI "InputCli", "InputSer" e "InputSys"
        MyInputCli.terminaEsecuzione ();
        MyInputSer.terminaEsecuzione ();
        MyInputSys.terminaEsecuzione ();
        try {
          while (MyInputCli.isAlive()) ;   // Attende fine del processo InputCli
        }
        catch (Exception E) { }
        try {
          while (MyInputSer.isAlive()) ;   // Attende fine del processo InputSer
        }
        catch (Exception E) { }
        try {
          while (MyInputSys.isAlive()) ;   // Attende fine del processo InputSys
        }
        catch (Exception E) { }

        Connesso = false;                  // DISCONNESSO!
      } // if
      else {
        Debug.outln ("SERVER --- PROTOCOLLO DI DISCONNESSIONE ABORTITO!");
        MySerDisp.apriDialogoErrModale (Dialogo.BOTT_OK, 
                                        "ERRORE: C' Stato un ERRORE Durante il Protocollo di",
                                        "Disconnessione o i Vicini Hanno Rifiutato di Farlo");
      }
      MyDial.dispose();

      if (Stato != Protocollo.STATO_FATTO)  // SE ERRORE,
        throw new Exception ("ERRORE ESEGUENDO IL PROTOCOLLO DI DISCONNESSIONE");
    } //if (! Connesso)
  } //disconnessione


  public void eseguiConfig () {
    // CHIAMATA DAL LISTENER "CONFIG"
    // Crea l'apposito display (modale con il corrente)
    Frame FinGen = (Frame) MySerDisp.rendiFinContenente ();
    ServerDispCfg MyDispcfg = new ServerDispCfg (this, "CONFIG", FinGen);
    // side-effect sull'intero stato
  } //eseguiConfig


  public void eseguiInfo () {
    Frame FinGen = (Frame) MySerDisp.rendiFinContenente();
    DisplayGigi MyD = new DisplayGigi (FinGen, "server");
  } //eseguiInfo


  public void eseguiStato () {
    Frame FinGen = (Frame) MySerDisp.rendiFinContenente();
    ServerDispStato MyD = new ServerDispStato (this, "STATO", FinGen);
  } //eseguiStato


  public void eseguiPassa (boolean InviaPopUp) {
    if (Connesso) {
      if (InviaPopUp) {
        // Richiede all'utente la frase da inviare
        Frame FinGen = (Frame) MySerDisp.rendiFinContenente();
        ServerDispMsg MyD = new ServerDispMsg (this, "INVIA MESSAGGIO", FinGen);
        // side-effect su FrasePopUp

        if (! FrasePopUp.equals(""))
          Avvio_MessSerPassami (FrasePopUp);      // Invio messaggio "passami"
      }
      else {
        MySerDisp.apriDialogoLibero (Dialogo.BOTT_OK, "LANCIATO ORDINE DI CONTROLLO", "");
        Avvio_MessSerPassami ();          // Invio messaggio "passami" PER CONTROLLO
      }
    }
    else {
      String Frase = "NON SONO CONNESSO IN ANELLO!";
      Debug.outln ("SERVER --- "+Frase);
      MySerDisp.apriDialogoErrModale (Dialogo.BOTT_OK, "ERRORE!", Frase);
    }
  } //eseguiPassa


  protected boolean ConnessoONoMaContinua () {
    // se si e' "connessi", rende True; se no, apre un "dialog" e chiede all'utente
    // se continuare o no; se l'utente dice si', rende True; false altrimenti.
    if (Connesso)
      return true;
    else {
      int PremutoChe = MySerDisp.apriDialogoErrModale (Dialogo.BOTT_SINO, 
                                            "Non Sono Connesso! Posso Accedere Solo",
                                            "al DataBase Locale!  CONTINUO?");
     return PremutoChe == Dialogo.PREMUTO_SI;
   }
  } //ConnessoONoMaContinua
  

  public void eseguiVediDB () {
    if (! ConnessoONoMaContinua()) return;    // Controllo con eventuale richiesta

    // Crea un vettore con l'elenco degli spettacoli
    Vector ElencoSc = IDataBase.rendiElencoSpettacoli ();
    // Scelta dello spettacolo tramite un'apposita finestra
    Frame FinGen = (Frame) MySerDisp.rendiFinContenente ();
    ServerDispLista MyDisp = new ServerDispLista (this, ElencoSc, 
                                                  ServerDispLista.USO_MOSTRA_SC,
                                                  "VISUALIZZA SPETTACOLO", FinGen);
  } //eseguiVediDB


  public void eseguiVediSpettacolo (Spettacolo St) {
    try {
      // Crea un vettore con l'elenco posti+spettatori
      Vector ElencoPSt = IDataBase.rendiElencoPostiOccStr (St);
      // E apre una nuova finestra (ma col solo tastone rosso)
      Frame FinGen = (Frame) MySerDisp.rendiFinContenente ();
      ServerDispLista MyDisp = new ServerDispLista (this, ElencoPSt,
                                                    ServerDispLista.USO_ELENCO_ST,
                                                    "POSTI PRENOTATI PER "+St.toString(), FinGen);
    }
    catch (Exception E) { Errore (E); }
  } //eseguiVediSpettacolo


  public void eseguiTermina () {
    // Chiede CONFERMA a utente prima di chiudere
    int PremutoChe;

    if (Connesso) {
      PremutoChe = MySerDisp.apriDialogoModale (Dialogo.BOTT_SINO, 
                                              "Sei Sicuro di Voler Uscire?", 
                                              "(questo server sara' disconnesso)");
      if (PremutoChe == Dialogo.PREMUTO_SI) {
        int PremutoPoi = MySerDisp.apriDialogoModale (Dialogo.BOTT_SINO, 
                                              "Vuoi Salvare il Data Base?", 
                                              "(senn tutti i dati saranno persi)");
        if (PremutoPoi == Dialogo.PREMUTO_SI)
          eseguiSaveDB();
      }
    }
    else 
      PremutoChe = MySerDisp.apriDialogoModale (Dialogo.BOTT_SINO, 
                                              "Sei Sicuro di Voler Uscire?", "");

    if (PremutoChe == Dialogo.PREMUTO_SI)
      terminazione();  // TERMINAZIONE!

  } //eseguiTermina


  public void eseguiAggSpettacolo () {
    if (! ConnessoONoMaContinua()) return;    // Controllo con eventuale richiesta

    // Input dei nuovi dati da una apposita finestra
    Frame FinGen = (Frame) MySerDisp.rendiFinContenente ();
    ServerDispAggSc MyDisp = new ServerDispAggSc (this, 
                                            "AGGIUNTA SPETTACOLO", FinGen);
  } //eseguiAggSpettacolo


  public void eseguiAggSpettacolo (Spettacolo Sc) {
   Avvio_MessSerDgAggCanSc (new MessSerDgAggSc (Sc));
  } //eseguiAggSpettacolo

  
  public void eseguiCanSpettacolo () {
    if (! ConnessoONoMaContinua()) return;    // Controllo con eventuale richiesta

    // Crea un vettore con l'elenco degli spettacoli
    Vector ElencoSc = IDataBase.rendiElencoSpettacoli ();

    // Scelta dello spettacolo tramite un'apposita finestra
    Frame FinGen = (Frame) MySerDisp.rendiFinContenente ();
    ServerDispLista MyDisp = new ServerDispLista (this, ElencoSc, 
                                                  ServerDispLista.USO_CANCELLA,
                                                  "CANCELLAZIONE SPETTACOLO", FinGen);
  } //eseguiCanSpettacolo


  public void eseguiCanSpettacolo (Spettacolo Sc) {
    Avvio_MessSerDgAggCanSc (new MessSerDgCanSc (Sc));
  } //eseguiCanSpettacolo


  public synchronized void Avvio_MessSerDgAggCanSc (MessSerDgAggSc MyMess) {
    // La "Servi_XXX" e' contenuta in "GestSer"
    // inizia il protocollo per cancellare lo spettacolo nel messaggio MyMess
    // e' "synchronized" perche' per fare la "wait" deve possedere il monitor
    GestSer          MyGestSer;

    // CONTROLLO
   try {
      boolean Esiste = Server.IDataBase.esisteSpettacolo (MyMess.cheSpettacolo());
      String Frase = null;
      if (MyMess instanceof MessSerDgCanSc)
        if (! Esiste) 
          Frase = "LO SPETTACOLO NON ESISTE";
        else ; //nulla
      else  // e' un MessSerDgAggSc
        if (Esiste)
          Frase = "LO SPETTACOLO ESISTE GIA'";

      if (Frase != null) {
        // NOTIFICA A AMMINISTRATORE E TERMINA
        Debug.outln ("Server  -- Attenzione: "+Frase);
        MySerDisp.apriDialogoErrModale (Dialogo.BOTT_OK, "ERRORE", Frase);
        return;  // RITORNA!
      }
    } 
    catch (Exception E) { }

    // Creo in MessBis il messaggio che sara' passato tra i server
    // NB: controllo prima "CANCELLA" perche' e' sottoclasse di PRENOTA

    MyMess.defMacchina (Server.MaMa);      // Definisce la macchina di origine del mess.

    TransazioneFatta=GestCli.FATTA_ERR;    // per ora, non e' stata fatta la prenotazione
    // NOTA: questo processo NON scrive nel DataBase (lo fara' il "GestSer")

    Debug.outln ("Server  -- scrivendo spettacolo in 'MioSpet'");
    Server.MioSpet = MyMess.cheSpettacolo();  // SALVA LO SPETTACOLO

    byte Tenta=0;
    do {

      Tenta++;
      Debug.outln ("Server  -- innescando un thread GestSer (tentativo "+Tenta+")");
      // crea un processo di gestione GestSer per il messaggio appena creato, 
      MyGestSer = new GestSer (MyMess, GestSer.GEN_ORA_DA_ME);
      // lo innesca e si mette in ATTESA!
      MyGestSer.start();

      Debug.outln ("Server  -- attendendo risposta o time-out..");

      // Attende finche' o scade il tempo o transazione fatta (Si' o No),
      // ossia attende il risveglio con "notify" entro il "time-out"
      // Il notify e' invocato dalla "operazioneFatta"
      try {
        try {
          wait (GestCli.TIMEOUT_GC);
        } 
        catch (InterruptedException E) { Debug.outln ("'wait' Server interrotta"); }
      }
      catch (Exception Er) { Errore (Er); }


      // Ora per sapere cosa e' successo, basta vedere se e' stata 
      // modificata la variabile "TransazioneFatta".
      if (TransazioneFatta == GestCli.FATTA_ERR)
        Debug.outln ("Server  -- si e' superato il TIME-OUT!");
      else
        Debug.outln ("Server  -- sono stato risvegliato, con TransFatta="+GestCli.FATTA_2_CHAR[TransazioneFatta]);

    }
    while ((TransazioneFatta == GestCli.FATTA_ERR) &&  // esce quando transazione fatta S/N,
           (Tenta <  GestCli.MAX_TENTA_GC));           // o non e' stato possibile farla

    // Prepara la risposta per l'amministratore (a seconda del contenuto di "TransazioneFatta")
    Debug.outln ("Server  -- inviando risposta all'amministratore");
    String Frase;
    if (TransazioneFatta == GestCli.FATTA_SI)
        Frase = "Operazione Completata con Successo";
    else
      if (TransazioneFatta == GestCli.FATTA_NO)
        Frase = "NON E' STATO POSSIBILE COMPLETARE L'OPERAZIONE";
      else
        Frase = "NON E' STATO POSSIBILE FARE L'OPERAZIONE";

    MySerDisp.apriDialogoModale (Dialogo.BOTT_OK, "Risultato dell'Operazione:", Frase);

  } //Avvio_MessSerDgAggCanSc


  public void eseguiLoadDB () {
    Debug.outln ("SERVER --- Caricando DataBase da file "+NOME_FILE_DB);
    try {
      if (Connesso)           // Posso caricare solo se NON SONO CONNESSO in anello
        throw new Exception ("SONO CONNESSO IN ANELLO: NON POSSO CARICARE");

      FileInputStream FIS = new FileInputStream (NOME_FILE_DB);
      ObjectInputStream OIS = new ObjectInputStream (FIS);
      IDataBase = (DataBase) OIS.readObject ();
      IDataBase.defDefaultPriority (MaMa.chePriorita());
      FIS.close();

      Debug.out (IDataBase.toString("[DataBase]"));
      MySerDisp.apriDialogoModale (Dialogo.BOTT_OK, "Data Base", "Caricato");

    }
    catch (FileNotFoundException E) {
      Errore (E);
      MySerDisp.apriDialogoErrModale (Dialogo.BOTT_OK, "ERRORE!", 
                                      "IL DATA BASE NON E' MAI STATO SALVATO");
    }
    catch (Exception Err) {
      Errore (Err);
      MySerDisp.apriDialogoErrModale (Dialogo.BOTT_OK, "ERRORE!", Err.getMessage());
    }
  } //eseguiLoadDB


  public void eseguiSaveDB () {
    Debug.outln ("SERVER --- Salvando DataBase su file "+NOME_FILE_DB);
    try {
      if (! Connesso)         // Posso salvare solo se SONO CONNESSO in anello
        throw new Exception ("NON SONO CONNESSO IN ANELLO: NON POSSO SALVARE");

      FileOutputStream   FOS = new FileOutputStream (NOME_FILE_DB);

      ObjectOutputStream OOS = new ObjectOutputStream (FOS);
      OOS.writeObject (IDataBase);
      OOS.flush();
      FOS.close();

      MySerDisp.apriDialogoModale (Dialogo.BOTT_OK, "Data Base", "Salvato");
    } 
    catch (Exception Err) { 
      Errore (Err);
      MySerDisp.apriDialogoErrModale (Dialogo.BOTT_OK, "ERRORE!", Err.getMessage());
    }
  } //eseguiSaveDB


  public void salvaConfig () {
    // Salva su disco i dati di configurazione
    try {
      FileOutputStream FOS = new FileOutputStream (NOME_FILE_CFG);
      DataOutputStream DOS = new DataOutputStream (FOS);
      Debug.outln ("SERVER --- salvando la configurazione su disco");

      String IndDaSalvare;  // Indirizzo a cui poter chiedere di riagganciarsi
      try {
        Macchina MioIndietro = GestTransaz.rendiMacchinaVicina (MessSer.INDIETRO);
        IndDaSalvare = MioIndietro.cheIndirizzo();
      }
      catch (Exception E) { IndDaSalvare = SerAggiunta; }

      DOS.writeUTF (IndDaSalvare);
      DOS.writeInt (GestCli.TIMEOUT_GC);
      DOS.writeInt (GestCli.MAX_TENTA_GC);
      DOS.writeInt (Protocollo.TIMEOUT_SYS);
      DOS.writeBoolean (Debug.esisteFinestra());
      DOS.writeBoolean (GigiRight);
      
      FOS.close();
    }
    catch (Exception Err) { }  // nulla
  } //salvaConfig


  public void caricaConfig () {
    // Carica da disco i dati di configurazione (se sono stati salvati)
    try {
      FileInputStream FIS = new FileInputStream (NOME_FILE_CFG);
      DataInputStream DIS = new DataInputStream (FIS);
      Debug.outln ("SERVER --- caricando la configurazione da disco");

      SerAggiunta = DIS.readUTF();
      GestCli.TIMEOUT_GC     = DIS.readInt ();
      GestCli.MAX_TENTA_GC   = DIS.readInt ();
      Protocollo.TIMEOUT_SYS = DIS.readInt ();
      apriDebug (DIS.readBoolean());
      GigiRight              = DIS.readBoolean ();

      FIS.close();
    }
    catch (Exception Err) { }  // nulla
  } //caricaConfig


  public void AvvisoDiTimeOut (boolean FuoriTempoMax) {
    // Dichiarata nell'interfaccia "TimeOutAvvisa"
    if (FuoriTempoMax) 
      if (Connesso)

          Avvio_MessSerPassami ();          // Invio messaggio "passami" PER CONTROLLO

      else
        Debug.outln ("SERVER --- CONTROLLO AUTOMATICO FALLITO - NON SONO IN ANELLO!");
  } //AvvisoDiTimeOut


  protected void DebugInfo (String Frase) {
    // Verra' sovrascritta nelle sottoclassi
    Debug.outln ("SERVER --- "+Frase);
  } //DebugInfo


  protected void Errore (Exception Err) {
    // Overriding di "Errore" in "Protocollo"
    Debug.outln ("SERVER --- ERRORE: "+Err);
  } //Errore

} //Server