/**  ServerDispAgg - display di "aggiungi-a-chi" per il server
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.awt.Frame;
import java.awt.Label;
import java.awt.TextField;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Font;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.net.InetAddress;


//-------------- I "LISTENER" DEI BOTTONI ---------------------------------------------------------

class ListenSerAggAn implements ActionListener {
  // Astrazione di un listener che usa un'applicazione Server e
  // la finestra di configurazione (da distruggere)
  protected Server        SerApp;   // Riferimento al Server da usare
  protected ServerDispAgg FinAgg;   // Finestra coi dati

  public ListenSerAggAn (Server SerApp, ServerDispAgg FinAgg) { 
    this.SerApp = SerApp;
    this.FinAgg = FinAgg;
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    SerApp.SerAggiunta = "";
    FinAgg.distruggiDisplay ();
  } //actionPerformed

} //ListenSerAggAn


class ListenSerAggOK extends ListenSerAggAn implements ActionListener {

  public ListenSerAggOK (Server SerApp, ServerDispAgg FinAgg) { 
    super (SerApp, FinAgg);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    // MODIFICA I DATI DEL SERVER:
    String StrInd = FinAgg.AreaNomeSer.getText();
    ServerDisplay SerDsp = SerApp.rendiDisplay();
    Dialogo MyDial = SerDsp.apriDialogoLibero (Dialogo.BOTT_nulla,
                                "CONTROLLANDO L'INDIRIZZO...", "ATTENDERE PREGO");
    try	{
      if (StrInd.equals(""))    // Se indirizzo nullo, rende indirizzo macchina locale
        SerApp.SerAggiunta = SerApp.MaMa.cheIndirizzo();

      else {
        // Prova a ricavarne l'indirizzo IP (se no, eccezione)
        InetAddress IPAddr = InetAddress.getByName (StrInd);
        // Se sono qui, tutto OK, aggiorno l'indirizzo
        SerApp.SerAggiunta = StrInd;
      }

      MyDial.dispose();
      FinAgg.distruggiDisplay ();   // distruggo solo se tutto OK
    } 
    catch (Exception E) {
      MyDial.dispose();
      SerApp.SerAggiunta = "";
      SerDsp.apriDialogoErrModale (Dialogo.BOTT_OK, 
                                   "ERRORE: INDIRIZZO", "ERRATO O NON ESISTENTE");
    }
  } //actionPerformed

} //ListenSerAggOK


//============== IL DISPLAY DI AGGIUNGI A CHI =====================================================

class ServerDispAgg extends Display {
  /** Il display del programma (applicazione) Server
   */

  // Uso il pattern Model-View-Control:
  // Il server usa un display che a sua volta usa il server
  protected Server MySerAp;

  public static TextField  AreaNomeSer;


  public ServerDispAgg (Server MyServer) {
    this (MyServer, "", null);
  } //costruttore

  public ServerDispAgg (Server MyServer, String TitoloFin) {
    this (MyServer, TitoloFin, null);
  } //costruttore

  public ServerDispAgg (Server MyServer, String TitoloFin, Frame FinPapa) {
    super (TitoloFin, FinPapa);
    MySerAp = MyServer;
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore

  public ServerDispAgg (Server MyServer, Frame FinGiaCreata) {
    super (FinGiaCreata);
    MySerAp = MyServer;
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore


  protected void CostruisciDisplay () {
    super.CostruisciDisplay ();

    // Riempie le caselle dei dati
    AreaNomeSer = new TextField (15);
    AreaNomeSer.setText (MySerAp.SerAggiunta);

    GridBagLayout      GBL = new GridBagLayout (); 
    GridBagConstraints GBC = new GridBagConstraints ();
    setLayout(GBL);

    Label L1A = new Label ("DAMMI IL SERVER A CUI");
    Label L1B = new Label ("RICHIEDERE DI AGGANCIARSI:");
    L1A.setFont(FONT_TEXT0);
    L1B.setFont(FONT_TEXT0);

    Label LSSu  = new Label ("Indirizzo di un Server Attivo:");
    Label LSGiu = new Label ("(nulla, per essere un server unico)  ");

    GBC.anchor = GridBagConstraints.CENTER;
    GBC.gridx = 2;  GBC.gridy = 0;
    GBL.setConstraints (L1A, GBC);
    add (L1A);
    GBC.gridx = 2;  GBC.gridy = 1;
    GBL.setConstraints (L1B, GBC);
    add (L1B);

    GBC.anchor = GridBagConstraints.EAST;
    GBC.gridx = 2;  GBC.gridy = 2;
    GBL.setConstraints (LSSu, GBC);
    add (LSSu);
    GBC.gridx = 2;  GBC.gridy = 3;
    GBL.setConstraints (LSGiu, GBC);
    add (LSGiu);

    GBC.anchor = GridBagConstraints.WEST;
    GBC.gridx = 3;  GBC.gridy = 2;
    GBL.setConstraints (AreaNomeSer, GBC);
    add (AreaNomeSer);

    GBC.anchor = GridBagConstraints.WEST;
    Bottone.defFont (FONT_BOTT);
    Bottone BottCfgOK  = new Bottone ("CONNETTITI", COL_SFO_BOTOK, COL_TXT_BOTOK);
    Bottone BottCfgAn  = new Bottone ("ANNULLA",    COL_SFO_BOTAN, COL_TXT_BOTAN);

    BottCfgOK.addActionListener (new ListenSerAggOK(MySerAp, this));
    GBC.gridx = 1;  GBC.gridy = 4;
    GBL.setConstraints (BottCfgOK, GBC);
    add (BottCfgOK);

    BottCfgAn.addActionListener (new ListenSerAggAn(MySerAp, this));
    GBC.gridx = 4;  GBC.gridy = 4;
    GBL.setConstraints (BottCfgAn, GBC);
    add (BottCfgAn);

    riposiziona (50, 200);

  } //CostruisciDisplay


  public Server rendiServer () {
    return MySerAp;
  } //rendiServer

} //ServerDispAgg