/**  ServerDispAggSc - display del Server per l'aggiunta di uno spettacolo
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.awt.Frame;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;


//-------------- I "LISTENER" DEI BOTTONI ---------------------------------------------------------

class ListenSerAggScAn implements ActionListener {
  // Astrazione di un listener che usa un'applicazione Server e
  // la finestra di configurazione (da distruggere)
  protected Server          SerApp;   // Riferimento al Server da usare
  protected ServerDispAggSc FinAgg;   // Finestra coi dati

  public ListenSerAggScAn (Server SerApp, ServerDispAggSc FinAgg) { 
    this.SerApp = SerApp;
    this.FinAgg = FinAgg;
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    // Non modifica i dati del Server
    FinAgg.distruggiDisplay ();
  } //actionPerformed

} //ListenSerAggScAn


class ListenSerAggScOK extends ListenSerAggScAn implements ActionListener {

  public ListenSerAggScOK (Server SerApp, ServerDispAggSc FinAgg) { 
    super (SerApp, FinAgg);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    // MODIFICA I DATI DEL SERVER:
    try {
      Spettacolo Sc = FinAgg.rendiSpettacolo();
      if (! Sc.corretto())
        throw new Exception ();

      SerApp.eseguiAggSpettacolo (Sc);

      // NO: FinAgg.distruggiDisplay ();  NON rimuovo il display
    }
    catch (Exception E) {
      ServerDisplay Dsp = SerApp.rendiDisplay();
      Dsp.apriDialogoErrModale (Dialogo.BOTT_OK, "ERRORE", "DATI NON CORRETTI");
    }
  } //actionPerformed

} //ListenSerAggScOK


//============== IL DISPLAY PER L'AGGIUNTA DI UNO SPETTACOLO ============================

class ServerDispAggSc extends Display {
  /** Il display del Server per l'aggiunta di uno spettacolo
   */

  // Uso il pattern Model-View-Control:
  // Il client usa un display che a sua volta usa il client
  protected Server MySerAp;

  protected SpettacoloZona SpecZona;   // Zona per lo spettacolo


  public ServerDispAggSc (Server MyServer) {
    this (MyServer, "");
  } //costruttore

  public ServerDispAggSc (Server MyServer, String TitoloFin) {
    super (TitoloFin);
    MySerAp = MyServer;   // SALVA IL RIFERIMENTO AL SUO APPLET
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore

  public ServerDispAggSc (Server MyServer, String TitoloFin, Frame FinPapa) {
    super (TitoloFin, FinPapa);
    MySerAp = MyServer;
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore

  public ServerDispAggSc (Server MyServer, Frame FinGiaCreata) {
    super (FinGiaCreata);
    MySerAp = MyServer;
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore


  protected void CostruisciDisplay () {
    super.CostruisciDisplay ();

    GridBagLayout      GBL = new GridBagLayout (); 
    GridBagConstraints GBC = new GridBagConstraints ();
    setLayout (GBL);
    
    GBC.anchor = GridBagConstraints.WEST;
    SpecZona = new SpettacoloZona ();
    GBC.gridx = 2;  GBC.gridy = 2;  
    GBL.setConstraints (SpecZona, GBC);
    add (SpecZona);

    GBC.anchor = GridBagConstraints.WEST;
    Bottone.defFont (FONT_BOTT);
    Bottone BottOK  = new Bottone ("AGGIUNGI", COL_SFO_BOTOK, COL_TXT_BOTOK);
    Bottone BottAn  = new Bottone ("FINITO",   COL_SFO_BOTAN, COL_TXT_BOTAN);

    BottOK.addActionListener (new ListenSerAggScOK(MySerAp, this));
    GBC.gridx = 1;  GBC.gridy = 4;
    GBL.setConstraints (BottOK, GBC);
    add (BottOK);

    BottAn.addActionListener (new ListenSerAggScAn(MySerAp, this));
    GBC.gridx = 3;  GBC.gridy = 4;
    GBL.setConstraints (BottAn, GBC);
    add (BottAn);

    riposiziona (100, 100);
  } //CostruisciDisplay


  public Spettacolo rendiSpettacolo () throws Exception {
     return (Spettacolo) SpecZona.rendiDati();
  } //rendiSpettacolo

  public void cancella () {
    SpecZona.cancella();
  } //cancella

} //ServerDispAggSc