/**  ServerDispLista - display Server per mostrare e scegliere da una lista
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.awt.Frame;
import java.awt.Panel;
import java.awt.Label;
import java.awt.List;
import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.util.Enumeration;
import java.util.Vector;


//-------------- I "LISTENER" DEI BOTTONI ---------------------------------------------------------

class ListenSerLstAn implements ActionListener {
  // Astrazione di un listener che usa un'applicazione Server e
  // la finestra di configurazione (da distruggere)
  protected Server          SerApp;   // Riferimento al Server da usare
  protected ServerDispLista FinCfg;   // Finestra coi dati

  public ListenSerLstAn (Server SerApp, ServerDispLista FinCfg) { 
    this.SerApp = SerApp;
    this.FinCfg = FinCfg;
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    // Non modifica i dati del Server
    FinCfg.distruggiDisplay ();
  } //actionPerformed

} //ListenSerLstAn


class ListenSerLstOK extends ListenSerLstAn implements ActionListener {

  public ListenSerLstOK (Server SerApp, ServerDispLista FinCfg) { 
    super (SerApp, FinCfg);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    // Preleva il N. di voce scelto nella lista su AWT
    int N = FinCfg.AwtList.getSelectedIndex();
    if (N >= 0) {
      // Preleva l'oggetto dalla lista (come Vector)
      Object OO = FinCfg.VectDato.elementAt(N);

      // E lo invia al server per cancellarlo o quantaltro
      switch (FinCfg.UsoChe) {
        case ServerDispLista.USO_CANCELLA: {
                  SerApp.eseguiCanSpettacolo ((Spettacolo) OO);

                  FinCfg.AwtList.remove (N);          // E la toglie dalla lista!
                  FinCfg.VectDato.removeElementAt(N); // e dal vettore!
                  break;
             }
        case ServerDispLista.USO_MOSTRA_SC: {
                  SerApp.eseguiVediSpettacolo((Spettacolo) OO);
                  // Sara' il server stesso a mostrare i dati dello spettacolo scelto
                  break;
             }
        case ServerDispLista.USO_ELENCO_ST: {
                  // NULLA: non e' attivo questo "listener"
                  break;
             }
      } //switch
    } //if
  } //actionPerformed

} //ListenSerLstOK


//============== IL DISPLAY VISUALIZZAIZONE LISTA =================================================

class ServerDispLista extends Display {
  /** Il display del programma (applet) Server
   */

  // Costanti pubbliche:
  public    static final byte  USO_CANCELLA  = 0;
  public    static final byte  USO_MOSTRA_SC = 1;
  public    static final byte  USO_ELENCO_ST = 2;

  // Costanti private:
  protected static final Color COL_SFO_LST   = Color.black;
  protected static final Color COL_TXT_LST   = Color.green;
  protected static final Color COL_SFO_BOTOK = Color.green;
  protected static final Color COL_TXT_BOTOK = Color.black;
  protected static final Color COL_SFO_BOTAN = Color.red;
  protected static final Color COL_TXT_BOTAN = Color.white;
  
  // Uso il pattern Model-View-Control:
  // Il Server usa un display che a sua volta usa il Server
  protected Server MySerAp;

  public List     AwtList;
  public Vector   VectDato;    // pubblici: acceduti dal listener "ok"
  public byte     UsoChe;      // idem, per sapere cosa fare..


  public ServerDispLista (Server MyServer, Vector Lista, byte USO_CHE) {
    this (MyServer, Lista, USO_CHE, "", null);
  } //costruttore

  public ServerDispLista (Server MyServer, Vector Lista, byte USO_CHE, 
                          String TitoloFin) {
    this (MyServer, Lista, USO_CHE, TitoloFin, null);
  } //costruttore

  public ServerDispLista (Server MyServer, Vector Lista, byte USO_CHE, 
                          String TitoloFin, Frame FinPapa) {
    super (TitoloFin, FinPapa);
    MySerAp = MyServer;   // SALVA RIFERIMENTO A APPLICAZIONE
    VectDato= Lista;
    UsoChe  = USO_CHE;
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore

  public ServerDispLista (Server MyServer, Vector Lista, byte USO_CHE, 
                          Frame FinGiaCreata) {
    super (FinGiaCreata);
    MySerAp  = MyServer;
    VectDato = Lista;
    UsoChe   = USO_CHE;
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore


  protected void CostruisciDisplay () {
    super.CostruisciDisplay ();

    // Riempie le caselle dei dati
    AwtList = new List (16);
    AwtList.setMultipleMode (false);
    AwtList.setBackground (COL_SFO_LST);
    AwtList.setForeground (COL_TXT_LST);

    Enumeration Enum = VectDato.elements();
    if (Enum.hasMoreElements())
      while (Enum.hasMoreElements()) {             // Inserisce elementi in lista
        AwtList.add ((Enum.nextElement()).toString());
      } //while
    else {
      String Frase;
      switch (UsoChe) {
        case USO_CANCELLA  : // va al seguente
        case USO_MOSTRA_SC : { Frase = "<nessuno spettacolo in programma>";  break; }
        case USO_ELENCO_ST : { Frase = "<nessun posto prenotato>";           break; }
        default            :   Frase = "<lista vuota>";
      }
      AwtList.add (Frase);
      AwtList.setEnabled(false);  // disabilita la lista!
    }

    setLayout (new BorderLayout());

    String CheCosa;
    switch (UsoChe) {
      case USO_CANCELLA  : 
      case USO_MOSTRA_SC : { CheCosa = "Ecco l'Elenco degli Spettacoli in Programma:"; break; }
      case USO_ELENCO_ST : { CheCosa = "Ecco la Lista di Tutti i Posti Prenotati:";    break; }
      default            :   CheCosa = "scegli:";
    }

    Label L1  = new Label (CheCosa);
    L1.setFont(FONT_TEXT0);
    add (L1, "North");

    add (AwtList, "Center");

    Panel PamPam = new Panel ();

    Bottone.defFont (FONT_BOTT);        // Definizione per il tasto "verde"
    String TxtBot;
    switch (UsoChe) {
      case USO_CANCELLA  : { TxtBot = "CANCELLA SPETTACOLO"; break; }
      case USO_MOSTRA_SC : { TxtBot = "MOSTRA SPETTACOLO";   break; }
      default            : { TxtBot = null; } // Tutti gli altri casi (niente bottone)
    }
    if (TxtBot != null) {
      Bottone.defColori (COL_SFO_BOTOK, COL_TXT_BOTOK);
      Bottone BottCfgOK  = new Bottone (TxtBot);
      BottCfgOK.addActionListener (new ListenSerLstOK(MySerAp, this));
      PamPam.add (BottCfgOK);
    }

    Bottone.defColori (COL_SFO_BOTAN, COL_TXT_BOTAN);
    Bottone BottCfgAn  = new Bottone ("FINITO");
    BottCfgAn.addActionListener (new ListenSerLstAn(MySerAp, this));
    PamPam.add (BottCfgAn);

    add (PamPam, "South");

    switch (UsoChe) {
      case USO_CANCELLA  :   // segue
      case USO_MOSTRA_SC : { riposiziona (120, 10); break; }
      default            : { riposiziona (130, 20); }        // Tutti gli altri casi
    }

    

  } //CostruisciDisplay

} //ServerDispLista