/**  ServerDispStato - display del Server per mostrare lo stato interno
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.awt.Frame;
import java.awt.Label;
import java.awt.TextField;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;


//-------------- I "LISTENER" DEI BOTTONI ---------------------------------------------------------

class ListenSerStatoAn implements ActionListener {
  // Astrazione di un listener che usa un'applicazione Server e
  // la finestra di configurazione (da distruggere)
  protected Server          SerApp;   // Riferimento al Server da usare
  protected ServerDispStato FinAgg;   // Finestra coi dati

  public ListenSerStatoAn (Server SerApp, ServerDispStato FinAgg) { 
    this.SerApp = SerApp;
    this.FinAgg = FinAgg;
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    // Non modifica i dati del Server
    FinAgg.distruggiDisplay ();
  } //actionPerformed

} //ListenSerStatoAn


class ListenSerStatoUpd extends ListenSerStatoAn implements ActionListener {

  public ListenSerStatoUpd (Server SerApp, ServerDispStato FinAgg) { 
    super (SerApp, FinAgg);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    // MODIFICA I DATI DEL SERVER:
    FinAgg.ritraccia();
  } //actionPerformed

} //ListenSerStatoUpd


//============== IL DISPLAY PER MOSTRARE LO STATO INTERNO ===============================

class ServerDispStato extends Display {
  /** Il display del Server per mostrare lo stato interno
   */

  // Uso il pattern Model-View-Control:
  // Il client usa un display che a sua volta usa il client
  protected Server MySerAp;

  public static TextField[] AreaIndSer = new TextField [MessSer.INDIETRO+1];
  public static TextField[] AreaBlocco = new TextField [MessSer.INDIETRO+1];
  public static TextField   AreaTrnAtt;


  public ServerDispStato (Server MyServer) {
    this (MyServer, "");
  } //costruttore

  public ServerDispStato (Server MyServer, String TitoloFin) {
    super (TitoloFin);
    MySerAp = MyServer;   // SALVA IL RIFERIMENTO AL SUO APPLET
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore

  public ServerDispStato (Server MyServer, String TitoloFin, Frame FinPapa) {
    super (TitoloFin, FinPapa);
    MySerAp = MyServer;
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore

  public ServerDispStato (Server MyServer, Frame FinGiaCreata) {
    super (FinGiaCreata);
    MySerAp = MyServer;
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore


  protected void CostruisciDisplay () {
    super.CostruisciDisplay ();

    // Costuisce le caselle dei dati
    for (byte dir=MessSer.AVANTI; dir<=MessSer.INDIETRO; dir++) {
      AreaIndSer[dir] = new TextField (22);
      AreaIndSer[dir].setEditable(false);
      AreaBlocco[dir] = new TextField (4);
      AreaBlocco[dir].setEditable(false);
    }
    AreaTrnAtt = new TextField (4);
    AreaTrnAtt.setEditable(false);

    GridBagLayout      GBL = new GridBagLayout (); 
    GridBagConstraints GBC = new GridBagConstraints ();
    setLayout (GBL);

    Label L1  = new Label ("STATO DEL SERVER:");
    L1.setFont(FONT_TEXT0);
    GBC.anchor = GridBagConstraints.CENTER;
    GBC.gridx = 1;  GBC.gridy = 1;
    GBL.setConstraints (L1, GBC);
    add (L1);

    GBC.anchor = GridBagConstraints.EAST;
    for (byte dir=MessSer.AVANTI; dir<=MessSer.INDIETRO; dir++) {
      Label L2A = new Label ("Indirizzo & Priorit Server '"+MessSer.NOME_DIR[dir]+"':");
      GBC.gridx = 1;  GBC.gridy = 2+dir;
      GBL.setConstraints (L2A, GBC);
      add (L2A);
      Label L2B = new Label ("Blocco sulla Direzione '"+MessSer.NOME_DIR[dir]+"'?");
      GBC.gridx = 1;  GBC.gridy = 6+dir;
      GBL.setConstraints (L2B, GBC);
      add (L2B);
    }
    Label LRT = new Label ("Numero di Transazioni Attive:");
    GBC.gridx = 1;  GBC.gridy = 10;
    GBL.setConstraints (LRT, GBC);
    add (LRT);

    GBC.anchor = GridBagConstraints.WEST;
    for (byte dir=MessSer.AVANTI; dir<=MessSer.INDIETRO; dir++) {
      GBC.gridx = 2;  GBC.gridy = 2+dir;
      GBL.setConstraints (AreaIndSer[dir], GBC);
      add(AreaIndSer[dir]);
      GBC.gridx = 2;  GBC.gridy = 6+dir;
      GBL.setConstraints (AreaBlocco[dir], GBC);
      add(AreaBlocco[dir]);
    }
    GBC.gridx = 2;  GBC.gridy = 10;
    GBL.setConstraints (AreaTrnAtt, GBC);
    add(AreaTrnAtt);

    GBC.anchor = GridBagConstraints.WEST;
    Bottone.defFont (FONT_BOTT);
    Bottone BottOK  = new Bottone ("RITRACCIA", COL_SFO_BOTOK, COL_TXT_BOTOK);
    Bottone BottAn  = new Bottone ("HO VISTO",  COL_SFO_BOTAN, COL_TXT_BOTAN);

    BottOK.addActionListener (new ListenSerStatoUpd(MySerAp, this));
    GBC.gridx = 1;  GBC.gridy = 14;
    GBL.setConstraints (BottOK, GBC);
    add (BottOK);

    BottAn.addActionListener (new ListenSerStatoAn(MySerAp, this));
    GBC.gridx = 3;  GBC.gridy = 14;
    GBL.setConstraints (BottAn, GBC);
    add (BottAn);

    ritraccia ();

    riposiziona (100, 100);
  } //CostruisciDisplay


  public void ritraccia () {
    // Riempie le caselle dei dati
    String IndSer;
    for (byte dir=MessSer.AVANTI; dir<=MessSer.INDIETRO; dir++)
      try {
        IndSer = MySerAp.GestTransaz.rendiMacchinaVicina(dir).toString();
        AreaIndSer[dir].setText (IndSer);
        if (MySerAp.GestTransaz.hoTransazioneBloccata(dir))
          AreaBlocco[dir].setText ("SI'");
        else
          AreaBlocco[dir].setText ("NO");
      }
      catch (Exception E) { }
    int NT = MySerAp.GestTransaz.quanteTransAttive();
    AreaTrnAtt.setText (String.valueOf(NT));
  }  //ritraccia

} //ServerDispStato