/**  ServerDisplay - display per il server
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.awt.Frame;
import java.awt.Panel;
import java.awt.Label;
import java.awt.Font;
import java.awt.TextArea;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;


//-------------- I "LISTENER" DEI BOTTONI ---------------------------------------------------------

abstract class ListenSerGenerico {
  // Astrazione di un listener che usa un'applicazione Server
  protected Server SerApp;   // Riferimento al Server da usare

  public ListenSerGenerico (Server SerApp) { 
    this.SerApp = SerApp;
  } //costruttore

} //ListenSerGenerico


class ListenSerTermina extends ListenSerGenerico implements ActionListener {

  public ListenSerTermina (Server SerApp) { 
    super (SerApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    SerApp.eseguiTermina();
  } //actionPerformed

} //ListenSerTermina


class ListenSerConfig extends ListenSerGenerico implements ActionListener {

  public ListenSerConfig (Server SerApp) { 
    super (SerApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    SerApp.eseguiConfig();
  } //actionPerformed

} //ListenSerConfig

class ListenSerConnDis extends ListenSerGenerico implements ActionListener {

  public ListenSerConnDis (Server SerApp) { 
    super (SerApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    Bottone MyBot = (Bottone) evt.getSource();

    if (SerApp.sonoConnesso()) {      // Chiama "conn/disconn"
      int PremutoChe = SerApp.rendiDisplay().apriDialogoModale (Dialogo.BOTT_SINO, 
                       "Vuoi Veramente DISCONNETTERE", "Questo Server dall'Anello?");
      if (PremutoChe == Dialogo.PREMUTO_SI)
        try {
          SerApp.disconnessione();        	
        }
        catch (Exception E) { } //nulla
    }
    else
      SerApp.connessione();

    if (SerApp.sonoConnesso()) {        // Modifica scritta bottone
      MyBot.setLabel(ServerDisplay.TestoDisc);
      MyBot.setBackground (ServerDisplay.COL_SFO_BCONND);
    }
    else {
      MyBot.setLabel(ServerDisplay.TestoConn);
      MyBot.setBackground (ServerDisplay.COL_SFO_BCONNC);
    }

  } //actionPerformed

} //ListenSerConnDis


class ListenSerAggSpec extends ListenSerGenerico implements ActionListener {

  public ListenSerAggSpec (Server SerApp) { 
    super (SerApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    SerApp.eseguiAggSpettacolo();
  } //actionPerformed

} //ListenSerAggSpec


class ListenSerCanSpec extends ListenSerGenerico implements ActionListener {

  public ListenSerCanSpec (Server SerApp) { 
    super (SerApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    SerApp.eseguiCanSpettacolo();
  } //actionPerformed

} //ListenSerCanSpec


class ListenSerVediDB extends ListenSerGenerico implements ActionListener {

  public ListenSerVediDB (Server SerApp) { 
    super (SerApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    SerApp.eseguiVediDB();
  } //actionPerformed

} //ListenSerVediDB


class ListenSerLoadSaveDB extends ListenSerGenerico implements ActionListener {

  protected boolean Load;

  public ListenSerLoadSaveDB (Server SerApp, boolean Load) { 
    super (SerApp);
    this.Load = Load;
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    if (Load) 
      SerApp.eseguiLoadDB();
    else
      SerApp.eseguiSaveDB();
  } //actionPerformed

} //ListenSerLoadSaveDB


class ListenSerMostraDB extends ListenSerGenerico implements ActionListener {

  public ListenSerMostraDB (Server SerApp) { 
    super (SerApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    if (! SerApp.ConnessoONoMaContinua()) return;    // Controllo con eventuale richiesta
    Finestra LaFin = new Finestra ("CONTENUTO DEL DATA-BASE");

    String DBStr = SerApp.IDataBase.toString();

    TextArea Testo = new TextArea (DBStr, 25, 80);
    Testo.setBackground (Color.black);
    Testo.setForeground (Color.white);
    Testo.setFont (new Font("Courier", Font.PLAIN, 14));
    Testo.setEditable (false);
    LaFin.add (Testo);

    LaFin.pack();
    LaFin.setVisible (true);  // mostra la finestra
    // LaFin.toFront();
  } //actionPerformed

} //ListenSerMostraDB


class ListenSerStato extends ListenSerGenerico implements ActionListener {

  public ListenSerStato (Server SerApp) { 
    super (SerApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    SerApp.eseguiStato();
  } //actionPerformed

} //ListenSerStato


class ListenSerInfo extends ListenSerGenerico implements ActionListener {

  public ListenSerInfo (Server SerApp) { 
    super (SerApp);
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    SerApp.eseguiInfo();
  } //actionPerformed

} //ListenSerInfo


class ListenSerPassa extends ListenSerGenerico implements ActionListener {

  protected boolean InviaPopUp;

  public ListenSerPassa (Server SerApp, boolean InviaPopUp) { 
    super (SerApp);
    this.InviaPopUp = InviaPopUp;
  } //costruttore

  public void actionPerformed (ActionEvent evt) {
    SerApp.eseguiPassa (InviaPopUp);
  } //actionPerformed

} //ListenSerPassa



//============== IL DISPLAY DELL'APPLICAZIONE "Server" ============================================

class ServerDisplay extends Display {
  /** Il display del programma (applicazione) Server
   */

  // Costanti:
  public    static final Color COL_SFO_BCONNC= Color.green.brighter();
  public    static final Color COL_SFO_BCONND= Color.green.darker();
  protected static final Color COL_TXT_BCONN = Color.blue;
  protected static final Color COL_SFO_BPASS = Color.green.brighter();
  protected static final Color COL_TXT_BPASS = Color.blue;

  protected static final Color COL_SFO_BSPET = Color.cyan;
  protected static final Color COL_TXT_BSPET = Color.red;

  protected static final Color COL_SFO_BDBMO = Color.white;
  protected static final Color COL_SFO_BDBLS = Color.lightGray;
  protected static final Color COL_TXT_BDB   = Color.blue;

  protected static final Color COL_SFO_BSTAT = Color.yellow;
  protected static final Color COL_TXT_BSTAT = Color.black;

  protected static final Color COL_SFO_BEXIT = Color.red;
  protected static final Color COL_TXT_BEXIT = Color.white;

  protected static final Color COL_SFO_BINFO = Color.black;
  protected static final Color COL_TXT_BINFO = Color.green;

  public    static final String TestoConn    = "     CONNETTI IN ANELLO   ";
  public    static final String TestoDisc    = "  DISCONNETTI DA ANELLO   ";

  // Uso il pattern Model-View-Control:
  // Il server usa un display che a sua volta usa il server
  protected Server  MySerAp;


  public ServerDisplay (Server MyServer) {
    this (MyServer, "");
  } //costruttore

  public ServerDisplay (Server MyServer, String TitoloFin) {
    super (TitoloFin);
    MySerAp = MyServer;   // SALVA IL RIFERIMENTO ALLA SUA APPLICAZ.
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore

  public ServerDisplay (Server MyServer, Frame FinGiaCreata) {
    super (FinGiaCreata);
    MySerAp = MyServer;
    CostruisciDisplay ();
    FineCostrDisplay ();
  } //costruttore


  protected void CostruisciDisplay () {
    super.CostruisciDisplay ();

    GridBagLayout      GBL = new GridBagLayout (); 
    GridBagConstraints GBC = new GridBagConstraints ();
    setLayout (GBL);

    Label L1 = new Label ("PANNELLO DI CONTROLLO");
    L1.setFont (FONT_TEXT0);

    Bottone.defFont (FONT_BOTT);

    GridBagLayout      GBL2 = new GridBagLayout (); 
    GridBagConstraints GBC2 = new GridBagConstraints ();
    Panel PanConn = new Panel (GBL2);
    PanConn.setBackground (COL_SFO_BCONNC.darker());
    PanConn.setForeground (COL_TXT_BCONN);
    Label LabC = new Label ("Anello: ");
    GBC2.gridx = 1;  GBC2.gridy = 1;
    GBL2.setConstraints (LabC, GBC2);
    PanConn.add (LabC);
    Bottone.defColori (COL_SFO_BCONNC, COL_TXT_BCONN);
    Bottone BottConnDis  = new Bottone (TestoConn);      // CONNETTI/DISCONNETTI
    BottConnDis.addActionListener (new ListenSerConnDis (MySerAp));
    GBC2.gridx = 2;  GBC2.gridy = 1;
    GBL2.setConstraints (BottConnDis, GBC2);
    PanConn.add (BottConnDis);
    Bottone.defColori (COL_SFO_BPASS, COL_TXT_BPASS);
    Bottone BottPassa    = new Bottone (" CONTROLLO ");
    BottPassa.addActionListener (new ListenSerPassa (MySerAp, false));
    GBC2.anchor = GridBagConstraints.WEST;
    GBC2.gridx = 2;  GBC2.gridy = 2;
    GBL2.setConstraints (BottPassa, GBC2);
    PanConn.add (BottPassa);
    Bottone BottPasMes   = new Bottone (" MESSAGGIO ");
    BottPasMes.addActionListener (new ListenSerPassa (MySerAp, true));
    GBC2.anchor = GridBagConstraints.EAST;
    GBC2.gridx = 2;  GBC2.gridy = 2;
    GBL2.setConstraints (BottPasMes, GBC2);
    PanConn.add (BottPasMes);


    Panel PanSpett = new Panel ();
    PanSpett.setBackground (COL_SFO_BSPET.darker());
    PanSpett.setForeground (COL_TXT_BSPET);
    PanSpett.add (new Label ("Spettacolo: "));
    Bottone.defColori (COL_SFO_BSPET, COL_TXT_BSPET);
    Bottone BottAggSpec  = new Bottone ("AGGIUNGI");
    BottAggSpec.addActionListener (new ListenSerAggSpec (MySerAp));
    PanSpett.add (BottAggSpec);
    Bottone BottCanSpec  = new Bottone ("RIMUOVI");
    BottCanSpec.addActionListener (new ListenSerCanSpec (MySerAp));
    PanSpett.add (BottCanSpec);

    Panel PanDBase = new Panel ();
    PanDBase.setBackground (COL_SFO_BDBLS.darker());
    PanDBase.setForeground (COL_TXT_BDB);
    PanDBase.add (new Label ("Data Base: "));
    Bottone.defColori (COL_SFO_BDBMO, COL_TXT_BDB);
    Bottone BottVediDB   = new Bottone (" VEDI ");
    BottVediDB.addActionListener (new ListenSerVediDB (MySerAp));
    PanDBase.add (BottVediDB);
    Bottone BottMostra = new Bottone ("MOSTRA");
    BottMostra.addActionListener (new ListenSerMostraDB (MySerAp));
    PanDBase.add (BottMostra);
    Bottone.defColori (COL_SFO_BDBLS, COL_TXT_BDB);
    Bottone BottLoad     = new Bottone ("CARICA");
    BottLoad.addActionListener (new ListenSerLoadSaveDB (MySerAp, true));
    PanDBase.add (BottLoad);
    Bottone BottSave     = new Bottone ("SALVA");
    BottSave.addActionListener (new ListenSerLoadSaveDB (MySerAp, false));
    PanDBase.add (BottSave);

    Panel PanStato = new Panel ();
    PanStato.setBackground (COL_SFO_BSTAT.darker());
    PanStato.setForeground (COL_TXT_BSTAT);
    PanStato.add (new Label ("Stato Interno: "));
    Bottone.defColori (COL_SFO_BSTAT, COL_TXT_BSTAT);
    Bottone BottStato    = new Bottone ("STATO");
    BottStato.addActionListener (new ListenSerStato (MySerAp));
    PanStato.add (BottStato);
    Bottone BottConfig   = new Bottone ("CONFIGURAZIONE");
    BottConfig.addActionListener (new ListenSerConfig (MySerAp));
    PanStato.add (BottConfig);

    Bottone BottTermina  = new Bottone ("ESCI", COL_SFO_BEXIT, COL_TXT_BEXIT);
    BottTermina.addActionListener (new ListenSerTermina (MySerAp));

    Bottone BottInfo     = new Bottone ("INFO", COL_SFO_BINFO, COL_TXT_BINFO);
    BottInfo.addActionListener (new ListenSerInfo (MySerAp));

    GBC.anchor = GridBagConstraints.CENTER;
    GBC.gridx = 2;  GBC.gridy = 1;         // "Pannello di Controllo"
    GBL.setConstraints (L1, GBC);
    add (L1);

    GBC.anchor = GridBagConstraints.EAST;
    GBC.gridx = 2;  GBC.gridy = 25;        // Bottone "info"
    GBL.setConstraints (BottInfo, GBC);
    add (BottInfo);

    GBC.anchor = GridBagConstraints.CENTER;
    GBC.fill   = GridBagConstraints.HORIZONTAL;
    GBC.gridx = 2;  GBC.gridy = 2;         // Pannello "conn/disc"
    GBL.setConstraints (PanConn, GBC);
    add (PanConn);

    GBC.gridx = 2;  GBC.gridy = 5;         // Pannello "spettacolo"
    GBL.setConstraints (PanSpett, GBC);
    add (PanSpett);

    GBC.gridx = 2;  GBC.gridy = 10;        // Pannello "data-base"
    GBL.setConstraints (PanDBase, GBC);
    add (PanDBase);

    GBC.gridx = 2;  GBC.gridy = 20;        // pannello "stato"
    GBL.setConstraints (PanStato, GBC);
    add (PanStato);

    GBC.anchor = GridBagConstraints.WEST;
    GBC.fill   = GridBagConstraints.NONE;
    GBC.gridx = 2;  GBC.gridy = 25;        // Bottone "exit"
    GBL.setConstraints (BottTermina, GBC);
    add (BottTermina);

    riposiziona (100, 130);

    Frame MyFrame = (Frame) rendiFinContenente();
    MyFrame.setResizable(false);
    // MyFrame.toFront();

  } //CostruisciDisplay


  public void windowClosing  (WindowEvent evt) {
    MySerAp.eseguiTermina();
  } //windowClosing

} //ServerDisplay