/**  Spettacolo
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.Serializable;
import java.io.IOException;


class Spettacolo implements Dato, Serializable {
  /** Astrazione di uno spettacolo fatto nel teatro
  */

  //Costanti:
  public static final int TURNO_MIN = 1;
  public static final int TURNO_MAX = 3;

  // Variabili d'istanza:
  protected String Nome;
  protected byte   GG, MM;
  protected short  AA;
  protected byte   Turno;

  public Spettacolo (String Nome, int Giorno, int Mese, int Anno, int Turno) {
    this.Nome=Nome;
    GG=(byte) Giorno;
    MM=(byte) Mese;
    AA=(short) Anno;
    this.Turno=(byte) Turno;
  } //costruttore

  public Spettacolo (DataInputStream DIS) throws IOException {
    // rende un posto leggendo i dati da uno stream di byte
    Nome = DIS.readUTF();
    GG   = DIS.readByte();
    MM   = DIS.readByte();
    AA   = DIS.readShort();
    Turno= DIS.readByte();
  } //costruttore

    
  public void scriviInStream (DataOutputStream DOS) throws IOException {
    // scrive su uno stream di dati i dati di uno spettatore
    DOS.writeUTF  (Nome);
    DOS.writeByte  (GG);
    DOS.writeByte  (MM);
    DOS.writeShort (AA);
    DOS.writeByte (Turno);
  } //scriviInStream


  public String rendiNome () {
    return Nome;
  } //rendiNome

  public int rendiGiorno  () {
    return GG;
  } //rendiGiorno

  public int rendiMese    () {
    return MM;
  } //rendiMese

  public int rendiAnno    () {
    return AA;
  } //rendiAnno

  public int rendiTurno   () {
    return Turno;
  } //rendiTurno


  protected boolean AnnoBisestile () {
    // Rende true se l'anno AA e' bisestile
    // Se AA e' divisibile per 4, e' bisestile; ma con le eccezioni:
    if ((AA % 4000) == 0)          // Ogni 4000 anni si salta la bisestilita'
      return false;
    else
      if ((AA % 400) == 0)         // Ogni 400 anni, si mantiene la bisestilita'
        return true;               // (per es. il 2000 sara' bisestile per questo)
      else
        if ((AA % 100) == 0)       // Ogni 100 anni si salta la bisestilita'
          return false;
        else
        return (AA % 4) == 0;      // se Anno e' divisibile per 4, e' bisestile
  } //NumGGdiFebbraio

  
  protected boolean CorrettoGGMM() {
    // rende true se la data GG/MM e' corretta
    if (GG >= 1) 
      if ( (MM == 11) || (MM == 4) || (MM == 6) || (MM == 9))    // 30 gg
        return GG <= 30;
      else
        if ( (MM == 1) || (MM == 3)  || (MM == 5) || (MM == 7) ||
             (MM == 8) || (MM == 10) || (MM == 12))              // 31 gg
          return GG <= 31;
        else
          if (AnnoBisestile())                                   // febbraio
            return GG <= 29;
          else
            return GG <= 28;
    else 
      return false;
  } //CorrettoGGMM()


  public boolean corretto () {
    if (Nome != null) 
      return ( (! Nome.equals("") ) &&
                CorrettoGGMM()      &&
               (AA >= 1990)         &&
               (Turno >= TURNO_MIN) && 
               (Turno <= TURNO_MAX)   );
    else
      return false;
  } //corretto
  
  
  public String toString() {
      // Overriding del metodo definito in "Object" per rappresentazione in stringa
    return new String (Nome+"-"+
                       Integer.toString(GG)+"/"+
                       Integer.toString(MM)+"/"+
                       Integer.toString(AA)+"-"+
                       Integer.toString(Turno));
  } //toString


  public boolean equals (Object obj) {
    // Overriding del metodo definito in "Object" per fare la ricerca nella Hashtable
    Spettacolo S2 = (Spettacolo)obj;
    return (  Nome.equals( S2.Nome ) &&
             (GG      ==   S2.GG)    &&
             (MM      ==   S2.MM)    &&
             (AA      ==   S2.AA)    &&
             (Turno   ==   S2.Turno)   );
  } //equals


  public int hashCode() {
    // Overriding del metodo definito in "Object" per fare la ricerca nella Hashtable
    return toString().hashCode();
  } //hashCode


  public boolean minoreDi (Spettacolo S2) {
    // definita da me, per ordinare gli spettacoli
    if (Nome.equals(S2.Nome))   // nomi uguali, confronta le date
      if (AA == S2.AA) 
        if (MM == S2.MM)
          if (GG == S2.GG)
            return Turno < S2.Turno;
          else
           return GG < S2.GG;
        else
          return MM < S2.MM;
      else
        return AA < S2.AA;
    else // Nomi diversi: confronta solo i nomi
      return Nome.compareTo(S2.Nome) < 0;   // Nome e' < di S2.Nome ?
  } //minoreDi

} //Spettacolo