/**  TraslaMess 
 *     @autore     Luigi Antenucci
 *     @versione   2.0
 *     @linguaggio jdk 1.2
 */

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;


/** GESTORE TRASFERIMENTO MESSAGGI IN DataI/OStream E VICEVERSA --------- */

class TraslaMess {
  /** Converte un messaggio dalla rappresentazione in oggetto a quella
   *  sottoforma di array di byte (byte[]).
   *  Per l'esattezza, si usa un Data-I/O-Stream anziche' un byte[]
   */

  // Costanti (private!) per identificare ogni messaggio su data-stream
  // Occorre definirne una per ogni classe di messaggio scrivibile
  protected final static byte   MESS_NULLO   =  0;   // MESS. nullo

  protected final static byte   MESS_C_CANC  = 30;   // MESS. CLI<==>SER
  protected final static byte   MESS_C_RICH  = 31;
  protected final static byte   MESS_C_INFO  = 33;
  protected final static byte   MESS_C_DSPC  = 41;
  protected final static byte   MESS_C_DPL   = 42;
  protected final static byte   MESS_C_DPOM  = 43;
  protected final static byte   MESS_C_D2SE  = 45;

  protected final static byte   MESS_SD_CANC = 50;   // MESS. SER<==>SER DATAGRAMMI
  protected final static byte   MESS_SD_PREN = 51;
  protected final static byte   MESS_SD_TOSC = 60;
  protected final static byte   MESS_SD_AGSC = 61;

  protected final static byte   MESS_SS_FATT =  5;   // MESS. SER<==>SER STREAM
  protected final static byte   MESS_SS_CAMB =  8;
  protected final static byte   MESS_SS_SBLO = 80;
  protected final static byte   MESS_SS_BLOC = 81;
  protected final static byte   MESS_SS_TOME = 90;
  protected final static byte   MESS_SS_AGME = 91;
  protected final static byte   MESS_SS_RIAG = 101;
  protected final static byte   MESS_SS_RICM = 102;
  protected final static byte   MESS_SS_PASS = 111;


  // Messaggi di errore (lanciati con le eccezioni)
  protected final static String STRERR_NONCE = "Non Esiste una Classe per il Messaggio!";


  public static Mess creaMess (DataInputStream DIS) throws IOException {
    // dato uno stream di dati contenente un messaggio, rende il messaggio
    // rappresentato sottoforma di oggetto (uso del late-binding: rendo un "Mess")
    byte Cod = DIS.readByte();
    //   Debug.outln ("TraslaMes- conversione byte[] --> messaggio, Cod="+Cod);
    // RICORDA: PRIMA LE SOTTOCLASSI, POI LA LORO SUPERCLASSE, E COSI' VIA
    if (Cod == MESS_C_CANC)  return new MessCliCancella   (DIS);
    if (Cod == MESS_C_RICH)  return new MessCliRichiedi   (DIS);
    if (Cod == MESS_C_INFO)  return new MessCliInfo       (DIS);
    if (Cod == MESS_C_DSPC)  return new MessCliDammiSc    (DIS);
    if (Cod == MESS_C_DPL )  return new MessCliDammiPL    (DIS);
    if (Cod == MESS_C_DPOM)  return new MessCliDammiPOMe  (DIS);
    if (Cod == MESS_C_D2SE)  return new MessCliDammi2Ser  (DIS);

    if (Cod == MESS_SD_CANC) return new MessSerDgCancella (DIS);
    if (Cod == MESS_SD_PREN) return new MessSerDgPrenota  (DIS);
    if (Cod == MESS_SD_TOSC) return new MessSerDgCanSc    (DIS);
    if (Cod == MESS_SD_AGSC) return new MessSerDgAggSc    (DIS);

    if (Cod == MESS_SS_FATT) return new MessSerFatto      (DIS);
    if (Cod == MESS_SS_CAMB) return new MessSerCambSer    (DIS);
    if (Cod == MESS_SS_SBLO) return new MessSerSblocca    (DIS);
    if (Cod == MESS_SS_BLOC) return new MessSerBlocca     (DIS);
    if (Cod == MESS_SS_TOME) return new MessSerTogMe      (DIS);
    if (Cod == MESS_SS_AGME) return new MessSerAggMe      (DIS);
    if (Cod == MESS_SS_RIAG) return new MessSerRiaggancio (DIS);
    if (Cod == MESS_SS_RICM) return new MessSerRiaggConMe (DIS);
    if (Cod == MESS_SS_PASS) return new MessSerPassami    (DIS);

    //se non riesce,
    throw new IOException (STRERR_NONCE);
  } //creaMess


  public static void scriviMess (Mess MyMess, DataOutputStream DOS) throws IOException {
    // dato un messaggio, lo scrive sottoforma di stream di dati in uscita
    // (si usa il late-binding per riferire in MyMess qualsiasi messaggio)
    // NB: prima si controlla se e' istanza di una cottoclasse e poi della classe 
    //     ad es, prima vedo se e' un "cancella" e se no, vedo se e' un "prenota"
    byte Codice = MESS_NULLO;
    if (MyMess instanceof MessCliCancella)     Codice = MESS_C_CANC;
    else
      if (MyMess instanceof MessCliRichiedi)   Codice = MESS_C_RICH;
    if (MyMess instanceof MessCliInfo)         Codice = MESS_C_INFO;
    if (MyMess instanceof MessCliDammiSc)      Codice = MESS_C_DSPC;
    if (MyMess instanceof MessCliDammiPL)      Codice = MESS_C_DPL;
    if (MyMess instanceof MessCliDammiPOMe)    Codice = MESS_C_DPOM;
    if (MyMess instanceof MessCliDammi2Ser)    Codice = MESS_C_D2SE;

    if (Codice == MESS_NULLO) {  // Per non avere errore nel Client se tolgo le classi MessSer
      if (MyMess instanceof MessSerDgCancella)   Codice = MESS_SD_CANC;
      else
       if (MyMess instanceof MessSerDgPrenota)   Codice = MESS_SD_PREN;
      if (MyMess instanceof MessSerDgCanSc)      Codice = MESS_SD_TOSC;
      else
        if (MyMess instanceof MessSerDgAggSc)    Codice = MESS_SD_AGSC;

      if (MyMess instanceof MessSerFatto)        Codice = MESS_SS_FATT;
      if (MyMess instanceof MessSerCambSer)      Codice = MESS_SS_CAMB;    
      if (MyMess instanceof MessSerSblocca)      Codice = MESS_SS_SBLO;
      else
        if (MyMess instanceof MessSerBlocca)     Codice = MESS_SS_BLOC;

      if (MyMess instanceof MessSerTogMe)        Codice = MESS_SS_TOME;
      else
        if (MyMess instanceof MessSerAggMe)      Codice = MESS_SS_AGME;

      if (MyMess instanceof MessSerRiaggancio)   Codice = MESS_SS_RIAG;
      else
        if (MyMess instanceof MessSerRiaggConMe) Codice = MESS_SS_RICM;
        else
          if (MyMess instanceof MessSerPassami)  Codice = MESS_SS_PASS;
    }

    if (Codice != MESS_NULLO) {
      //   Debug.outln ("TraslaMes- conversione messaggio --> byte[], Cod="+Codice);
      DOS.writeByte (Codice);
      MyMess.scriviInStream (DOS);        // uso il late-binding
      DOS.flush();
    }
    else {
      throw new IOException (STRERR_NONCE);
    }
  } //scriviMess

} // TraslaMess